<?php
namespace Teknisa\FIN\Controllers;

use Doctrine\ORM\EntityManager;
use Teknisa\FIN\Controllers\TeknisaCrud;
use Teknisa\Libs\Service\CRUD as CrudService;
use Teknisa\Libs\Util\Environment;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Request\Row as Row;
use Zeedhi\Framework\DTO\Request\DataSet as Data;
use Zeedhi\Framework\DTO\Response;
use Teknisa\FIN\Services\ArquivosService;

/**
 * Class ArquivosController
 * @package Teknisa\FIN\Controllers
 */
class ArquivosController extends TeknisaCrud{
    
    /**
     * @param CrudService $crudService
     * @param ParameterBag $parameterBag
     * @param Environment $environment
     * @param ArquivosService $arquivosService
     */
    public function __construct(CrudService $crudService, ParameterBag $parameterBag, Environment $environment, ArquivosService $arquivosService){
        $this->crudService  = $crudService;
        $this->parameterBag = $parameterBag;
        $this->environment  = $environment;
        $this->arquivosService  = $arquivosService;
    }

    /**
     * Download Arquivo
     * @param Data $dataset
     * @param Response $response
     */
    public function downloadArquivo(Data $dataset, Response $response)
    {
        try {
            // Dados do Registro
            $arquivo  = $dataset->getDataSet()->getRows();

            // Arquivo para Download[Caminho + Nome + Extenção]
            $arquivo = $arquivo[0]['DSARQUIVO'];
            
            // Validação para nao permitir alteração de pasta do arquivo servidor
            if ((strpos($arquivo, '../') !== false) || (strpos($arquivo, 'tmp/') === false))
                die("Acesso não permitido, contate o administrador do sistema.");
                
            

            // Download Arquivo
            $this->arquivosService->downloadArquivo($arquivo);

        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function downloadArquivoAnexo(Row $dataset, Response $response) {
        try {
            // Dados do Registro
            $row = $dataset->getRow();

            // Arquivo para Download [Nome + Extensão]
            $arquivo = $row['DSARQUIVO'];
            
            // Download Arquivo
            $this->arquivosService->downloadArquivoAnexo($arquivo);

        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function downloadArquivoAnexoFileServer(Row $dataset, Response $response) {
        try {
            // Dados do Registro
            $row = $dataset->getRow();

            // Arquivo para Download [Nome + Extensão]
            $fileName = $row['FILENAME'];
            
            // Path do Arquivo no Servidor File Server
            $path = $row['PATH'];
            
            // Url do Servidor File Server
            $serviceUrl = $this->arquivosService->getFileServerUrl();

            // Url do Servidor File Server + Path do Arquivo no Servidor File Server
            $arquivo = $serviceUrl.$path;
            
            
            // Download Arquivo
            $this->arquivosService->downloadArquivoAnexoFileServer($arquivo, $fileName);

        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    
      /**
     * Download Arquivo
     * @param Data $dataset
     * @param Response $response
     */
    public function salvaArquivoParaDownload(Data $dataset, Response $response)
    {
        try {
            // Dados do Registro
            $arquivo  = $dataset->getDataSet()->getRows();

            // Arquivo para Download[Caminho + Nome + Extenção]
            $arquivo = $arquivo[0];

            $htmlFile = fopen(sys_get_temp_dir() . DIRECTORY_SEPARATOR . $arquivo['ARQUIVO'] . $arquivo['NUMERO'] .'.txt', "w");
            fwrite($htmlFile,  $arquivo['CONTEUDO']);
            fclose($htmlFile);

        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
}