<?php

namespace Teknisa\FIN\Modules\ContaPagar\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\Model\Financeiro\Entities\Adiantapagar;
use Teknisa\Model\Financeiro\Entities\Devolucaopag;
use Teknisa\Model\Fiscal\Entities\Notafiscal;
use Teknisa\Model\Financeiro\Entities\Ajbolpag;
use Teknisa\Model\Financeiro\Entities\Movfluxo;
use Teknisa\Model\Financeiro\Entities\Boletpagar;
use Teknisa\Model\Financeiro\Entities\Contapagar;
use Teknisa\FIN\Modules\ContaPagar\Factory\AjusteTituloFactory;
use Teknisa\FIN\Modules\FluxoCaixa\Services\FluxoDeCaixaService;

/**
 * Class AjusteTituloService
 * @package Teknisa\FIN\Modules\ContaPagar\Services
 */
class AjusteTituloService {

    protected $entityManager;
    protected $ajusteTituloFactory;

    /**
     * @param EntityManager $entityManager
     * @param AjusteTituloFactory $ajusteTituloFactory
     * @param FluxoDeCaixaService $fluxoDeCaixaService
     */
    public function __construct(EntityManager $entityManager, AjusteTituloFactory $ajusteTituloFactory, FluxoDeCaixaService $fluxoDeCaixaService)
    {
        $this->entityManager = $entityManager;
        $this->ajusteTituloFactory = $ajusteTituloFactory;
        $this->fluxoDeCaixaService = $fluxoDeCaixaService;
    }

    /**
     * Salva Ajuste Título a Pagar
     * @param $nrorg
     * @throws \Exception
     */
    public function salvaAjusteBoleto(
        $nrorg, $cdfilial, $nrlanbolpag, $idajuctpag, $cdajuctpag, $vrajbolpag, $idorigajbpag = '2', $idtpajbolpag = '1'
    ){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {

            // Salva Ajuste Boleto Pagar
            self::salvaAjusteBoletoPag(
                $nrorg, $cdfilial, $nrlanbolpag, $idajuctpag, $cdajuctpag, $vrajbolpag, $idorigajbpag, $idtpajbolpag
            );
            
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * Salva Ajuste Título a Pagar
     * @param $nrorg
     * @throws \Exception
     */
    public function salvaAjusteBoletoPag(
        $nrorg, $cdfilial, $nrlanbolpag, $idajuctpag, $cdajuctpag, $vrajbolpag, $idorigajbpag = '2', $idtpajbolpag = '1'
    ){
        try {
            // Busca Tipo de Ajuste a Pagar
            $tipoajpagar = $this->ajusteTituloFactory->buscaTipoajpagar($nrorg, $cdajuctpag);
            $idajuctpag = $tipoajpagar->getIdajuctpag();
            
            // Valida Codigo Ajuste Existente
            self::validaCodigoAjusteExistente($nrorg, $cdfilial, $nrlanbolpag, $cdajuctpag);

            // Valida Status do Boleto
            self::validaStatusBoleto($nrorg, $cdfilial, $nrlanbolpag);

            // Insere Ajuste Boleto
            self::inserirAjusteBoleto($nrorg, $cdfilial, $nrlanbolpag, $cdajuctpag, $vrajbolpag, $idorigajbpag, $idtpajbolpag);

            // Atualiza Valor Boleto
            self::atualizaValorBoleto($nrorg, $cdfilial, $nrlanbolpag, $idajuctpag, $vrajbolpag);

            // Removido - Ajuste do Boleto não movimenta fluxo por causa da Associação
            // Atualiza Fluxo de Caixa
            //self::atualizaFluxoCaixa($nrorg, $cdfilial, $nrlanbolpag, $idajuctpag, $vrajbolpag);

            // Recalcular Multa e Mora

            // Altera Tipo de Recebimento

            // Contabiliza na BAIXA

            $this->entityManager->flush();
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * Salva Ajuste Título a Pagar
     * @param $nrorg
     * @throws \Exception
     */
    public function insereAjusteAdiantDevolFornecedor(
        $nrorg, $cdfilial, $nrlanbolpag, $idajuctpag, $cdajuctpag, $vrajbolpag,  $idorigajbpag = '2', $idtpajbolpag = '1'
    ){
        try {

            // Busca Ajuste Titulo
            /** @var Ajbolpag $ajbolpag */
            $ajbolpag = $this->ajusteTituloFactory->buscaAjbolpag($nrorg, $cdfilial, $nrlanbolpag, $cdajuctpag);

            // Verifica Existência Ajuste Titulo
            if(!$ajbolpag) {
                // Insere Ajuste Titulo
                self::inserirAjusteBoleto($nrorg, $cdfilial, $nrlanbolpag, $cdajuctpag, $vrajbolpag, $idorigajbpag, $idtpajbolpag);
            }else{
                // Atualiza Ajuste Titulo
                self::atualizaAjusteBoleto($ajbolpag, $vrajbolpag);
            }

            // Atualiza Valor Boleto
            self::atualizaValorBoleto($nrorg, $cdfilial, $nrlanbolpag, $idajuctpag, $vrajbolpag);

            // Removido - Ajuste do Boleto não movimenta fluxo por causa da Associação
            // Atualiza Fluxo de Caixa
            //self::atualizaFluxoCaixa($nrorg, $cdfilial, $nrlanbolpag, $idajuctpag, $vrajbolpag);

            // Recalcular Multa e Mora

            // Altera Tipo de Recebimento

            // Contabiliza na BAIXA

        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Deleta Ajuste Título a Pagar
     * @param $nrorg
     * @throws \Exception
     */
    public function deletaAjusteBoleto(
        $nrorg, $cdfilial, $nrlanbolpag, $idajuctpag, $cdajuctpag, $vrajbolpag, $valida = true
    ){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            self::deletaAjusteBol($nrorg, $cdfilial, $nrlanbolpag, $idajuctpag, $cdajuctpag, $vrajbolpag, $valida);
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
    
    /**
     * Deleta Ajuste Título a Pagar
     * @param $nrorg
     * @throws \Exception
     */
    public function deletaAjusteBol(
        $nrorg, $cdfilial, $nrlanbolpag, $idajuctpag, $cdajuctpag, $vrajbolpag, $valida = true
    ){
        try {

            // Valida Status do Boleto
            self::validaStatusBoleto($nrorg, $cdfilial, $nrlanbolpag);

            // Insere Ajuste Boleto
            self::deletaAjusteBoletoPagar($nrorg, $cdfilial, $nrlanbolpag, $cdajuctpag);

            // Inverte Tipo de Ajuste p/ Exclusão
            $idajuctpag = ($idajuctpag == 'D')?'A':'D';

            // Atualiza Valor Boleto
            self::atualizaValorBoleto($nrorg, $cdfilial, $nrlanbolpag, $idajuctpag, $vrajbolpag, $valida);

            $this->entityManager->flush();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Valida Codigo Ajuste Existente
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolpag
     * @param $cdfiliajbolp
     * @param $cdajuctpag
     * @throws \Exception
     */
    public function deletaAdiantamentoPagar(
        $nrorg, $cdfilial, $nrlanbolpag, $cdfiliajbolp, $cdajuctpag
    ){
        try {

            /** @var Ajbolpag $ajbolpag */
            $adiantapagar = $this->ajusteTituloFactory->buscaAdiantapagar(
                $nrorg, $cdfilial, $nrlanbolpag, $cdfiliajbolp, $cdajuctpag
            );

            if ($adiantapagar instanceof Adiantapagar)
                $this->entityManager->remove($adiantapagar);

        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Valida Codigo Ajuste Existente
     * @param $nrorg
     * @param $cdfilibolpag
     * @param $nrlanbolpag
     * @param $cdajuctpag
     * @throws \Exception
     */
    public function deletaDevolucaoPagar(
        $nrorg, $cdfilibolpag, $nrlanbolpag, $cdajuctpag
    ){
        try {

            /** @var Devolucaopag $devolucaopag */
            $devolucaopag = $this->ajusteTituloFactory->buscaDevolucaopag(
                $nrorg, $cdfilibolpag, $nrlanbolpag, $cdajuctpag
            );

            if ($devolucaopag instanceof Devolucaopag)
                $this->entityManager->remove($devolucaopag);

        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Valida Codigo Ajuste Existente
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolpag
     * @param $cdajuctpag
     * @throws \Exception
     */
    public function validaCodigoAjusteExistente(
        $nrorg, $cdfilial, $nrlanbolpag, $cdajuctpag
    ){
        try {
            /** @var Ajbolpag $ajbolpag */
            $ajbolpag = $this->ajusteTituloFactory->buscaAjbolpag($nrorg, $cdfilial, $nrlanbolpag, $cdajuctpag);

            if ($ajbolpag instanceof Ajbolpag)
                throw new \Exception('Operação bloqueada. Ajuste já existente para esse título.');
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Valida Status do Boleto
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolpag
     * @throws \Exception
     */
    public function validaStatusBoleto(
        $nrorg, $cdfilial, $nrlanbolpag
    ){
        try {
            /** @var Boletpagar $boletpagar */
            $boletpagar = $this->ajusteTituloFactory->buscaBoletpagar($nrorg, $cdfilial, $nrlanbolpag);

            if ($boletpagar->getIdstatbpag() == 'D')
                throw new \Exception('Operação bloqueada. Título descontado não pode ser alterado.');
            elseif ($boletpagar->getIdstatbpag() == 'B')
                throw new \Exception('Operação bloqueada. Título baixado não pode ser alterado.');
            elseif ($boletpagar->getNrcnabpagrem())
                throw new \Exception('Operação bloqueada. Título enviado via CNAB não pode ser alterado, apenas prorrogado se possuir permissão.');
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Insere Ajuste Boleto
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolpag
     * @param $cdajuctpag
     * @param $vrajbolpag
     * @return null|Ajbolpag
     * @throws \Exception
     */
    public function inserirAjusteBoleto($nrorg, $cdfilial, $nrlanbolpag, $cdajuctpag, $vrajbolpag, $idorigajbpag = '2', $idtpajbolpag = '1'){
        try {

            /** @var Ajbolpag $ajbolpag */
            $ajbolpag = $this->ajusteTituloFactory->inserirAjbolpag(
                $nrorg, $cdfilial, $nrlanbolpag, $cdajuctpag, $vrajbolpag, $idorigajbpag, $idtpajbolpag
            );

            $this->entityManager->persist($ajbolpag);

            return $ajbolpag;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Atualiza Ajuste Boleto
     * @param $ajbolpag
     * @param $vrajbolpag
     * @throws \Exception
     */
    public function atualizaAjusteBoleto(Ajbolpag $ajbolpag, $vrajbolpag){
        try {
            // Atualiza Valor Titulo A Pagar
            $vrajbolpag += $ajbolpag->getVrajbolpag();

            // Busca Ajuste Titulo A Pagar
            /** @var Ajbolpag $ajbolpag */
            $ajbolpag = $this->ajusteTituloFactory->atualizaAjbolpag(
                $ajbolpag, $vrajbolpag
            );

            $this->entityManager->persist($ajbolpag);

        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Deleta Ajuste Boleto
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolpag
     * @param $cdajuctpag
     * @return Ajbolpag
     * @throws \Exception
     */
    public function deletaAjusteBoletoPagar($nrorg, $cdfilial, $nrlanbolpag, $cdajuctpag){
        try {

            /** @var Ajbolpag $ajbolpag */
            $ajbolpag = $this->ajusteTituloFactory->buscaAjbolpag($nrorg, $cdfilial, $nrlanbolpag, $cdajuctpag);

            // Deleta Adiantamento Pagar
            self::deletaAdiantamentoPagar(
                $nrorg, $cdfilial, $nrlanbolpag, $ajbolpag->getCdfilial(), $ajbolpag->getCdajuctpag()
            );

            // Deleta Devolução Pagar
            self::deletaDevolucaoPagar($nrorg, $cdfilial, $nrlanbolpag, $ajbolpag->getCdajuctpag());

            if ($ajbolpag instanceof Ajbolpag)
                $this->entityManager->remove($ajbolpag);

        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Atualiza Valor Boleto
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolpag
     * @param $idajuctpag
     * @param $vrajbolpag
     * @throws \Exception
     */
    public function atualizaValorBoleto($nrorg, $cdfilial, $nrlanbolpag, $idajuctpag, $vrajbolpag, $valida = true){
        try {

            /** @var Boletpagar $boletpagar */
            $boletpagar = $this->ajusteTituloFactory->buscaBoletpagar($nrorg, $cdfilial, $nrlanbolpag);

            // Verifica Tipo de Ajuste[Acrescer|Decrescer]
            $vrajbolpag = ($idajuctpag == 'A')?$vrajbolpag:-($vrajbolpag);

            // Calcula Valor Boleto
            $vrfinalbpag = $boletpagar->getVrfinalbpag() + $vrajbolpag;

            // Verifica Valor Negativo Boleto
            if(($valida) && ($vrfinalbpag < 0))
                throw new \Exception('Operação bloqueada. Valor do ajuste deve ser menor que valor do boleto.');

            /** @var Boletpagar $boletpag */
            $boletpag = $this->ajusteTituloFactory->atualizaBoletpagar($boletpagar, $vrfinalbpag);
            $this->entityManager->persist($boletpag);

        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Atualiza Valor Boleto
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolpag
     * @param $idajuctpag
     * @param $vrajbolpag
     * @throws \Exception
     */
    public function atualizaFluxoCaixa($nrorg, $cdfilial, $nrlanbolpag, $idajuctpag, $vrajbolpag){
        try {

            /** @var Contapagar $contapagar */
            $contapagar = $this->ajusteTituloFactory->buscaContapagar($nrorg, $cdfilial, $nrlanbolpag);

            /** @var Notafiscal $notafiscal */
            $notafiscal = $this->ajusteTituloFactory->buscaNotafiscal($contapagar->getNrorg(), $contapagar->getCdfilial(), $contapagar->getNrlanctonf());

            // Verifica Tipo de Movimentação de Fluxo
            $idmovfluxo = ($notafiscal->getIdstatus() == 'V')?'P':'E';

            // Verifica Tipo de Ajuste[Acrescer|Decrescer]
            $vrajbolpag = ($idajuctpag == 'A')?$vrajbolpag:-($vrajbolpag);

            // Atualiza Fluxo de Caixa
            /** @var Movfluxo $movfluxo */
            $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                $nrorg, $contapagar->getCdempresa(), $contapagar->getCdfimovflpag(), $contapagar->getCdclassfina(), $contapagar->getDtatuavenpag(), $vrajbolpag, $idmovfluxo, 'S'
            );
            $this->entityManager->persist($movfluxo);

        } catch (\Exception $e) {
            throw $e;
        }
    }

}