<?php

namespace Teknisa\FIN\Modules\ContaPagar\Services;

use Teknisa\FIN\Modules\ContaPagar\Factory\Pag16000EmprestimoLeasingPagarFactory;
use Doctrine\ORM\EntityManager;
use \Teknisa\FIN\Modules\FluxoCaixa\Services\FluxoDeCaixaService;
use \Teknisa\FIN\Modules\ContaPagar\Services\RateioContaService;
use \Teknisa\FIN\Modules\ContaPagar\Services\ControleTituloAlteradoService;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\Libs\Util\Environment;
use Teknisa\FIN\Services\ContabilizacaoService;
use Teknisa\Libs\Util\Masks\Masks;

class Pag16000EmprestimoLeasingPagarService
{
    protected $entityManager;
    protected $factory;
    protected $rateioContaService;
    protected $contabilizacaoService;
    protected $newCode;
    protected $environment;
    const CREDITO = 'C';
    const DEBITO = 'D';
    const SAIDA = 'S';
    const ENTRADA = 'S';
    const ABERTO = 'A';
    const TIPOCONTA_CIRCULANTE = 'C';
    const TIPO_MOV_REAL = 'R';
    const TIPO_MOV_EMPENHO = 'E';
    const STATUS_PAG_ABERTO = 'A';

    public function __construct(EntityManager $entityManager, Environment $environment, Pag16000EmprestimoLeasingPagarFactory $factory, FluxoDeCaixaService $fluxoDeCaixaService,
                                RateioContaService $rateioContaService, $newCode, ContabilizacaoService $contabilizacaoService, ControleTituloAlteradoService $controleTituloAlteradoService, Masks $masks)
    {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->factory = $factory;
        $this->fluxoDeCaixaService = $fluxoDeCaixaService;
        $this->rateioContaService = $rateioContaService;
        $this->contabilizacaoService = $contabilizacaoService;
        $this->controleTituloAlteradoService = $controleTituloAlteradoService;
        $this->newCode = $newCode;
        $this->cdfiliproc = $this->environment->getCdFilial();
        $this->masks = $masks;
        $this->nrorg = $this->environment->getNrOrgTrab();
    }

    public function verificaNotaCadastrada($rowEmprestimo){
        if ($this->factory->verificaNotaCadastrada(
                $rowEmprestimo['CDFORNECED'],  $rowEmprestimo['CDSERIEENTR'],
                $rowEmprestimo['NRNOTAFISC'], $this->nrorg
            )
        ){
            throw new \Exception("Número de nota fiscal e série já cadastrados para o fornecedor informado.");
        }
    }

    public function verificaDocumento($emprestimo){
        $documento = $this->factory->verificaDocumento(
            $this->nrorg,               $emprestimo['CDFORNECED'],
            $emprestimo['CDSERIEENTR'], $emprestimo['NRNOTAFISC']
        );
        if ($documento){
            throw new \Exception("Documento já cadastrado.");
        }
    }

    public function geraSequenciaisEmprestimo($rowEmprestimo){
        $this->nrlanbolpag = $this->newCode->getCodeByTableName('BOLETPAGAR'.$this->cdfiliproc,$this->nrorg);
        $this->nrlanctonf = $this->newCode->getCodeByTableName('LANCNF/FILIAL'.$this->cdfiliproc,$this->nrorg);
        $sequencialLancamento = 'LANCCONTABIL'.$rowEmprestimo['CDEMPRESA'].date('YYYY/MM');
        $this->nrlanccont = $this->newCode->getCodeByTableName($sequencialLancamento ,$this->nrorg);
        $this->nrsequencialemp = $this->factory->getNovoCodigoEmprestimo($this->nrorg, $this->cdfiliproc);
    }

    public function geraSequenciaisCreditoBancario($rowCredito){
        $this->nrmovbanc = $this->newCode->getCodeByTableName('MOVBANCARIO'.$rowCredito['CDREDUCONTCRED'],$this->nrorg);
        $this->nrmovicred = $this->newCode->getCodeByTableName('MOVCREDBANC'.$rowCredito['CDREDUCONTCRED'],$this->nrorg);
    }

    public function geraSequenciaisDebitoBancario($rowCredito){
        $this->nrmovbanc = $this->newCode->getCodeByTableName('MOVBANCARIO'.$rowCredito['CDREDUCONT'],$this->nrorg);
        $this->nrmovidebi = $this->newCode->getCodeByTableName('MOVDEBBANC'.$rowCredito['CDREDUCONT'],$this->nrorg);
    }

    public function saveEmprestimoPagar($row, $dataSetParcelas, $rowCredito, $response)
    {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        $this->validaVinculosEmprestimo($row);
        $this->geraSequenciaisEmprestimo($row);
        $fornecedor = $this->factory->getFornecedor($this->nrorg, $row['CDFORNECED']);
        $rowEmprestimo = $this->constroiRowEmprestimoLeasing($row);

        try {
            $notafiscal = $this->factory->insereNotaFiscalPagar($rowEmprestimo);
            $this->entityManager->persist($notafiscal);

            $empleasing = $this->factory->insereEmprestimoLeasing($rowEmprestimo);
            $this->entityManager->persist($empleasing);
            $this->entityManager->flush();

            $this->insereParcelas($rowEmprestimo, $dataSetParcelas);
            $this->executaContabilizacao($rowEmprestimo);
            if ($rowCredito['CREDITO'] == 'S'){
                $this->geraLancamentoCredito($rowEmprestimo, $rowCredito);
            }
            if ($rowCredito['DEBIOF'] == 'S'){
                $this->geraLancamentoDebito($rowEmprestimo, $rowCredito);
            }
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            $this->contabilizacaoService->saveContabilizacaoData($response);
            throw $e;
        }
    }

    public function geraLancamentoCredito($rowEmprestimo, $rowCredito){
        $this->geraSequenciaisCreditoBancario($rowCredito);
        $rowMovbancario = $this->constroiRowMovbancario($rowCredito, self::CREDITO);
        $rowMovcredbanc = $this->constroiRowMovcredbanc($rowEmprestimo, $rowCredito);
        $movbancario = $this->factory->getMovimentoBancario($rowMovcredbanc);
        if (!$movbancario){
            $centroCusto = $this->factory->getCentroCusto($this->nrorg, $rowEmprestimo['CDFILIAL']);
            $contaContabil = $this->factory->getContaContabil(
                $rowEmprestimo['CDEMPRESA'], $rowEmprestimo['CDFILIAL'], $rowEmprestimo['NRINSJURFORN']
            );
            $contCtblMov = ($rowCredito['CDCONTCTBLCRED']) ? $rowCredito['CDCONTCTBLCRED'] : $contaContabil->getCdcontctbl();
            $this->executaFluxoDeCaixaCredito($rowEmprestimo, $rowCredito);
            $movbancario = $this->factory->insereMovbancario($rowMovbancario);
            $this->entityManager->persist($movbancario);
            $this->entityManager->flush();

            $this->factory->getContas($rowCredito['CDREDUCONTCRED']);

            $movcredbanc = $this->factory->insereMovcredbanc($rowMovcredbanc);
            $this->entityManager->persist($movcredbanc);
            $this->entityManager->flush();

            $this->fluxoDeCaixaService->atualizaSaldoContCorr(
                $rowCredito['CDREDUCONTCRED'],
                $this->formataData($rowCredito['DTMOVICRED']),
                $this->nrorg,
                $rowCredito['VRMOVICRED']
            );
            $this->contabilizaBcoCredito($rowEmprestimo, $rowCredito);
        }
    }

    public function geraLancamentoDebito($rowEmprestimo, $rowCredito){
        $this->geraSequenciaisDebitoBancario($rowCredito);
        $rowMovbancario = $this->constroiRowMovbancario($rowCredito, self::DEBITO);
        $rowMovdebibanc = $this->constroiRowMovdebbanc($rowEmprestimo, $rowCredito);
        $movbancario = $this->factory->getMovimentoBancario($rowMovdebibanc);
        if (!$movbancario){
            $centroCusto = $this->factory->getCentroCusto($this->nrorg, $rowEmprestimo['CDFILIAL']);
            $contaContabil = $this->factory->getContaContabil(
                $rowEmprestimo['CDEMPRESA'], $rowEmprestimo['CDFILIAL'], $rowEmprestimo['NRINSJURFORN']
            );
            $contCtblMov = ($rowCredito['CDCONTCTBL']) ? $rowCredito['CDCONTCTBL'] : $contaContabil->getCdcontctbl();
            $this->factory->getContas($rowCredito['CDREDUCONT']);
            $this->executaFluxoDeCaixaDebito($rowEmprestimo, $rowCredito);
            $movbancario = $this->factory->insereMovbancario($rowMovbancario);
            $this->entityManager->persist($movbancario);
            $this->entityManager->flush();
            $movdebbanc = $this->factory->insereMovdebbanc($rowMovdebibanc);
            $this->entityManager->persist($movdebbanc);
            $this->entityManager->flush();
            $this->fluxoDeCaixaService->atualizaSaldoContCorr(
                $rowCredito['CDREDUCONT'],
                $this->formataData($rowCredito['DTMOVIDEBI']),
                $this->nrorg,
                $rowCredito['VRMOVIDEBI'] * -1
            );
            $this->contabilizaBcoDebito($rowEmprestimo, $rowCredito);
        }
    }

    public function executaContabilizacao($rowEmprestimo){
        $this->contabilizacaoService->contabilizaPagamento(
            $rowEmprestimo['CDFILIAL'],
            $rowEmprestimo['CDEMPRESA'],
            $rowEmprestimo['NRLANCTONF'],
            '00',
            '01',
            self::ABERTO,
            $rowEmprestimo['CDTPCTPAGAR']
        );
    }

    public function validaVinculosEmprestimo($rowEmprestimo){
        $this->verificaNotaCadastrada($rowEmprestimo);
        $this->verificaDocumento($rowEmprestimo);
        $this->fluxoDeCaixaService->validaOrcamento(
            $rowEmprestimo['CDEMPRESA'],     $rowEmprestimo['CDFILIAL'],
            $rowEmprestimo['CDCLASSFINA'],   $this->cdfiliproc,
            $rowEmprestimo['DTEMPRESTIMO'],  $rowEmprestimo['VRTOTALEMP'],
            $this->environment->getNrOrgTrab()
        );
    }

    public function constroiRowEmprestimoLeasing($row){
        $row['NRLANCTONF']      = $this->nrlanctonf;
        $row['CDFILIAL']        = $this->cdfiliproc;
        $row['CDOPERACTPG']     = $this->environment->getCdoperador();
        $row['VRTOTALEMP']      = floatval($row['VRTOTALEMP']);
        $row['VRTXJUROSEMP']    = floatval($row['VRTXJUROSEMP']);
        $row['VRTOTALAJUSTEMP'] = floatval($row['VRTOTALAJUSTEMP']);
        $row['NRSEQUENCIALEMP'] =  str_pad($this->nrsequencialemp, 10, "0", STR_PAD_LEFT);
        $row['CDAJUCTPAG']      = (isset($row['CDAJUCTPAG'])) ? $row['CDAJUCTPAG'] : null;
        $row['DSAJUSTEEMP']     = (isset($row['DSAJUSTEEMP'])) ? $row['DSAJUSTEEMP'] : null;
        $dtemprestimo = $this->formataData($row['DTEMPRESTIMO']);
        $dtprivencemp = $this->formataData($row['DTPRIVENCEMP']);
        $dtemprestimo = $this->formataData($row['DTEMPRESTIMO']);
        return array(
            'NRORG'           => $this->nrorg,              'CDFILIPROC'      => $this->cdfiliproc,
            'CDFILIAL'        => $row['CDFILIAL'],          'NRLANCTONF'      => $this->nrlanctonf,
            'CDFILIORIG'      => $this->cdfiliproc,         'VRTOTALEMP'      => $row['VRTOTALEMP'],
            'VRTXJUROSEMP'    => $row['VRTXJUROSEMP'],      'VRTOTALAJUSTEMP' => $row['VRTOTALAJUSTEMP'],
            'NRSEQUENCIALEMP' => $row['NRSEQUENCIALEMP'],   'CDAJUCTPAG'      => $row['CDAJUCTPAG'],
            'DSAJUSTEEMP'     => $row['DSAJUSTEEMP'],       'DTEMPRESTIMO'    => $dtemprestimo,
            'CDEMPRESA'       => $row['CDEMPRESA'],         'CDINSCESTA'      => $row['CDINSCESTA'],
            'CDINSCMUNI'      => $row['CDINSCMUNI'],        'IDTPIJURNF'      => $row['IDTPIJURFORN'],
            'NRINSJURNF'      => $row['NRINSJURFORN'],      'CDFORNECED'      => $row['CDFORNECED'],
            'VRNOTAFISC'      => $row['VRTOTALEMP'],        'NRNOTAFISC'      => $row['NRNOTAFISC'],
            'DTCONTRATOEMP'   => $dtemprestimo,             'CDTIPOOPER'      => $row['CDTIPOOPER'],
            'CDSERIEENTR'     => $row['CDSERIEENTR'],       'IDSTATUS'        => self::STATUS_PAG_ABERTO,
            'CDOPERACTPG'     => $row['CDOPERACTPG'],       'DTPRIVENCEMP'    => $dtprivencemp,
            'IDTXJUROSEMP'    => 'M',                       'DSTXJUROSEMP'    => null,
            'CDTPBAIXAPAG'    => $row['CDTPBAIXAPAG'],      'CDTPCTPAGAR'     => $row['CDTPCTPAGAR'],
            'QTPARCELAEMP'    => $row['PARCELAS'],          'QTDIAPRXVENCEMP' => 30,
            'CDTPGARANTIAEMP' => $row['CDTPGARANTIAEMP'],   'IDTIPOEMP'       => $row['IDTIPOEMP'],
            'IDMOVFLUXO'      => $row['IDMOVFLUXO'],        'CDCLASSFINA'     => $row['CDCLASSFINA'],
            'NRINSJURFORN'    => $row['NRINSJURFORN'],
        );
    }

    public function constroiRowAjusteBoleto($row, $valorAjuste){
        return array(
            'NRORG'        => $this->nrorg,        'CDFILIPROC'   => $row['CDFILIAL'],
            'NRLANBOLPAG'  => $this->nrlanbolpag,  'CDAJUCTPAG' => $row['CDAJUCTPAG'],
            'IDORIGAJBPAG'  => '4',                 'IDTPAJBOLPAG'   => '1',
            'VRAJBOLPAG' => $valorAjuste
        );
    }

    public function constroiRowConta($row, $parcela){
        $idmovfluxo = ($row['IDMOVFLUXO']) ? $row['IDMOVFLUXO']: self::TIPO_MOV_EMPENHO;
        $parcela['DTATUAVENPAG'] = $this->formataData($parcela['DTATUAVENPAG']);
        
        return array(
            'NRORG'        => $this->nrorg,              'CDFILIAL'        => $row['CDFILIAL'],
            'NRLANCTONF'   => $row['NRLANCTONF'],        'NRSEQVENPAG'     => $parcela['NRSEQVENPAG'],
            'NRPARCELAPAG' => $parcela['NRPARCELAPAG'],  'DTORIGVENPAG'    => $parcela['DTATUAVENPAG'],
            'VRATUAPAG'    => $parcela['VRATUAPAG'],     'CDTPCTPAGAR'     => $row['CDTPCTPAGAR'],
            'DTATUAVENPAG' => $parcela['DTATUAVENPAG'],  'VRORIGPAG'       => $parcela['VRATUAPAG'],
            'CDFILIBPAG'   => $row['CDFILIAL'],          'NRLANBOLPAG'     => $this->nrlanbolpag,
            'IDSTATPAG'    => self::STATUS_PAG_ABERTO,   'QTDIASVENPAG'    => '0',
            'CDCLASSFINA'  => $row['CDCLASSFINA'],       'CDEMPRESA'       => $row['CDEMPRESA'],
            'DTMOVFLUXO'   => $parcela['DTATUAVENPAG'],  'IDMOVFLUXO'      => $idmovfluxo,
            'CDFIMOVFLPAG' => $row['CDFILIAL'],          'DSCONTAPAG'      => $parcela['DSCONTAPAG'],
            'CDMOTIPARC'   => '',                        'NRSEQUENCIALEMP' => $row['NRSEQUENCIALEMP'],
            'NRSEQVENPAGVISU' => $parcela['NRSEQVENPAGVISU']
        );
    }

    public function constroiRowMovbancario($row, $tipoMovimento){
        $row['NRDOCMBANCCRED'] = (isset($row['NRDOCMBANCCRED'])) ? $row['NRDOCMBANCCRED'] : null;
        $row['NRDOCMBANC'] = (isset($row['NRDOCMBANC'])) ? $row['NRDOCMBANC'] : null;
        $row['VRMOVICRED'] = floatval($row['VRMOVICRED']);
        if ($tipoMovimento == 'C'){
            $dtmovicred = $this->formataData($row['DTMOVICRED']);
            return array(
                'CDREDUCONT' => $row['CDREDUCONTCRED'],  'NRMOVBANC'  => $this->nrmovbanc,
                'DTMOVBANC'  => $dtmovicred,             'DEBCRED'    => $tipoMovimento,
                'DSHISMBANC' => $row['DSMOVICRED'],      'VRMOVBANC' => $row['VRMOVICRED'],
                'IDSTCONEXT' => '0',                     'NRDOCMBANC' => $row['NRDOCMBANCCRED'],
                'NRORG'      => $this->nrorg
            );
        }else{
            $dtmovidebi = $this->formataData($row['DTMOVIDEBI']);
            return array(
                'CDREDUCONT' => $row['CDREDUCONT'],  'NRMOVBANC'  => $this->nrmovbanc,
                'DTMOVBANC'  => $dtmovidebi,         'DEBCRED'    => $tipoMovimento,
                'DSHISMBANC' => $row['DSMOVIDEBI'],  'VRMOVBANC'  => $row['VRMOVIDEBI'],
                'IDSTCONEXT' => '0',                 'NRDOCMBANC' => $row['NRDOCMBANC'],
                'NRORG'      => $this->nrorg
            );
        }
    }

    public function constroiRowMovcredbanc($row, $credito){
        $cdfilial = ($row['CDFILIAL']) ? $row['CDFILIAL'] : '';
        $cdforneced = ($row['CDFORNECED']) ? $row['CDFORNECED'] : '';
        $dtmovicred = $this->formataData($credito['DTMOVICRED']);
        $credito['VRMOVICRED'] = floatval($credito['VRMOVICRED']);
        $credito['DSHISMBANC'] = $credito['DSMOVICRED'];
        $credito['VRMOVICRED'] = floatval($credito['VRMOVICRED']);
        $credito['NRDOCMBANCCRED'] = (isset($credito['NRDOCMBANCCRED'])) ? $credito['NRDOCMBANCCRED'] : null;
        $credito['NRDOCMBANC'] = (isset($credito['NRDOCMBANC'])) ? $credito['NRDOCMBANC'] : null;
        if ($credito['IDTIPOCCORRCRED'] == 'C'){
            return array(
                'CDREDUCONT'      => $credito['CDREDUCONTCRED'],   'NRMOVICRED'   => $this->nrmovicred,
                'CDCLASSFINA'     => $credito['CDCLASSFINACRED'],  'CDEMPRESA'    => $row['CDEMPRESA'],
                'DTMOVBANC'       => $dtmovicred,                  'IDMOVFLUXO'   => self::TIPO_MOV_REAL,
                'NRMOVBANC'       => $this->nrmovbanc,             'CDTPCREDBANC' => $credito['CDTPCREDBANC'],
                'DATAMOVIMENTO'   => $credito['DTMOVICRED'],       'DSMOVICRED'   => $credito['DSMOVICRED'],
                'VRMOVICRED'      => $credito['VRMOVICRED'],       'NRORG'        => $this->nrorg,
                'NRSEQUENCIALEMP' => $row['NRSEQUENCIALEMP'],      'NRLANCCONT'   => $this->nrlanccont,
                'CDFILIAL'        => $cdfilial,                    'CDFORNECED'   => $cdforneced,
                'NRDOCMBANC'      => $credito['NRDOCMBANC'],       'VRMOVBANC'    => $credito['VRMOVICRED'],
                'DSHISMBANC'      => $credito['DSHISMBANC'], 'CDFILIPROC'      => $this->cdfiliproc
            );
        }else{
            return array(
                'CDREDUCONT'      => $credito['CDREDUCONTCRED'],   'NRMOVICRED'   => $this->nrmovicred,
                'CDCLASSFINA'     => '',                           'CDEMPRESA'    => '',
                'DTMOVBANC'       => $dtmovicred,                  'IDMOVFLUXO'   => '',
                'NRMOVBANC'       => $this->nrmovbanc,             'CDTPCREDBANC' => $credito['CDTPCREDBANC'],
                'DATAMOVIMENTO'   => $credito['DTMOVICRED'],       'DSMOVICRED'   => $credito['DSMOVICRED'],
                'VRMOVICRED'      => $credito['VRMOVICRED'],       'NRORG'        => $this->nrorg,
                'NRSEQUENCIALEMP' => $row['NRSEQUENCIALEMP'],      'NRLANCCONT'   => $this->nrlanccont,
                'CDFILIAL'        => '',                           'CDFORNECED'   => $cdforneced,
                'NRDOCMBANC'      => $credito['NRDOCMBANC'],       'VRMOVBANC'    => $credito['VRMOVICRED'],
                'DSHISMBANC'      => $credito['DSHISMBANC'], 'CDFILIPROC'      => $this->cdfiliproc
            );
        }
    }

    public function constroiRowMovdebbanc($row, $credito){
        $nrsequencialemp = ($row['NRSEQUENCIALEMP']);
        $this->nrlanccont = ($this->nrlanccont) ? $this->nrlanccont : '';
        $cdfilial = ($row['CDFILIAL']) ? $row['CDFILIAL'] : '';
        $cdforneced = ($row['CDFORNECED']) ? $row['CDFORNECED'] : '';
        $credito['VRMOVIDEBI'] = floatval($credito['VRMOVIDEBI']);
        $credito['DSHISMBANC'] = $credito['DSMOVIDEBI'];
        $dtmovidebi = $this->formataData($credito['DTMOVIDEBI']);
        $credito['VRMOVIDEBI'] = floatval($credito['VRMOVIDEBI']);
        $credito['NRDOCMBANC'] = (isset($credito['NRDOCMBANC'])) ? $credito['NRDOCMBANC'] : null;
        if ($credito['IDTIPOCCORR'] == 'C'){
            return array(
                'CDREDUCONT'      => $credito['CDREDUCONT'],   'NRMOVIDEB'    => $this->nrmovidebi,
                'CDCLASSFINA'     => $credito['CDCLASSFINA'],  'CDEMPRESA'    => $row['CDEMPRESA'],
                'DTMOVBANC'       => $dtmovidebi,              'IDMOVFLUXO'   => self::TIPO_MOV_REAL,
                'NRMOVBANC'       => $this->nrmovbanc,         'CDTPDEBIBANC' => $credito['CDTPDEBIBANC'],
                'DATAMOVIMENTO'   => $credito['DTMOVIDEBI'],   'DSMOVIDEBI'   => $credito['DSMOVIDEBI'],
                'VRMOVIDEBI'      => $credito['VRMOVIDEBI'],   'NRORG'        => $this->nrorg,
                'NRSEQUENCIALEMP' => $nrsequencialemp,         'NRLANCCONT'   => $this->nrlanccont,
                'CDFILIAL'        => $cdfilial,                'CDFORNECED'   => $cdforneced,
                'NRDOCMBANC'      => $credito['NRDOCMBANC'],   'VRMOVBANC'    => $credito['VRMOVIDEBI'],
                'DSHISMBANC'      => $credito['DSHISMBANC'], 'CDFILIPROC'      => $this->cdfiliproc
            );
        }else{
            return array(
                'CDREDUCONT'      => $credito['CDREDUCONT'],       'NRMOVIDEB'    => $this->nrmovidebi,
                'CDCLASSFINA'     => '',                           'CDEMPRESA'    => '',
                'DTMOVBANC'       => $dtmovidebi,                  'IDMOVFLUXO'   => '',
                'NRMOVBANC'       => $this->nrmovbanc,             'CDTPDEBIBANC' => $credito['CDTPDEBIBANC'],
                'DATAMOVIMENTO'   => $credito['DTMOVIDEBI'],       'DSMOVIDEBI'   => $credito['DSMOVIDEBI'],
                'VRMOVIDEBI'      => $credito['VRMOVIDEBI'],       'NRORG'        => $this->nrorg,
                'NRSEQUENCIALEMP' => $nrsequencialemp,             'NRLANCCONT'   => $this->nrlanccont,
                'CDFILIAL'        => '',                           'CDFORNECED'   => $cdforneced,
                'NRDOCMBANC'      => $credito['NRDOCMBANC'],       'VRMOVBANC'    => $credito['VRMOVIDEBI'],
                'DSHISMBANC'      => $credito['DSHISMBANC'], 'CDFILIPROC'      => $this->cdfiliproc
            );
        }
    }

    public function constroiRowBoletoPagar($row, $parcela){
        $ccorforn = $this->factory->getContaCorrenteFornecedor(
            $this->nrorg,
            $row['CDFORNECED']
        );
        $cdtpbaixapag = ($row['CDTPBAIXAPAG']) ? $row['CDTPBAIXAPAG']: '';
        if ($ccorforn){
            $bancoForn = $ccorforn->getCdbancforn();
            $agenciaForn = $ccorforn->getCdagenforn();
            $contaCorrenteForn = $ccorforn->getCdccorforn();
        }else{
            $bancoForn = null;
            $agenciaForn = null;
            $contaCorrenteForn = null;
        }
        $parcela['DTATUAVENPAG'] = $this->formataData($parcela['DTATUAVENPAG']);
        return array(
            'NRORG'       => $this->nrorg,             'CDFILIAL'     => $row['CDFILIAL'],
            'NRLANBOLPAG' => $this->nrlanbolpag,       'CDFORNECED'   => $row['CDFORNECED'],
            'CDEMPRESA'   => $row['CDEMPRESA'],        'VRBOLPAG'     => $parcela['VRATUAPAG'],
            'VRFINALBPAG' => $parcela['VRFINALBPAG'],  'DTVENBOLPAG'  => $parcela['DTATUAVENPAG'],
            'vrmultabpag' => 0,                        'CDBANCBOLP'   => $bancoForn,
            'CDAGENBOLP'  => $agenciaForn,             'CDCCORBOLP'   => $contaCorrenteForn,
            'IDHABIBPAG'  => 'S',                      'CDTPBAIXAPAG' => $cdtpbaixapag,
            'IDBPAGREAL'  => 'N',                      'NRPARCBOLPAG' => $parcela['NRPARCELAPAG'],
            'CDOPERINCLUSAO'  => $this->environment->getCdOperador()
        );
    }

    public function contabilizaBcoDebito($row, $credito){
        $this->contabilizacaoService->contabilizaLancamentoControleBancario(
            $row['CDEMPRESA'],
            $credito['CDREDUCONT'],
            $this->nrmovidebi,
            self::DEBITO,
            $credito['IDCTBLDBBANC']
        );
    }

    public function contabilizaBcoCredito($row, $credito){
        $this->contabilizacaoService->contabilizaLancamentoControleBancario(
            $row['CDEMPRESA'],
            $credito['CDREDUCONTCRED'],
            $this->nrmovicred,
            self::CREDITO,
            $credito['IDCTBLCRBANC']
        );
    }

    public function executaFluxoDeCaixaDebito($row, $credito){
        if ($credito['IDTIPOCCORR'] == self::TIPOCONTA_CIRCULANTE){
            $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                $this->nrorg,
                $row['CDEMPRESA'],
                $row['CDFILIAL'],
                $credito['CDCLASSFINA'],
                $credito['DTMOVIDEBI'],
                $credito['VRMOVIDEBI'],
                self::TIPO_MOV_REAL,
                self::ENTRADA
            );
            $this->entityManager->persist($movfluxo);
            $this->entityManager->flush();
        }
    }

    public function executaFluxoDeCaixaCredito($row, $credito){
        if ($credito['IDTIPOCCORRCRED'] == self::TIPOCONTA_CIRCULANTE){
            $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                $this->nrorg,                $row['CDEMPRESA'],
                $row['CDFILIAL'],            $credito['CDCLASSFINACRED'],
                $credito['DTMOVICRED'],      $credito['VRMOVICRED'],
                self::TIPO_MOV_REAL,         self::ENTRADA
            );
            $this->entityManager->persist($movfluxo);
        }
    }

    public function executaFluxoDeCaixaParcelas($rowConta){
        $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
            $this->nrorg,                 $rowConta['CDEMPRESA'],
            $rowConta['CDFILIAL'],        $rowConta['CDCLASSFINA'],
            $rowConta['DTATUAVENPAG'],    $rowConta['VRATUAPAG'],
            $rowConta['IDMOVFLUXO'],      self::SAIDA
        );
        $this->entityManager->persist($movfluxo);
        $this->entityManager->flush();
    }

    public function constroiRowFluxoCaixa($row, $valorAjuste){
        return array(
            'NRORG'        => $this->nrorg,          'CDEMPRESA'   => $row['CDEMPRESA'],
            'CDFILIAL'  => $row['CDFILIAL'],         'CDCLASSFINA' => $row['CDCLASSFINA'],
            'DTMOVICRED'  => $row['DTMOVICRED'],     'VRMOVICRED'   => $row['DTMOVICRED'],
            'TIPO_MOV_REAL'  => self::TIPO_MOV_REAL, 'ENTRADA'   => self::ENTRADA
        );
    }

    public function saveReparcelamentoEmprestimoPagar($rowEmprestimo, $parcelasExclusao, $dataSetParcelas){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try{
            foreach ($parcelasExclusao as $parcela) {
                $this->controleTituloAlteradoService->validaContaAlterada(
                    $parcela['CDFILIAL'],
                    $parcela['NRLANCTONF'],
                    $parcela['NRSEQVENPAG'],
                    $parcela['NRPARCELAPAG'],
                    $this->formataData($parcela['DTATUAVENPAG']),
                    $parcela['VRATUAPAG']
                );
                $this->excluiParcela($parcela);
            }
            $this->insereParcelas($rowEmprestimo, $dataSetParcelas);
            $this->entityManager->flush();
            $connection->commit();
        }catch(\Exception $e){
            $connection->rollback();
            Throw $e;
        }
    }

    public function saveCreditoEmprestimoPagar($credito){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try{
            if ($credito['DEBIOF'] == 'S'){
                $movbancario = $this->factory->getMovbancario(
                    $this->nrorg, $credito['NRMOVBANC'], $credito['CDREDUCONT']
                );
                $movbancario->setDshismbanc($credito['DSMOVIDEBI']);
                $movbancario->setNrdocmbanc($credito['NRDOCMBANC']);
                $this->entityManager->persist($movbancario);
                $movdebbanc = $this->factory->getMovdebbanc(
                    $this->nrorg,
                    $credito['NRMOVBANC'],
                    $credito['CDREDUCONT']
                );
                $movdebbanc->setDsmovidebi($credito['DSMOVIDEBI']);
                $this->entityManager->persist($movdebbanc);
            }
            if ($credito['CREDITO'] == 'S'){
                $movbancario = $this->factory->getMovbancario(
                    $this->nrorg,
                    $credito['NRMOVBANCCRED'],
                    $credito['CDREDUCONTCRED']
                );
                $movbancario->setDshismbanc($credito['DSMOVICRED']);
                $movbancario->setNrdocmbanc($credito['NRDOCMBANCCRED']);
                $this->entityManager->persist($movbancario);
                $movcredbanc = $this->factory->getMovcredbanc(
                    $this->nrorg,
                    $credito['NRMOVBANCCRED'],
                    $credito['CDREDUCONTCRED']
                );
                $movcredbanc->setDsmovicred($credito['DSMOVICRED']);
                $this->entityManager->persist($movcredbanc);
            }
            $this->entityManager->flush();
            $connection->commit();
        }catch(\Exception $e){
            $connection->rollback();
            Throw $e;
        }
    }

     public function insereParcelas($rowEmprestimo, $dataSetParcelas){
        try{
            foreach($dataSetParcelas as $parcela){
                $this->nrlanbolpag = $this->newCode->getCodeByTableName(
                    'BOLETPAGAR'.$rowEmprestimo['CDFILIAL'],$this->nrorg
                );
                
                // Transforma NRSEQVENPAG em Alfanúmerico
                $parcela['NRSEQVENPAGVISU'] = $parcela['NRSEQVENPAG'];
                if($parcela['NRSEQVENPAG'] >= 100){
                    $nrseqvenpag = $this->factory->getParcelaAlfanumerica($parcela['NRSEQVENPAG']);
                    $parcela['NRSEQVENPAG'] = $nrseqvenpag['NRPARCELAPAG'];
                }
                
                $rowBoletoPagar = $this->constroiRowBoletoPagar($rowEmprestimo, $parcela);
                $rowConta = $this->constroiRowConta($rowEmprestimo, $parcela);
                $boletPagar = $this->factory->insereBoletoPagar($rowBoletoPagar);
                $this->entityManager->persist($boletPagar);
                $this->entityManager->flush();
                $this->validaInsercaoAjuste($parcela, $rowEmprestimo);
                $this->executaFluxoDeCaixaParcelas($rowConta);
                $contaPagar = $this->factory->insereContaPagar($rowConta);
                $this->entityManager->persist($contaPagar);
                $this->entityManager->flush();
                $this->geraRateio($contaPagar, $rowEmprestimo, $parcela);
            }
        }catch(\Exception $e){
            Throw $e;
        }
    }

    public function geraRateio($contaPagar, $rowEmprestimo, $parcela){
        $ratctpagar = $this->rateioContaService->geraRateioPag(
            $this->nrorg,                        $contaPagar->getCdfilial(),
            $rowEmprestimo['CDFILIAL'],          $rowEmprestimo['NRLANCTONF'],
            $parcela['NRSEQVENPAG'],             $parcela['NRPARCELAPAG'],
            $rowEmprestimo['CDTPCTPAGAR'],       $parcela['VRATUAPAG']
        );
        foreach($ratctpagar as $rateio){
            $this->entityManager->persist($rateio);
        }
    }

    public function validaInsercaoAjuste($parcela, $rowEmprestimo){
        $valorAjuste = floatval($parcela['VRFINALBPAG']) - floatval($parcela['VRATUAPAG']);
        if ($valorAjuste > 0){
            $rowAjusteBoleto = $this->constroiRowAjusteBoleto($rowEmprestimo, $valorAjuste);
            $ajbolpag = $this->factory->insereAjusteBoletPagar($rowAjusteBoleto);
            $this->entityManager->persist($ajbolpag);
        }
    }

    public function deleteDocumentoEmprestimoPagar($rowEmprestimo, $dataSetParcelas, $credito, $debito, $response){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try{
            $ajustes = $this->factory->getAjustePagar(
                $dataSetParcelas[0]['CDFILIAL'],
                $dataSetParcelas[0]['NRLANCTONF'],
                '', 'T', '', 'T', '', 'T', '', 'T'
            );
            $this->nrlanccont = $this->newCode->getCodeByTableName('LANCCONTABIL'.$rowEmprestimo['CDEMPRESA'].date('YYYY/MM'),$this->nrorg);
            $paramGeral = $this->factory->getParamgeral();
            if ($paramGeral->getIdexclanctb() != '2'){
                $this->factory->excluiLancamentoContabilEstorno($rowEmprestimo['CDEMPRESA'], $rowEmprestimo['NRLANCCONT'], $this->nrorg);
            }else{
                $this->excluiLancamentoContabil($rowEmprestimo['CDEMPRESA'], $rowEmprestimo['NRLANCCONT'], $rowEmprestimo['DTCONTRATOEMP']);
            }
            $this->excluiAjusteRateioDocumento($ajustes);
            $this->validaSituacaoOrcamento($dataSetParcelas, $rowEmprestimo['CDEMPRESA']);
            $cdfilial = $dataSetParcelas[0]['CDFILIAL'];
            $nrlanctonf = $dataSetParcelas[0]['NRLANCTONF'];
            $this->excluiDocumento($cdfilial, $nrlanctonf);
            if (isset($credito['CREDITO'])){
                if ($credito['CREDITO'] == 'S'){
                    $this->validaExclusaoDebitoIof($paramGeral, $credito['CDREDUCONTCRED'], $credito['DTMOVICRED'], $credito['VRMOVICRED'],
                                                   $rowEmprestimo['CDEMPRESA'], $credito['NRLCTBMCRED'], 'C', $credito['NRSEQMOVREBACRED'], $credito['CDSEQUEXBACRED'], $credito['NRMOVBANCCRED']);
                }
            }
            if (isset($debito['DEBIOF'])){
                if ($debito['DEBIOF'] == 'S'){
                    
                    $this->validaExclusaoDebitoIof($paramGeral, $debito['CDREDUCONT'], $debito['DTMOVIDEBI'], $debito['VRMOVIDEBI'],
                                                   $rowEmprestimo['CDEMPRESA'], $debito['NRLCTBMDEB'], 'D', $debito['NRSEQMOVREBA'], $debito['CDSEQUEXBA'], $debito['NRMOVBANC']);
                }
            }
            $empleasing = $this->factory->getEmpleasing($this->nrorg, $rowEmprestimo['NRSEQUENCIALEMP'], $rowEmprestimo['CDFILIAL']);
            $this->entityManager->remove($empleasing);
            $this->entityManager->flush();
            $connection->commit();
        }catch(\Exception $e){
            $connection->rollback();
            $this->contabilizacaoService->saveContabilizacaoData($response);
            Throw $e;
        }
    }

    public function validaExclusaoDebitoIof($paramGeral, $cdreducont, $dtmovidebi, $vrmovbanc, $cdempresa, $nrlanccont, $tipoLancamento, $nrseqmovreba, $cdsequexba, $nrmovbanc){
        if ($paramGeral->getIdexclanctb() != 'S'){
            $this->factory->excluiLancamentoContabilEstorno($cdempresa, $nrlanccont, $this->nrorg);
        }else{
            $this->excluiLancamentoContabil($cdempresa, $nrlanccont, $dtmovidebi);
        }
        $this->validaMovimentosConciliados($nrmovbanc, $cdreducont);
            
        $dadosLancamento = $this->factory->getDadosLanc($nrmovbanc, $cdreducont, $tipoLancamento);
        if ($dadosLancamento){
            $operacao = ($tipoLancamento) ? 'D' : 'C';
            $this->fluxoDeCaixaService->atualizaFluxoCaixa($this->nrorg,
                                                           $dadosLancamento[0]['CDEMPRESA'],
                                                           $dadosLancamento[0]['CDFILIAL'],
                                                           $dadosLancamento[0]['CDCLASSFINA'],
                                                           $dadosLancamento[0]['DTMOVFLUXO'],
                                                           $vrmovbanc * -1,
                                                           self::TIPO_MOV_REAL,
                                                           $operacao);
        }
        if ((!$nrseqmovreba) & (!$cdsequexba)) {
            
            if ($tipoLancamento == 'C'){
                    
                $movcredbanc = $this->factory->getMovcredbanc($this->nrorg, $nrmovbanc, $cdreducont);
                
                if($movcredbanc){
                    $this->entityManager->remove($movcredbanc);
                    $this->fluxoDeCaixaService->atualizaSaldoContCorr(
                        $cdreducont, $this->formataData($dtmovidebi), $this->nrorg, $vrmovbanc * -1
                    );
                }
            }else{
                
                $movdebbanc = $this->factory->getMovdebbanc($this->nrorg, $nrmovbanc, $cdreducont);
                //trava pois $movdebbanc está errado
                //$movdebbanc ta errado pois $nrmovbanc e $cdreducont são nulls
                if($movdebbanc){
                    $this->entityManager->remove($movdebbanc);
                    $this->fluxoDeCaixaService->atualizaSaldoContCorr(
                        $cdreducont, $this->formataData($dtmovidebi), $this->nrorg, $vrmovbanc
                    );
                }
            }
            $this->entityManager->flush();
            $movbancario = $this->factory->getMovbancario($this->nrorg, $nrmovbanc, $cdreducont);
            if($movbancario){
                $this->entityManager->remove($movbancario);
            }

        }
    }

    public function validaMovimentosConciliados($nrmovbanc, $cdreducont){
        try{
            $movBancario = $this->factory->getMovbancario($this->nrorg, $nrmovbanc, $cdreducont);
            
            if($movBancario)
                if (($movBancario->getNrseqmovreba() && $movBancario->getCdsequexba()) || ($movBancario->getNrconcimovb())){
                    throw new \Exception('Exclusão não permitida! Movimento bancário do lancamento foi conciliado junto com outros movimentos bancários.');
                }
                
        }catch(\Exception $e){
            Throw $e;
        }
    }

    public function excluiAjusteRateioDocumento($ajustes){
        foreach ($ajustes as $ajuste) {
            $idmovfluxo = ($ajuste['IDMOVFLUXO']) ? $ajuste['IDMOVFLUXO'] : '';
            if ($ajuste['IDAJUCTPAG'] == 'A'){
                $this->fluxoDeCaixaService->atualizaFluxoCaixa($this->nrorg,
                                                               $ajuste['CDEMMFAJCNTP'],
                                                               $ajuste['CDFIMFAJCNTP'],
                                                               $ajuste['CDCLASSFINA'],
                                                               $ajuste['DTMOVFLUXO'],
                                                               $ajuste['VRAJUSTEPAG'] * -1,
                                                               $idmovfluxo,
                                                               self::SAIDA);
            } else if ($ajuste['IDAJUCTPAG'] == 'D'){
                $this->fluxoDeCaixaService->atualizaFluxoCaixa($this->nrorg,
                                                               $ajuste['CDEMMFAJCNTP'],
                                                               $ajuste['CDFIMFAJCNTP'],
                                                               $ajuste['CDCLASSFINA'],
                                                               $ajuste['DTMOVFLUXO'],
                                                               $ajuste['VRAJUSTEPAG'],
                                                               $idmovfluxo,
                                                               self::SAIDA);
            }

            $this->factory->excluiLancamentoContabilEstorno($ajuste['CDECTBAJPAG'], $ajuste['NRLCTBAJPAG'], $this->nrorg);
            $ajctpagar = $this->factory->getAjusteContaPagar($this->nrorg, $ajuste['CDAJUCTPAG'], $AJUSTE['DTAJUSTEPAG'], $ajuste['CDFILIAL'],
                                                             $ajuste['NRLANCTONF'], $ajuste['NRSEQVENPAG'], $ajuste['NRPARCELAPAG']);
            $this->entityManager->remove($ajctpagar);
            $rateios = $this->factory->getRatctpagar(
                array('nrorg' => $this->nrorg,
                      'cdfilial' => $ajuste['CDFILIAL'],
                      'nrlanctonf' => $ajuste['NRLANCTONF'])
                );
            foreach ($rateios as $rateio) {
                $this->entityManager->remove($rateio);
            }
        }
    }

    public function validaSituacaoOrcamento($dataSetParcelas, $cdempresa){
        foreach ($dataSetParcelas as $parcela) {
            $idmovfluxo = ($parcela['IDMOVFLUXO']) ? $parcela['IDMOVFLUXO'] : '';
            if ($parcela['CDCLASSFINA']){
                $this->fluxoDeCaixaService->atualizaFluxoCaixa($this->nrorg,
                                                               $cdempresa,
                                                               $parcela['CDFIMOVFLPAG'],
                                                               $parcela['CDCLASSFINA'],
                                                               $parcela['DTATUAVENPAG'],
                                                               $parcela['VRORIGPAG'] * -1,
                                                               $idmovfluxo,
                                                               self::SAIDA);
                $this->fluxoDeCaixaService->validaOrcamento(
                    $cdempresa, $parcela['CDFIMOVFLPAG'], $parcela['CDCLASSFINA'],
                    $this->cdfiliproc, $parcela['DTATUAVENPAG'], $parcela['VRATUAPAG'] * -1,
                    $this->environment->getNrOrgTrab()
                );
            }
        }
    }

    public function excluiDocumento($cdfilial, $nrlanctonf){
        try{
            $contasDocumento = $this->factory->getContasPagar(
                        array('nrorg'       => $this->nrorg,
                              'cdfilial'    => $cdfilial,
                              'nrlanctonf'  => $nrlanctonf)
                );
            foreach ($contasDocumento as $conta) {
                $this->controleTituloAlteradoService->validaContaAlterada($conta->getCdfilial(), $conta->getNrlanctonf(), $conta->getNrseqvenpag(), $conta->getNrparcelapag(), $conta->getDtatuavenpag(), $conta->getVratuapag());
                $boletPagar = $this->factory->getBoletPagar(
                        array('cdfilial' => $conta->getCdfilibpag(),
                              'nrlanbolpag' => $conta->getNrlanbolpag())
                    );
                $ajustesBoleto = $this->factory->getAjustesboletoPagar(
                    array('nrorg' => $this->nrorg,
                          'cdfilial' => $boletPagar->getCdfilial(),
                          'nrlanbolpag' => $boletPagar->getNrlanbolpag())
                );
                $this->validaStatusTitulosAPagar($boletPagar, $ajustesBoleto);
                foreach ($ajustesBoleto as $ajusteBoleto) {
                    $this->entityManager->remove($ajusteBoleto);
                }
                $this->entityManager->remove($conta);
                $this->entityManager->remove($boletPagar);
                $this->contabilizacaoService->contabilizaPagamento(
                    $conta->getCdfilial(), $conta->getCdempresa(), $conta->getNrlanctonf(), $conta->getNrparcelapag(),
                    $conta->getNrseqvenpag(), $conta->getIdstatpag(), $conta->getCdtpctpagar()
                );
            }
            $notafiscal = $this->factory->getNotaFiscal(
                    array('cdfilial'    => $cdfilial,
                          'nrlanctonf' => $nrlanctonf)
                );
            $this->entityManager->remove($notafiscal);
        }catch(\Exception $e){
            Throw $e;
        }
    }

    public function validaStatusTitulosAPagar($boletPagar, $ajustesBoleto){
        if ($boletPagar->getIdbpagreal() == "S"){
            Throw new \Exception('Operação bloqueada! Conta relacionada a um boleto real não pode ser alterada.');
        }
        if ($boletPagar->getIdstatbpag() != "A"){
            Throw new \Exception('Operação bloqueada! Conta baixada não pode ser alterada.');
        }
        if ($boletPagar->getNrlancantecipa()){
            Throw new \Exception('Operação bloqueada! Conta antecipada não pode ser alterada.');
        }
        if ($boletPagar->getIdsitucnabp() != "00" && $boletPagar->getIdsitucnabp() != "01"){
            Throw new \Exception('Conta de um boleto enviado ao banco (via CNAB) não pode ser alterada.');
        }
        if ($this->verificaAjusteBoleto($boletPagar->getCdfilial(), $boletPagar->getNrlanbolpag())){
            Throw new \Exception('Operação bloqueada! Boleto da parcela possui ajustes que não foram gerados pelo empréstimo.');
        }
    }

    public function deleteParcelaEmprestimoPagar($dataSetParcelas){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try{
            $valorADebitar = 0;
            foreach ($dataSetParcelas as $parcela) {
                $valorADebitar += floatval($parcela['VRATUAPAG']);
                $this->controleTituloAlteradoService->validaContaAlterada(
                    $parcela['CDFILIAL'],                         $parcela['NRLANCTONF'],
                    $parcela['NRSEQVENPAG'],                      $parcela['NRPARCELAPAG'],
                    $this->formataData($parcela['DTATUAVENPAG']), $parcela['VRATUAPAG']
                );
                $this->excluiParcela($parcela);
            }
            $empleasing = $this->factory->getEmpleasing(
                $this->nrorg, $dataSetParcelas[0]['NRSEQUENCIALEMP'], $dataSetParcelas[0]['CDFILIAL']
            );
            $empleasing->setVrtotalemp(floatval($empleasing->getVrtotalemp()) - floatval($valorADebitar));
            $this->entityManager->persist($empleasing);
            $this->entityManager->flush();
            $connection->commit();
        }catch(\Exception $e){
            $connection->rollback();
            Throw $e;
        }
    }

    public function deleteReparcelamentoEmprestimoPagar($dataSetParcelas){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try{
            $ultimaParcela = $dataSetParcelas[count($dataSetParcelas) - 1];
            $penultimaParcela = $dataSetParcelas[count($dataSetParcelas) - 2];
            $valorAjusteParcelaAnterior = $ultimaParcela['VRAJUSTE'] + $penultimaParcela['VRAJUSTE'];
            $valorBoletoAnterior = $ultimaParcela['VRATUAPAG'] + $penultimaParcela['VRATUAPAG'];
            $this->validaPermissaoExclusaoParcela($ultimaParcela);
            $this->validaPermissaoAlteracaoParcela($penultimaParcela);
            $this->excluiParcela($ultimaParcela);
            $this->alteraPenultimaParcela($penultimaParcela, $valorAjusteParcelaAnterior, $valorBoletoAnterior);
            $this->entityManager->flush();
            $connection->commit();
        }catch(\Exception $e){
            $connection->rollback();
            Throw $e;
        }
    }

    public function excluiParcela($ultimaParcela){
        $this->fluxoDeCaixaService->atualizaFluxoCaixa(
            $this->nrorg,                   $ultimaParcela['CDEMPRESA'],
            $ultimaParcela['CDFIMOVFLPAG'], $ultimaParcela['CDCLASSFINA'],
            $ultimaParcela['DTATUAVENPAG'], $ultimaParcela['VRATUAPAG'] * -1,
            self::TIPO_MOV_EMPENHO,         self::SAIDA
            );
        $this->fluxoDeCaixaService->validaOrcamento(
            $ultimaParcela['CDEMPRESA'],    $ultimaParcela['CDFIMOVFLPAG'],
            $ultimaParcela['CDCLASSFINA'],  $this->cdfiliproc,
            $ultimaParcela['DTATUAVENPAG'], $ultimaParcela['VRATUAPAG'] * -1,
            $this->environment->getNrOrgTrab()
        );
        $ratctpagar = $this->factory->getRatctpagar(
            array('nrorg'        => $this->nrorg,
                  'cdfilial'     => $ultimaParcela['CDFILIAL'],
                  'nrlanctonf'   => $ultimaParcela['NRLANCTONF'],
                  'nrseqvenpag'  => $ultimaParcela['NRSEQVENPAG'],
                  'nrparcelapag' => $ultimaParcela['NRPARCELAPAG'])
            );
        $contaPagar = $this->factory->getVencimentoContaPagar(
            $this->nrorg,                   $ultimaParcela['CDFILIAL'],
            $ultimaParcela['NRLANCTONF'],   $ultimaParcela['NRSEQVENPAG'],
            $ultimaParcela['NRPARCELAPAG']
        );
        $ajbolpag = $this->factory->getAjusteboletoPagar(
                array('nrorg' => $this->nrorg,
                      'cdfilial' => $contaPagar->getCdfilibpag(),
                      'nrlanbolpag' => $ultimaParcela['NRLANBOLPAG'],
                      'cdajuctpag' => $ultimaParcela['CDAJUCTPAG'])
            );
        $boletPagar = $this->factory->getboletPagar(
                array('nrorg' => $this->nrorg,
                      'cdfilial' => $contaPagar->getCdfilibpag(),
                      'nrlanbolpag' => $contaPagar->getNrlanbolpag()
                      )
            );
        $this->validaStatusTitulosAPagar($boletPagar, $ajbolpag);
        if ($ratctpagar){
            $this->entityManager->remove($ratctpagar);
        }
        if ($ajbolpag){
            $this->entityManager->remove($ajbolpag);
        }
        $this->entityManager->remove($contaPagar);
        $this->entityManager->remove($boletPagar);
        $this->entityManager->flush();
    }

    public function alteraPenultimaParcela($penultimaParcela, $valorAjusteParcelaAnterior, $valorBoletoAnterior){
        $this->fluxoDeCaixaService->atualizaFluxoCaixa(
            $this->nrorg,                      $penultimaParcela['CDEMPRESA'],
            $penultimaParcela['CDFIMOVFLPAG'], $penultimaParcela['CDCLASSFINA'],
            $penultimaParcela['DTATUAVENPAG'], $valorBoletoAnterior,
            self::TIPO_MOV_EMPENHO,            self::SAIDA
            );
        $ajbolpag = $this->factory->getAjusteboletoPagar(
            array('nrorg'       => $this->nrorg,
                  'cdfilial'    => $penultimaParcela['CDFILIAL'],
                  'nrlanbolpag' => $penultimaParcela['NRLANBOLPAG'],
                  'cdajuctpag'  => $penultimaParcela['CDAJUCTPAG'])
        );
        if ($ajbolpag){
            $ajbolpag->setVrajbolpag($valorAjusteParcelaAnterior);
            $this->entityManager->persist($ajbolpag);
        }
        $contaPagar = $this->factory->getContaPagar(
            array('nrorg'        => $this->nrorg,
                  'cdfilial'     => $penultimaParcela['CDFILIAL'],
                  'nrlanctonf'   => $penultimaParcela['NRLANCTONF'],
                  'nrseqvenpag'  => $penultimaParcela['NRSEQVENPAG'],
                  'nrparcelapag' => $penultimaParcela['NRPARCELAPAG'])
        );
        $contaPagar->setVrorigpag($valorBoletoAnterior);
        $contaPagar->setVratuapag($valorBoletoAnterior);
        $this->entityManager->persist($contaPagar);
        $boletPagar = $this->factory->getboletPagar(
            array('nrorg'       => $this->nrorg,
                  'cdfilial'    => $penultimaParcela['CDFILIAL'],
                  'nrlanbolpag' => $penultimaParcela['NRLANBOLPAG'])
        );
        $boletPagar->setVrbolpag($valorAjusteParcelaAnterior + $valorBoletoAnterior);
        $boletPagar->setVrfinalbpag($valorAjusteParcelaAnterior + $valorBoletoAnterior);
        $this->entityManager->persist($boletPagar);
        $this->entityManager->flush();
    }

    public function validaPermissaoExclusaoParcela($parcela)
    {
        if ($parcela['IDBPAGREAL'] == 'S'){
            throw new \Exception('Operação bloqueada! Conta relacionada a um boleto real não pode ser excluída.');
        }else if ($parcela['IDSTATPAG'] != self::STATUS_PAG_ABERTO) {
            throw new \Exception('Operação bloqueada! Conta baixada não pode ser excluída.');
        }else if ($parcela['NRLANCANTECIPA']) {
            throw new \Exception('Operação bloqueada! Conta antecipada não pode ser excluída.');
        }else if (!($parcela['IDSITUCNABP'] == '00' || $parcela['IDSITUCNABP'] = '01')) {
            throw new \Exception('Conta de um boleto enviado ao banco (via CNAB) não pode ser excluída.');
        }else if ($this->verificaAjusteBoleto($parcela['CDFILIBPAG'], $parcela['NRLANBOLPAG'])){
            throw new \Exception('Operação bloqueada! Boleto da parcela possui ajustes que não foram gerados pelo empréstimo.');
        }
    }

    public function validaPermissaoAlteracaoParcela($parcela)
    {
        if ($parcela['IDBPAGREAL'] == 'S'){
            throw new \Exception('Operação bloqueada! Conta relacionada a um boleto real não pode ser alterada.');
        }else if ($parcela['IDSTATPAG'] != 'A') {
            throw new \Exception('Operação bloqueada! Conta baixada não pode ser alterada.');
        }else if ($parcela['NRLANCANTECIPA']) {
            throw new \Exception('Operação bloqueada! Conta antecipada não pode ser alterada.');
        }else if (!($parcela['IDSITUCNABP'] == '00' || $parcela['IDSITUCNABP'] = '01')) {
            throw new \Exception('Conta de um boleto enviado ao banco (via CNAB) não pode ser alterada.');
        }
    }

    /**
     * [verificaAjusteBoleto Retorna todos ajustes
     * não gerados pela rotina de empréstimo.]
     * @param  [type] $cdfilial    [description]
     * @param  [type] $nrlanbolpag [description]
     * @return [type]              [description]
     */
    public function verificaAjusteBoleto($cdfilial, $nrlanbolpag){
        $ajustes = $this->factory->getAjustes($cdfilial, $nrlanbolpag);
        return  count($ajustes) > 0;
    }

    public function validaPermissaoEdicaoTitulo($parcela){
        try{
            $this->controleTituloAlteradoService->validaContaAlterada(
                $parcela['CDFILIAL'],
                $parcela['NRLANCTONF'],
                $parcela['NRSEQVENPAG'],
                $parcela['NRPARCELAPAG'],
                $this->formataData($parcela['DTATUAVENPAG']),
                $parcela['VRATUAPAG']
            );
        }catch(\Exception $e){
            Throw $e;
        }
    }

    public function excluiLancamentoContabil($cdempresa, $nrLanccont, $dtlanccont){
        $lanccont = $this->factory->getLanccont($this->nrorg, $cdempresa, $nrLanccont);
        if ($cdempresa && $nrLanccont){
            if ($lanccont){
                $contopectbl = $this->factory->getContopectbl($this->nrorg, $cdempresa);
                if ((!$contopectbl) || (($contopectbl->getDtinicctbl() >= $dtlanccont) &&
                                         ($contopectbl->getDtfinacctbl() <= $dtlanccont))){
                    throw new \Exception("Data fora do período contábil.");
                }else{
                    $this->entityManager->remove($lanccont);
                }
            }else{
                return false;
            }
        }else{
            return false;
        }
    }

    public function verificaGrupoOperadores(){
        $operadorGrupo = false;
        $geralPag = $this->factory->getGeralPag();
        if ($geralPag){
            if ($geralPag->getIdbloqincntp() == 'S'){
                $opergrupop = $this->factory->getOpergrupop($this->environment->getCdoperador(), $geralPag->getCdgropercntp());
                if ($opergrupop){
                    $operadorGrupo = true;
                }
            }
        }
        return $operadorGrupo;
    }

    public function geraNovoCodigo(
        $nrorg, $cdcontador
    ){
        try {
            /** @var Novocodigo $novocodigo */
            $novocodigo = $this->newCode->getCodeByTableName($cdcontador, $nrorg);
            return $novocodigo;
        } catch (\Exception $e) {
            throw $e;
            return null;
        }
    }

    public function aplicarMascaraInscricao($rows) {
        foreach ($rows as &$row) {
            if (strlen($row['NRINSJURFORN']) == 14) {
                $row['NRINSJURFORNMASK'] = $this->masks->mask($row['NRINSJURFORN'], '##.###.###/####-##');
            }
            else if (strlen($row['NRINSJURFORN'])==11) {
                $row['NRINSJURFORNMASK'] = $this->masks->mask($row['NRINSJURFORN'], '###.###.###-##');
            }
        }
        return $rows;
    }

    public static function formataData($data)
    {
        if (strlen($data) == 19)
            $data = DateUtil::truncateString($data);
        $data = DateUtil::getDatadeString($data);
        $data = DateUtil::truncateData($data);

        return $data;
    }
}
?>