<?php

namespace Teknisa\FIN\Modules\ContaPagar\Services;

use Teknisa\FIN\Modules\ContaPagar\Services\RetornoDDA\RetornoFactory;
use Teknisa\FIN\Modules\ContaPagar\Factory\Pag30000RetornoDDAFactory;
use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\Libs\Util\Environment;
use Teknisa\FIN\Modules\ContaPagar\Services\InconsistenciasDDAService;
use Teknisa\Libs\Util\SMTP;

class Pag30000RetornoDDAService
{
    public function __construct(
        EntityManager $entityManager,
        Environment $environment,
        Pag30000RetornoDDAFactory $pag30000RetornoDDAFactory,
        InconsistenciasDDAService $inconsistenciasDDAService,
        SMTP $smtp
    )
    {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->pag30000RetornoDDAFactory = $pag30000RetornoDDAFactory;
        $this->inconsistenciasDDAService = $inconsistenciasDDAService;
        $this->smtp     = $smtp;
        $this->organizacao = $this->environment->getNrOrgTrab();
        $this->cdfiliproc = $this->environment->getCdFilial();
        $this->arquivos = array();
    }

    public function importaRetornoDDA($row, $accesStage = false){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            
            $nrorg = $this->environment->getNrOrgTrab();
            $valorCodificado = $row['ARQUIVO'];
            $valorDecodificado = $this->decodeFile($valorCodificado);
            $rowsRetornoDDA = $this->getStringInlines($valorDecodificado);
            $retornosDDA = $this->getMapeamentoRetorno($row, $rowsRetornoDDA, $nrorg, $accesStage);
            $this->deleteRetornoDDA($retornosDDA[0], $row, $nrorg);
            $this->inconsistenciasDDAService->validaInconsistenciasBoletosDoArquivo($retornosDDA[0]);
            $this->entityManager->flush();
            $connection->commit();
            return [$this->pag30000RetornoDDAFactory->getRetornoDDA(), $retornosDDA[1]];
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }
    
    private function deleteRetornoDDA($retornosDDA, $row, $nrorg){
        try{
            foreach($retornosDDA as $retorno){
                $nrnotafisc = str_pad(preg_replace("/[^0-9]/", "", $retorno->getNrnotafisc()), 20, 0, STR_PAD_LEFT);
                $retorno->setNrnotafisc($nrnotafisc);
                
                // Busca Empresa pela Inscrição Juridica ou por Raiz do CNPJ
                $empresa = $this->pag30000RetornoDDAFactory->getEmpresaRaizCNPJ($retorno->getCnpjEmpresa());
                
                if($empresa){
                    $this->pag30000RetornoDDAFactory->deleteRetornoDDA($retorno, $row, $empresa['CDEMPRESA']);
                }else{
                    throw new \Exception('Operação bloqueada. Retorno DDA não pertence a(s) empresa(s) cadastrada(s).');
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function getDiretorioImportacaoDDA($row){
        $banco = $this->pag30000RetornoDDAFactory->getBanco($row["CDBANCCNAB"], $row["NRORG"]);
        return $banco->getDsdirimparqdda();
    }
    
    private function getMapeamentoRetorno($row, $rowsRetornoDDA, $nrorg, $accesStage = false){
        try {
            $leitorCnab = RetornoFactory::getBancoCnab(
                $row['CDBANCCNAB'],
                $rowsRetornoDDA
            );
            $erro = $leitorCnab->leArquivo($nrorg, $accesStage);
            return [$leitorCnab->getRetornos(), $erro];
        }catch(\Exception $e){
            throw $e;
        }
    }

    private  static function decodeFile($encodedFile) {
        $file = base64_decode($encodedFile);
        return $file;
    }

    private function getStringInlines($file) {
        $file = preg_split('/\R/', $file);
        return $file;
    }
}
?>