<?php

namespace Teknisa\FIN\Modules\ContaPagar\Services\Retorno;

use Fiscal\Entities\Boletpagar;
use Geral\Entities\Empresa;

/**
 * Class LeitorCnabPadraoFebrabam
 * @package Financeiro\Logic\ContasPagar\Cnab\Retornos
 */
class LeitorCnabPadraoFebrabam extends LeitorCnab{

    CONST ID_REGISTRO_HEADER_ARQUIVO = '0';
    CONST ID_REGISTRO_HEADER_LOTE  = '1';
    CONST ID_REGISTRO_DETALHE = '3';
    CONST ID_REGISTRO_TRAILLER_LOTE = '5';
    CONST ID_REGISTRO_TRAILLER_ARQUIVO = '9';
    CONST TIPO_INS_JUR_CPF = 1;
    CONST TIPO_INS_JUR_CNPJ = 2;
    CONST RETORNO = 2;
    CONST SEGMENTO_A = 'A';
    CONST SEGMENTO_B = 'B';
    CONST SEGMENTO_J = 'J';
    CONST SEGMENTO_O = 'O';

    private $cdagencia;
    private $cdconta;
    private $ocorrenciaHeader;
    private $contHeaderLote = 0;
    private $cdfilial;
    private $nrlanbolpag;
    private $vencimento;

    /**
     * @param $arquivo
     * @param $cdempresa
     * @param $cdbanco
     * @throws \Exception
     */
    public function __construct($arquivo, $cdempresa, $cdbanco){
        try {
            $this->cdbanco = $cdbanco;
            $this->arquivo = $arquivo;
            $this->cdempresa = $cdempresa;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Valida Arquivo CNAB
     * @param $linha
     * @param $estruturaArquivo
     * @return mixed
     */
    private function validaArquivo($linha, $estruturaArquivo){
	
        parent::setStatusArquivo(true);
        
        // CAMPO: Tipo de Registro
        // POSIÇÃO: 008 a 008
        // PICTURE: 9(1)
        // DEFAULT:  1
        $idReg = substr($linha,7,1);
        if($this->contlinha == 1 and $idReg != self::ID_REGISTRO_HEADER_ARQUIVO ){
            self::gravaErros($this->contlinha, "Erro de identificação do registro header.",true);
            parent::setStatusArquivo(false);
        }
        if($this->contlinha > 1 and (! in_array($idReg, $estruturaArquivo)
        )){
            self::gravaErros($this->contlinha,"Identificação do registro não é válida",true);
            parent::setStatusArquivo(false);
        }

        return parent::getStatusArquivo();
    }

    /**
     * Busca Estreutura Arquivo
     * @return array
     */
     public function getEstruturaArquivo(){
        return array(
                self::ID_REGISTRO_HEADER_ARQUIVO,
                self::ID_REGISTRO_HEADER_LOTE,
                self::ID_REGISTRO_DETALHE,
                self::ID_REGISTRO_TRAILLER_LOTE,
                self::ID_REGISTRO_TRAILLER_ARQUIVO
            );
    }

    /**
     * Lê Arquivo CNAB
     * @param $nrorg
     * @throws \Exception
     */
    public function leArquivo($nrorg){
        try {
            foreach ($this->arquivo as $linha) {
                $this->contlinha++;
                if($linha){

                    $this->retorno = new Retorno();

                    // Busca o Tipo de Registro do Arquivo
                    $idRegistro = substr($linha,7,1);

                    if(self::validaArquivo($linha, self::getEstruturaArquivo())){
                        switch ($idRegistro) {
                            case(self::ID_REGISTRO_HEADER_ARQUIVO):
                                self::leHeaderArquivo($linha, $nrorg);
                                break;
                            case(self::ID_REGISTRO_HEADER_LOTE):
                                self::leHeaderLote($linha);
                                break;
                            case(self::ID_REGISTRO_DETALHE):
                                self::leDetalhe($linha, $nrorg);
                                break;
                            case(self::ID_REGISTRO_TRAILLER_LOTE):
                                self::leTrailerLote($linha);
                                break;
                            case(self::ID_REGISTRO_TRAILLER_ARQUIVO):
                                break;
                        }
                    }
                    $this->retornos[] = $this->retorno;
                }
            }
            self::preparaOcorrencias($this->cdbanco, $nrorg);

        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Busca Inscrição Empresa
     * @param $cdempresa
     * @param $nrorg
     * @return mixed
     * @throws \Exception
     */
    private function getInscricaoEmpresa($cdempresa, $nrorg){
        /** @var Empresa $empresa */
        $empresa = Empresa::findOneByCdempresaAndNrorg($cdempresa, $nrorg);
        if($empresa)
            return $empresa->getNrinsjurempr();
        else throw new \Exception ('Empresa não Encontrada.');
    }

    /**
     * // =====================================================================  //
     * // ====================  REGISTRO HEADER  DE LOTE   ====================  //
     * // =====================================================================  //
     * @param $header
     * @param $nrorg
     */
    private function leHeaderArquivo($header, $nrorg){
        
        // CAMPO: Código do Banco na compensação
        // POSIÇÃO: 001 a 003
        // PICTURE: 9(3)
        if(substr($header,0,3) != $this->cdbanco){
            self::gravaErros($this->contlinha,'Código do banco inválido.', true);
            parent::setStatusArquivo(false);
        }
        // CAMPO: Tipo de inscrição da empresa
        // POSIÇÃO: 18 a 18
        // PICTURE: 9(1)
        $tipoInscricao = substr($header,17,1);
        if(empty($tipoInscricao)){
            self::gravaErros($this->contlinha,'Tipo de Inscrição da Empresa não Informado.', true);
            parent::setStatusArquivo(false);
        }
        $nrinsjurempr = self::getInscricaoEmpresa($this->cdempresa, $nrorg);
        
        // CAMPO: Número de inscrição da empresa
        // POSIÇÃO: 19 a 32
        // PICTURE: 9(14)
        if($tipoInscricao == self::TIPO_INS_JUR_CPF){
            $inscricao = substr($header,21,11);
        }else{
            $inscricao = substr($header,18,14);
        }
        
        if($nrinsjurempr <> $inscricao){
            self::gravaErros($this->contlinha,'Inscrição da empresa não cadastrada.', true);
            parent::setStatusArquivo(false);
        }

        // CAMPO: Código de Remessa/Retorno
        // POSIÇÃO: 143 a 143
        // PICTURE: 9(1)
        if(substr($header,142,1) != self::RETORNO){
            self::gravaErros($this->contlinha,'Código de Retorno Inválido.', true);
            parent::setStatusArquivo(false);
        }
    }

    /**
     * // =====================================================================  //
     * // ====================      REGISTRO DETALHE       ====================  //
     * // =====================================================================  //
     * @param $header
     */
    private function leHeaderLote($header){
	
		// CAMPO: Agência debitada
		// POSIÇÃO: 53 a 57
		// PICTURE: 9(5)
        $this->cdagencia = substr($header,52,5);
        
        // CAMPO: Conta debitada
		// POSIÇÃO: 59 a 70
		// PICTURE: 9(12)
        $this->cdconta = substr($header,58,12);
        
        // CAMPO: Ocorrências
        // POSIÇÃO: 231 a 240
        // PICTURE: X(10)
        $this->ocorrenciaHeader = substr($header,230,10);
        
        $this->contHeaderLote++;
    }

    /**
     * // =====================================================================  //
     * // ====================      REGISTRO DETALHE       ====================  //
     * // =====================================================================  //
     * @param $detalhe
     */
    private function leDetalhe($detalhe, $nrorg){

        $segmento = substr($detalhe,13,1);
        $segmentoj52 = ((substr($detalhe,17,2)) == '52') ? true : false;
        
        $this->retorno->setIdheaderlote($this->contHeaderLote);
        
        switch($segmento){
            case self::SEGMENTO_A: $this->segmentoA($detalhe);
                break;
            case self::SEGMENTO_J: $this->segmentoJ($detalhe);
                break;
            case self::SEGMENTO_O: $this->segmentoO($detalhe);
                break;
        }
        
        if(($segmento == self::SEGMENTO_A) or (($segmento == self::SEGMENTO_J) and !$segmentoj52) or ($segmento == self::SEGMENTO_O)){
            
            $this->retorno->setCdlinha(str_pad($this->contlinha,10,'0',STR_PAD_LEFT));
            $this->retorno->setCdfilial($this->cdfilial);
            $this->retorno->setNrlanbolpag($this->nrlanbolpag);
            $this->retorno->setBoleto('BOLETO');
            $this->retorno->setCdconta($this->cdconta);
            $this->retorno->setCdagencia($this->cdagencia);
            $this->retorno->setDtvencimento($this->vencimento);
            
            $this->retorno->setCdtpbaixapag(self::getTipoBaixaPagamento($this->cdfilial, $this->nrlanbolpag, $nrorg));
            $this->retorno->setCdreducont(self::getReducont($this->cdfilial, $this->nrlanbolpag, $nrorg));
            $boletpagar = Boletpagar::findOneByCdfilialAndNrlanbolpagAndNrorg($this->cdfilial, $this->nrlanbolpag, $nrorg);
            $valorFinalPagoBoleto = ($boletpagar) ? $boletpagar->getVrfinalbpag() : 0;
            $this->retorno->setVrfinalpago($valorFinalPagoBoleto);
    
            // CAMPO: Tipo de movimento   [0 - Inclusão  9 - Exclusão]
            // POSIÇÃO: 15 a 17
            // PICTURE: 9(3)
            $tipomovimento = substr($detalhe,14,3);
            self::tipoMovimento($tipomovimento);
    
            // CAMPO: Ocorrência
            // POSIÇÃO: 231 a 240
            // PICTURE: X(10)
            $ocorrencia = substr($detalhe,230,10);
            $this->retorno->setOcorrencia($ocorrencia);
            if(!$ocorrencia or $ocorrencia == ' '){
                self::gravaErros($this->contlinha,'Ocorrência não informada.');
                parent::setStatusArquivo(false);
            }
        }
    }

    /**
     * Segmento A
     * @param $detalhe
     */
    public function segmentoA($detalhe){
        $this->cdfilial    = substr($detalhe,73,4);
        $this->nrlanbolpag = substr($detalhe,77,10);
        $this->retorno->setNossonumero(substr($detalhe,134,20));
        $this->vencimento  = substr($detalhe,93,8);

        // CAMPO: Data do pagamento
        // POSIÇÃO: 155 a 162
        // PICTURE: 9(8)
		if ((substr($detalhe,154,8)) != '00000000')
			$this->retorno->setDtpagamento(substr($detalhe,154,8));
		else $this->retorno->setDtpagamento('');

        // CAMPO: Valor do pagamento
        // POSIÇÃO: 163 a 177
        // PICTURE: 9(13)V9(2)
        $varpago = floatval(substr($detalhe,162,15));
        $this->retorno->setVrpago(round($varpago/100,2));
    }

    /**
     * Segmento J
     * @param $detalhe
     */
    public function segmentoJ($detalhe){
		$this->cdfilial    = substr($detalhe,182,4);
        $this->nrlanbolpag = substr($detalhe,186,10);
		$this->retorno->setNossonumero(substr($detalhe,202,15));
        $this->vencimento = substr($detalhe,91,8);

        /// CAMPO: Valor do deconto + Abatimento
        // POSIÇÃO: 115 a 129
        // PICTURE: 9(13)V9(2)
        $valordesconto = floatval(substr($detalhe,114,15));
        $this->retorno->setVrdesconto(round($valordesconto/100,2));
		
        // CAMPO: Valor da mora + Multa
        // POSIÇÃO: 130 a 144
        // PICTURE: 9(13)V9(2)
        $valoracrescimo = floatval(substr($detalhe,129,15));
        $this->retorno->setVracrescimo(round($valoracrescimo/100,2));
		
        // CAMPO: Data do pagamento
        // POSIÇÃO: 145 a 152
        // PICTURE: 9(8)
        $this->retorno->setDtpagamento(substr($detalhe,144,8));
		
        // CAMPO: Valor do pagamento
        // POSIÇÃO: 153 a 167
        // PICTURE: 9(13)V9(2)
        $varpago = floatval(substr($detalhe,152,15));
        $this->retorno->setVrpago(round($varpago/100,2));
    }

    /**
     * Segmento O
     * @param $detalhe
     */
	public function segmentoO($detalhe){
		$this->cdfilial    = substr($detalhe,122,4);
		$this->nrlanbolpag = substr($detalhe,126,10);
		$this->retorno->setNossonumero('');
		$this->vencimento = substr($detalhe,91,8);
	
		// CAMPO: Data do Pagamento
		// POSIÇÃO: 100 a 107
		// PICTURE: 9(8)
		$this->retorno->setDtpagamento(substr($detalhe,99,8));

		// CAMPO: Valor do Pagamento
		// POSIÇÃO: 108 a 122
		// PICTURE: 9(13)V9(2)
		$varpago = floatval(substr($detalhe,107,15));
        $this->retorno->setVrpago(round($varpago/100,2));

		// CAMPO: Nro Documento Banco
		// POSIÇÃO: 143 a 162
		// PICTURE: X(20)
		$nrret = substr($detalhe,142,20);
        if(empty($nrret))
            self::gravaErros($this->contlinha,'Número do Retorno não informado.');
	}
	
    /**
     * Lê Trailer Lote
     * @param $trailerlote
     */
    public function leTrailerLote($trailerlote){

        // CAMPO: Ocorrências
        // POSIÇÃO: 231 a 240
        // PICTURE: X(10)
        $ocorrenciastrailer = trim(substr($trailerlote,230,10));
        
        /** @var $retorno Retorno  */
        foreach($this->retornos as &$retorno){

            if($retorno->getIdheaderlote() == $this->contHeaderLote){

                //Insere Situação do Cnab
                $retorno->setIdsitucnabp(self::situacaoCnab($retorno->getOcorrencia().
                    $ocorrenciastrailer.
                    $this->ocorrenciaHeader));
                    
                //Adiciona as ocorrencias do trailler e do header.
                $retorno->setOcorrencia($retorno->getOcorrencia().
                    $ocorrenciastrailer.
                    $this->ocorrenciaHeader);
            }
        }
    }

    /**
	 * Verifica Situação CNAB
     * @param $cdocorrencia
     * @return string
     */
    private function situacaoCnab($cdocorrencia){
        if ($cdocorrencia){
            if (strpos($cdocorrencia,'BD') !== false)
                return '20'; // Retorno Ok (Agendado)
            if ((strpos($cdocorrencia,'BF') !== false) or
                (strpos($cdocorrencia,'BP') !== false) or
                (strpos($cdocorrencia,'BS') !== false))
                return '00'; // Não Agendado
            if ((strpos($cdocorrencia,'00') !== false) or
				(strpos($cdocorrencia,'BQ') !== false))
                return '90'; // Baixado via CNAB
            if ((strpos($cdocorrencia,'01') !== false) or 
                (strpos($cdocorrencia,'02') !== false) or 
                (strpos($cdocorrencia,'03') !== false) or 
                (strpos($cdocorrencia,'AA') !== false) or 
                (strpos($cdocorrencia,'AB') !== false) or 
                (strpos($cdocorrencia,'AC') !== false) or 
                (strpos($cdocorrencia,'AD') !== false) or 
                (strpos($cdocorrencia,'AE') !== false) or 
                (strpos($cdocorrencia,'AF') !== false) or 
                (strpos($cdocorrencia,'AG') !== false) or 
                (strpos($cdocorrencia,'AH') !== false) or 
                (strpos($cdocorrencia,'AI') !== false) or 
                (strpos($cdocorrencia,'AJ') !== false) or 
                (strpos($cdocorrencia,'AK') !== false) or 
                (strpos($cdocorrencia,'AL') !== false) or 
                (strpos($cdocorrencia,'AM') !== false) or 
                (strpos($cdocorrencia,'AN') !== false) or 
                (strpos($cdocorrencia,'AO') !== false) or 
                (strpos($cdocorrencia,'AP') !== false) or 
                (strpos($cdocorrencia,'AQ') !== false) or 
                (strpos($cdocorrencia,'AR') !== false) or 
                (strpos($cdocorrencia,'AS') !== false) or 
                (strpos($cdocorrencia,'AT') !== false) or 
                (strpos($cdocorrencia,'AU') !== false) or 
                (strpos($cdocorrencia,'AV') !== false) or 
                (strpos($cdocorrencia,'AW') !== false) or 
                (strpos($cdocorrencia,'AX') !== false) or 
                (strpos($cdocorrencia,'AY') !== false) or 
                (strpos($cdocorrencia,'AZ') !== false) or 
                (strpos($cdocorrencia,'BA') !== false) or 
                (strpos($cdocorrencia,'BB') !== false) or 
                (strpos($cdocorrencia,'BC') !== false) or 
                (strpos($cdocorrencia,'BG') !== false) or 
                (strpos($cdocorrencia,'BH') !== false) or 
                (strpos($cdocorrencia,'BI') !== false) or 
                (strpos($cdocorrencia,'BJ') !== false) or 
                (strpos($cdocorrencia,'BK') !== false) or 
                (strpos($cdocorrencia,'BL') !== false) or 
                (strpos($cdocorrencia,'BM') !== false) or 
                (strpos($cdocorrencia,'CA') !== false) or 
                (strpos($cdocorrencia,'CB') !== false) or 
                (strpos($cdocorrencia,'CC') !== false) or 
                (strpos($cdocorrencia,'CD') !== false) or 
                (strpos($cdocorrencia,'CE') !== false) or 
                (strpos($cdocorrencia,'CF') !== false) or 
                (strpos($cdocorrencia,'CG') !== false) or 
                (strpos($cdocorrencia,'CH') !== false) or 
                (strpos($cdocorrencia,'CI') !== false) or 
                (strpos($cdocorrencia,'CJ') !== false) or 
                (strpos($cdocorrencia,'CK') !== false) or 
                (strpos($cdocorrencia,'CL') !== false) or 
                (strpos($cdocorrencia,'CM') !== false) or 
                (strpos($cdocorrencia,'CN') !== false) or 
                (strpos($cdocorrencia,'CO') !== false) or 
                (strpos($cdocorrencia,'CP') !== false) or 
                (strpos($cdocorrencia,'HA') !== false) or 
                (strpos($cdocorrencia,'HB') !== false) or 
                (strpos($cdocorrencia,'HC') !== false) or 
                (strpos($cdocorrencia,'HD') !== false) or 
                (strpos($cdocorrencia,'HE') !== false) or 
                (strpos($cdocorrencia,'HF') !== false) or 
                (strpos($cdocorrencia,'HG') !== false) or 
                (strpos($cdocorrencia,'HH') !== false) or 
                (strpos($cdocorrencia,'HI') !== false) or 
                (strpos($cdocorrencia,'HJ') !== false) or 
                (strpos($cdocorrencia,'HK') !== false) or 
                (strpos($cdocorrencia,'HL') !== false) or 
                (strpos($cdocorrencia,'HM') !== false) or 
                (strpos($cdocorrencia,'HN') !== false) or 
                (strpos($cdocorrencia,'HO') !== false) or 
                (strpos($cdocorrencia,'HP') !== false) or 
                (strpos($cdocorrencia,'HQ') !== false) or 
                (strpos($cdocorrencia,'HR') !== false) or 
                (strpos($cdocorrencia,'HS') !== false) or 
                (strpos($cdocorrencia,'HT') !== false) or 
                (strpos($cdocorrencia,'HU') !== false) or 
                (strpos($cdocorrencia,'HV') !== false) or 
                (strpos($cdocorrencia,'HW') !== false) or 
                (strpos($cdocorrencia,'HX') !== false) or 
                (strpos($cdocorrencia,'HY') !== false) or 
                (strpos($cdocorrencia,'HZ') !== false) or 
                (strpos($cdocorrencia,'H1') !== false) or 
                (strpos($cdocorrencia,'H2') !== false) or 
                (strpos($cdocorrencia,'H3') !== false) or 
                (strpos($cdocorrencia,'H4') !== false) or 
                (strpos($cdocorrencia,'H5') !== false) or 
                (strpos($cdocorrencia,'H6') !== false) or 
                (strpos($cdocorrencia,'H7') !== false) or 
                (strpos($cdocorrencia,'H8') !== false) or 
                (strpos($cdocorrencia,'H9') !== false) or 
                (strpos($cdocorrencia,'IA') !== false) or 
                (strpos($cdocorrencia,'IB') !== false) or 
                (strpos($cdocorrencia,'IC') !== false) or 
                (strpos($cdocorrencia,'ID') !== false) or 
                (strpos($cdocorrencia,'IE') !== false) or 
                (strpos($cdocorrencia,'IF') !== false) or 
                (strpos($cdocorrencia,'IG') !== false) or 
                (strpos($cdocorrencia,'IH') !== false) or 
                (strpos($cdocorrencia,'II') !== false) or 
                (strpos($cdocorrencia,'IJ') !== false) or 
                (strpos($cdocorrencia,'IK') !== false) or 
                (strpos($cdocorrencia,'IL') !== false) or 
                (strpos($cdocorrencia,'IM') !== false) or 
                (strpos($cdocorrencia,'IN') !== false) or 
                (strpos($cdocorrencia,'IO') !== false) or 
                (strpos($cdocorrencia,'IP') !== false) or 
                (strpos($cdocorrencia,'IQ') !== false) or 
                (strpos($cdocorrencia,'IR') !== false) or 
                (strpos($cdocorrencia,'TA') !== false) or 
                (strpos($cdocorrencia,'YA') !== false) or 
                (strpos($cdocorrencia,'YB') !== false) or 
                (strpos($cdocorrencia,'YC') !== false) or 
                (strpos($cdocorrencia,'YD') !== false) or 
                (strpos($cdocorrencia,'YE') !== false) or 
                (strpos($cdocorrencia,'YF') !== false) or 
                (strpos($cdocorrencia,'ZA') !== false) or 
                (strpos($cdocorrencia,'ZB') !== false) or 
                (strpos($cdocorrencia,'ZD') !== false) or 
                (strpos($cdocorrencia,'ZE') !== false) or 
                (strpos($cdocorrencia,'ZF') !== false) or 
                (strpos($cdocorrencia,'ZG') !== false) or 
                (strpos($cdocorrencia,'ZH') !== false) or 
                (strpos($cdocorrencia,'ZI') !== false) or 
                (strpos($cdocorrencia,'ZJ') !== false) or 
                (strpos($cdocorrencia,'PB') !== false) or 
                (strpos($cdocorrencia,'PC') !== false) or 
                (strpos($cdocorrencia,'PD') !== false) or 
                (strpos($cdocorrencia,'PE') !== false) or 
                (strpos($cdocorrencia,'PF') !== false) or 
                (strpos($cdocorrencia,'PG') !== false) or 
                (strpos($cdocorrencia,'PH') !== false) or 
                (strpos($cdocorrencia,'PI') !== false) or 
                (strpos($cdocorrencia,'PJ') !== false) or 
                (strpos($cdocorrencia,'PK') !== false) or 
                (strpos($cdocorrencia,'PL') !== false) or 
                (strpos($cdocorrencia,'PM') !== false) or 
                (strpos($cdocorrencia,'PN') !== false))
                 return '01'; // Retorno com erro
        }
        return '10'; //Sem retorno
    }

}