<?php
namespace Teknisa\FIN\Modules\ContaReceber\Services\Pix\Banco;

use Teknisa\FIN\Modules\ContaReceber\Services\Pix\Util\ApiItau;
use Teknisa\FIN\Modules\ContaReceber\Services\Pix\Util\Payload;
use Doctrine\ORM\EntityManager;

use Mpdf\QrCode\QrCode;
use Mpdf\QrCode\Output;

class GeraPixItauService {
    
    function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
    }
    
    /**
     * Cria Cobrança Vencimento Pix
     * 
     * Aqui você pode tratar o resultado da criação da cobrança,
     * como exibir o QR Code para o pagador.
     */
    public function criaCobrancaVPix($cdclientid, $cdclientsecret, $dsdircertificado, $cdsenhacertif, $chavepix, $txtid, $vencimento, $validade, $cliente, $inscricao, $valor, $descricao, $homologacao = true){
        try {
            
            // Criação da Requisição
            $request = [
            	"calendario" => [
            		"dataDeVencimento" => $vencimento,
            		"validadeAposVencimento" => $validade
            	],
                "valor" => [
                    "original" => $valor
                ],
                "chave" => $chavepix,
                "solicitacaoPagador" => $descricao,
            ];
            
            // Verifica Tipo de Inscrisção do Devedor
            if (strlen($inscricao) == '14'){
                $request["devedor"] = [
                    "cnpj" => $inscricao,
                    "nome" => $cliente
                ];
            }else{
                $request["devedor"] = [
                    "cpf" => $inscricao,
                    "nome" => $cliente
                ];
            }
            
            // Construção do Objeto API Itau
            $obApiPix = new ApiItau($cdclientid, $cdclientsecret, $dsdircertificado, $homologacao);

            // Resposta da Requisição de Criação
            $response = $obApiPix->createCobV($txtid, $request);
            
            // Verifica existência de Violação
            if(isset($response['violacoes'][0]['razao'])){
                var_dump('Operação bloqueada. Erro na geração do QR Code Pix: '.$response['violacoes'][0]['razao']);die;
            }
            
            // Retorno do Pix Copia e Cola
            $payloadQrCode = $response['pixCopiaECola'];
            
            return $payloadQrCode;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    
    
    /**
     * Cria Cobrança Vencimento Pix
     * 
     * Aqui você pode tratar o resultado da criação da cobrança,
     * como exibir o QR Code para o pagador.
     */
    public function alteraCobrancaVPix($cdclientid, $cdclientsecret, $dsdircertificado, $cdsenhacertif, $chavepix, $txtid, $vencimento, $validade, $cliente, $inscricao, $valor, $descricao, $homologacao = true, $cancelamento = false){
        try {
            // Verifica Tipo de Inscrisção do Devedor
            if (!$cancelamento){
                
                // Criação da Requisição
                $request = [
                	"calendario" => [
                		"dataDeVencimento" => $vencimento,
                		"validadeAposVencimento" => $validade
                	],
                    "valor" => [
                        "original" => $valor
                    ],
                    "chave" => $chavepix,
                    "solicitacaoPagador" => $descricao,
                ];
                
                // Verifica Tipo de Inscrisção do Devedor
                if (strlen($inscricao) == '14'){
                    $request["devedor"] = [
                        "cnpj" => $inscricao,
                        "nome" => $cliente
                    ];
                }else{
                    $request["devedor"] = [
                        "cpf" => $inscricao,
                        "nome" => $cliente
                    ];
                }
            }else{   
                $request = [
                    "status" => "REMOVIDA_PELO_USUARIO_RECEBEDOR"
                ];
            }
                
            // Construção do Objeto API Itau
            $obApiPix = new ApiItau($cdclientid, $cdclientsecret, $dsdircertificado, $homologacao);
            
            // Resposta da Requisição de Criação
            $response = $obApiPix->updateCobV($txtid, $request);
            
            // Verifica existência de Violação
            if(isset($response['violacoes'][0]['razao'])){
                var_dump('Operação bloqueada. Erro na geração do QR Code Pix: '.$response['violacoes'][0]['razao']);die;
            }
            
            // Retorno do Pix Copia e Cola
            $payloadQrCode = $response['pixCopiaECola'];
            
            return $payloadQrCode;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
}