<?php

namespace Teknisa\FIN\Modules\ContaReceber\Services\Pix\Util;

class ApiItau{

  /**
   * URL base do PSP
   * @var string
   */
  private $baseUrl;

  /**
   * Client ID do oAuth2 do PSP
   * @var string
   */
  private $clientId;

  /**
   * Client secret do oAuht2 do PSP
   * @var string
   */
  private $clientSecret;

  /**
   * Caminho absoluto até o arquivo do certificado
   * @var string
   */
  private $certificate;

  /**
   * Define os dados iniciais da classe
   * @param string $baseUrl
   * @param string $clientId
   * @param string $clientSecret
   * @param string $certificate
   */
  public function __construct($clientId,$clientSecret,$certificate,$homologacao = true){
    if($homologacao)
         $baseUrl = 'https://sandbox.devportal.itau.com.br';
    else $baseUrl = 'https://secure.api.itau';
    
    $this->baseUrl      = $baseUrl;
    $this->clientId     = $clientId;
    $this->clientSecret = $clientSecret;
    $this->certificate  = $certificate;
  }

  /**
   * Método responsável por criar uma cobrança imediata
   * @param  string $txid
   * @param  array $request
   * @return array
   */
  public function createCob($txid,$request){
    return $this->send('PUT','/v2/cob/'.$txid,$request);
  }


  /**
   * Método responsável por criar uma cobrança imediata
   * @param  string $txid
   * @param  array $request
   * @return array
   */
  public function createCobV($txid,$request){
    return $this->send('PUT','/cobv/'.$txid, $request);
  }


  /**
   * Método responsável por criar uma cobrança imediata
   * @param  string $txid
   * @param  array $request
   * @return array
   */
  public function updateCobV($txid,$request){
    return $this->send('PATCH','/itau-ep9-gtw-pix-recebimentos-ext-v2/v2/cobv/'.$txid, $request);
  }

  /**
   * Método responsável por criar uma cobrança imediata
   * @param  string $txid
   * @param  array $request
   * @return array
   */
  public function getCobV($txid,$request){
    return $this->send('GET','/itau-ep9-gtw-pix-recebimentos-ext-v2/v2/cobv/'.$txid, $request);
  }

  /**
   * Método responsável por consultar uma cobrança imediata
   * @param  string $txid
   * @return array
   */
  public function consultCob($txid){
    return $this->send('GET','/pix');
  }


  /**
   * Método responsável por obter o token de acesso às APIs Pix
   * @return string
   */
  private function getAccessToken(){
    //ENDPOINT COMPLETO
    //$endpoint = $this->baseUrl.'/api/oauth/jwt';
    //$endpoint = 'https://sandbox.devportal.itau.com.br/api/oauth/jwt';
    $endpoint = 'https://sts.itau.com.br/api/oauth/token'; // TO DO - LINK DIFERENTE PRO TOKEN
    
    //HEADERS
    $headers = [
        'Content-Type: application/x-www-form-urlencoded',
        'User-Agent: insomnia/2023.5.5'
    ];

    //CORPO DA REQUISIÇÃO
    $request = 'grant_type=client_credentials&client_id='.$this->clientId.'&client_secret='.$this->clientSecret;
    
    //CERTIFICADOS
    $path_certificado = sys_get_temp_dir().'/certificate.crt'; // TO DO - COLOCAR NO FILESERVER
    $path_private_key = sys_get_temp_dir().'/ARQUIVO_CHAVE_PRIVADA.key'; // TO DO - COLOCAR NO FILESERVER
    
    //CONFIGURAÇÃO DO CURL
    $curl = curl_init();
    curl_setopt_array($curl, array(
      CURLOPT_URL => $endpoint,
      CURLOPT_RETURNTRANSFER => true,
      CURLOPT_ENCODING => '',
      CURLOPT_MAXREDIRS => 10,
      CURLOPT_TIMEOUT => 0,
      CURLOPT_FOLLOWLOCATION => true,
      CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
      CURLOPT_CUSTOMREQUEST => 'POST',
      CURLOPT_POSTFIELDS => $request,
      CURLOPT_HTTPHEADER => $headers,
      
      CURLOPT_SSLCERT => $path_certificado,
      CURLOPT_SSLKEY => $path_private_key
    ));
    
    //EXECUTA O CURL
    $response = curl_exec($curl);
    curl_close($curl);

    //RESPONSE EM ARRAY
    $responseArray = json_decode($response,true);
    
    //RETORNA O ACCESS TOKEN
    return $responseArray['access_token'] ?? '';
  }


  /**
   * Método responsável por enviar requisições para o PSP
   * @param  string $method
   * @param  string $resource
   * @param  array  $request
   * @return array
   */
  private function send($method,$resource,$request = []){
    //ENDPOINT COMPLETO
    //$endpoint = $this->baseUrl.$resource;
    //$endpoint = 'https://secure.api.itau/pix_recebimentos/v2'.$resource;
    $endpoint = 'https://secure.api.itau/pix_recebimentos/v2/cob/88ba8ec675e044178d434908d9b2a30a';
     
     
    $request = [
        "valor" => [
            "original" => '1.12'
        ],
        "chave" => '28.468.289/0001-97'
    ];
    
    //HEADERS
    $headers = [
      'x-itau-apikey: '.$this->clientId,
      'Content-type: application/json',
      'Authorization: Bearer '.$this->getAccessToken()
    ];
    
    
    var_dump($this->getAccessToken());die;
    
    //CERTIFICADOS
    $path_certificado = sys_get_temp_dir().'/certificate.crt'; // TO DO - COLOCAR NO FILESERVER
    $path_private_key = sys_get_temp_dir().'/ARQUIVO_CHAVE_PRIVADA.key'; // TO DO - COLOCAR NO FILESERVER
    
    //CONFIGURAÇÃO DO CURL
    $curl = curl_init();
    curl_setopt_array($curl, array(
/*      CURLOPT_URL => $endpoint,
      CURLOPT_RETURNTRANSFER => true,
      CURLOPT_ENCODING => '',
      CURLOPT_MAXREDIRS => 10,
      CURLOPT_TIMEOUT => 0,
      CURLOPT_FOLLOWLOCATION => true,
      CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
      CURLOPT_CUSTOMREQUEST  => $method,
      CURLOPT_HTTPHEADER => array(
        'Authorization: Bearer '.$this->getAccessToken(),
        'User-Agent: insomnia/2023.5.5'
      ),
      CURLOPT_SSLCERT => $path_certificado,
      CURLOPT_SSLKEY => $path_private_key*/
      CURLOPT_URL => 'https://secure.api.itau/pix_recebimentos/v2/cob/88ba8ec675e044178d434908d9b2a123',
  CURLOPT_RETURNTRANSFER => true,
  CURLOPT_ENCODING => '',
  CURLOPT_MAXREDIRS => 10,
  CURLOPT_TIMEOUT => 0,
  CURLOPT_FOLLOWLOCATION => true,
  CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
  CURLOPT_CUSTOMREQUEST => 'PUT',
  CURLOPT_POSTFIELDS =>'{
  "valor": {
    "original": "1.11"
  },
    "chave": "28.468.289/0001-97"
  }',
  CURLOPT_HTTPHEADER => array(
    'x-itau-apikey: '.$this->clientId,
    'x-itau-correlationID: 2',
    'Content-type: application/json',
    'Accept: application/json',
    'Authorization: Bearer '.$this->getAccessToken()
  ),
  CURLOPT_SSLCERT => $path_certificado,
      CURLOPT_SSLKEY => $path_private_key
    ));
    
    // switch ($method) {
    //   case 'POST':
    //   case 'PUT':
    //   case 'PATCH':
    //     curl_setopt($curl,CURLOPT_POSTFIELDS,json_encode($request));
    //     break;
    // }

    //EXECUTA O CURL
    $response = curl_exec($curl);
    curl_close($curl);
    var_dump($response);die;
    
    //RESPONSE EM ARRAY
    $responseArray = json_decode($response,true);
    
    //RETORNA O ARRAY DA RESPOSTA
    return $responseArray;
  }

}