<?php

namespace Teknisa\FIN\Modules\ContaReceber\Services\Retorno\Banco;

use Financeiro\Entities\Boletreceber;
use Teknisa\FIN\Modules\ContaReceber\Services\Retorno\LeitorRetorno;
use Teknisa\FIN\Modules\ContaReceber\Services\Retorno\Retorno;

class LeitorRetornoBrasil extends LeitorRetorno{

    CONST ID_TP_ARQUIVO_HEADER_ARQUIVO = '0';
    CONST ID_TP_ARQUIVO_HEADER_LOTE = '1';
    CONST ID_TP_ARQUIVO_DETALHE = '3';
    CONST ID_TP_TRAILER_LOTE = '5';
    CONST ID_TP_TRAILER_ARQUIVO = '9';

    private $arquivo;
    /** @var $retorno Retorno */
    private $retorno;

    /**
     * @param $arquivo
     * @param $cdempresa
     * @param $cdbanco
     * @throws \Exception
     */
    public function __construct($arquivo, $cdempresa, $cdbanco, $nrorg){
        try {
            $this->cdbanco = $cdbanco;
            $this->arquivo = $arquivo;
            $this->cdempresa = $cdempresa;
            $this->nrorg = $nrorg;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getEstruturaArquivo(){
        return array(
            self::ID_TP_ARQUIVO_HEADER_ARQUIVO,
            self::ID_TP_ARQUIVO_HEADER_LOTE,
            self::ID_TP_ARQUIVO_DETALHE,
            self::ID_TP_TRAILER_LOTE,
            self::ID_TP_TRAILER_ARQUIVO
        );
    }

    public function leArquivo(){
        try {
            foreach ($this->arquivo as $linha) {
                $cont = $this->contlinha++;
                if ($linha){
                    $idsegmento = substr($linha, 13, 1);
                    if($idsegmento == 'U')
                        $cont = $cont - 1;
                    else
                        $this->retorno = new Retorno();

                    // Busca o Tipo de Registro do Arquivo
                    $idRegistro = substr($linha, 7, 1);
                    if(self::validaArquivo($linha, self::ID_TP_ARQUIVO_HEADER_ARQUIVO, self::getEstruturaArquivo(), $this->contlinha)) {
                        switch ($idRegistro) {

                            case(self::ID_TP_ARQUIVO_HEADER_ARQUIVO):
                                self::leHeaderArquivo($linha);
                                break;

                            case(self::ID_TP_ARQUIVO_HEADER_LOTE):
                                self::leHeaderLote($linha);
                                break;

                            case(self::ID_TP_ARQUIVO_DETALHE):
                                self::leDetalhe($linha);
                                break;

                            case(self::ID_TP_TRAILER_LOTE):
                                break;

                            case(self::ID_TP_TRAILER_ARQUIVO):
                                break;

                            default:
                                throw new \Exception('Arquivo inválido.');
                        }
                    }
                    $this->retornos[$cont] = $this->retorno;
                }
            }
            self::preparaOcorrencias($this->cdbanco, $this->nrorg);

        } catch (\Exception $e) {
            throw $e;
        }
    }

    //########################################################################
    //############# Identificação do Registro Header #########################
    //########################################################################
    private function leHeaderArquivo($header){

        // CAMPO: Código do BCO na compensação
        // POSIÇÃO: 1 a 3
        // PICTURE: 9(3)
        if(substr($header, 0, 3) != self::CODIGO_COMPENSACAO_BRASIL)
            self::gravaErros($this->contlinha, 'Banco Inválido.', true);

        // CAMPO: Identificação do Arquivo de Retorno
        // POSIÇÃO: 143 a 143
        // PICTURE: 9(1)
        if (substr($header, 142, 1) != '2'){
            self::gravaErros($this->contlinha, 'Arquivo inválido.', true);
        }
    }

    //########################################################################
    //############# Identificação do Registro Header de Lote #################
    //########################################################################
    private function leHeaderLote($header){

        // CAMPO: Tipo de Inscrição da empresa
        // POSIÇÃO: 018 a 018
        // PICTURE: 9(1)
        $tpinscricao = trim(substr($header, 17, 1));
        if (empty($tpinscricao)){
            self::gravaErros($this->contlinha, 'Tipo de inscrição da empresa inválido.', true);
        }


        // CAMPO: Inscrição da empresa
        // POSIÇÃO: 019 a 033
        // PICTURE: 9(15)
        $inscricao = trim(substr($header, 18, 15));
        if (empty($inscricao)){
            self::gravaErros($this->contlinha, 'Inscrição da empresa inválida.', true);
        }
    }

    //########################################################################
    //############# Identificação do Registro Detalhe ########################
    //########################################################################
    private function leDetalhe($detalhe){
        $validaOcorre = substr($detalhe, 15, 02);
        if($validaOcorre != '28'){
            //############# Identificação do Registro de Segmento T
            if(substr($detalhe, 13, 1) == 'T'){
                $cdfilial = substr($detalhe, 105, 4);
                $nrlanbolrec = substr($detalhe, 109, 10);
                $idtitulobanco = trim(substr($detalhe, 37, 20));
                $cdcartbol = trim(substr($detalhe, 223, 2));
                $cdempresa = $this->cdempresa;
                $nossonumero = Boletreceber::getRepository();
                /** @var $nossonumero \Financeiro\Repositories\Boletreceber */
                $cdnosnumrec = $nossonumero->getNossoNumero($idtitulobanco, $cdempresa);
                $boleto = false;
                if(!empty($cdfilial) && !empty($nrlanbolrec))
                    /** @var $boleto Boletreceber */
                    $boleto = Boletreceber::findOneByNrlanbolrecAndCdfilialAndNrorg($nrlanbolrec, $cdfilial, $this->nrorg);
                elseif($cdnosnumrec)
                    /** @var $boleto Boletreceber */
                    $boleto = Boletreceber::findOneByNrlanbolrecAndCdfilialAndNrorg(
                        $cdnosnumrec[0]['NRLANBOLREC'], $cdnosnumrec[0]['CDFILIAL'], $this->nrorg
                    );
                $this->retorno->setBoleto('BOLETO');
                $this->retorno->setCdlinha(str_pad($this->contlinha, 4, 0, STR_PAD_LEFT));
                $this->retorno->setCdfilial($cdfilial);
                $this->retorno->setNrlanbolrec($nrlanbolrec);
                $this->retorno->setNossonumero($idtitulobanco);
                $this->retorno->setCarteira($cdcartbol);
                $this->retorno->setCdagencia(trim(substr($detalhe, 17, 6)));
                $this->retorno->setCdconta(trim(substr($detalhe, 23, 13)));

                if($boleto) {
                    $this->retorno->setVrfinalbrec($boleto->getVrfinalbrec());
                    $this->retorno->setVrbolrec($boleto->getVrbolrec());
                    $this->retorno->setIdstatrec($boleto->getIdstatbrec());
                    $this->retorno->setCdfilial($boleto->getCdfilial());
                    $this->retorno->setNrlanbolrec($boleto->getNrlanbolrec());
                    $this->retorno->setCdtpbaixarec($boleto->getCdtpbaixarec());
                    $this->retorno->setCdreducont($boleto->getCdreducont());
                }

                // CAMPO: Identificação do título no banco
                // POSIÇÃO: 038 a 057
                // PICTURE: 9(20)
                if (empty($idtitulobanco))
                    $this->gravaErros($this->contlinha, 'Nosso número inválido.', true);

                // CAMPO: Identificação da Ocorrência no Banco
                // POSIÇÃO: 016 a 017
                // PICTURE: 9(2)
                if (empty($validaOcorre))
                    $this->gravaErros($this->contlinha, 'Ocorrência inválida.');
                else
                    $this->retorno->setIdsitucnabr(self::situacaoCnab($validaOcorre));

                // CAMPO: Data de Vencimento do título
                // POSIÇÃO: 074 a 081
                // PICTURE:
                $vencimento = substr($detalhe, 73, 8);
                $vencimento = substr($vencimento, 0, 2) . "/" . substr($vencimento, 2, 2) . "/" . substr($vencimento, -4);
                $this->retorno->setDtvencimento($vencimento);

                // CAMPO: Valor nominal do título
                // POSIÇÃO: 082 a 096
                // PICTURE: 9(13)V9(2)
                $this->retorno->setVrnominal(round(floatval(substr($detalhe, 81, 15)) / 100, 2));

                // CAMPO: Erros - Registros rejeitados ou alegação do sacado
                // POSIÇÃO: 214 a 223
                // PICTURE: X(10)
                $iderros = trim(substr($detalhe, 213, 10));

                if(self::situacaoCnab($validaOcorre) == '01'){
                    $this->retorno->setOcorrencia($iderros);
                }
            }
            else{
                // CAMPO: Desconto concedido
                // POSIÇÃO: 033 a 047
                // PICTURE: 9(13)V9(2)

                // CAMPO: Abatimento concedito sobre o título
                // POSIÇÃO: 048 a 062
                // PICTURE: 9(13)V9(2)

                // CAMPO: IOF Recolhido
                // POSIÇÃO: 063 a 077
                // PICTURE: 9(13)V9(2)

                // CAMPO: Valor de outras Despesas
                // POSIÇÃO: 108 a 122
                // PICTURE: 9(13)V9(2)
                $this->retorno->setVrdesconto(
                    round(floatval(substr($detalhe, 32, 15)) / 100, 2) +
                    round(floatval(substr($detalhe, 47, 15)) / 100, 2) +
                    round(floatval(substr($detalhe, 62, 15)) / 100, 2) +
                    round(floatval(substr($detalhe, 107, 15)) / 100, 2)
                );

                // CAMPO: Juros/Multa/Encargos
                // POSIÇÃO: 018 a 032
                // PICTURE: 9(13)V9(2)

                // CAMPO: Valor de outros Creditos
                // POSIÇÃO: 123 a 137
                // PICTURE: 9(13)V9(2)
                $this->retorno->setVracrescimo(
                    round(floatval(substr($detalhe, 17, 15)) / 100, 2) +
                    round(floatval(substr($detalhe, 122, 15)) / 100, 2)
                );

                // CAMPO: Valor Pago pelo Sacado
                // POSIÇÃO: 093 a 107
                // PICTURE: 9(13)V9(2)
                $this->retorno->setVrpago(
                    round(floatval(substr($detalhe, 77, 15)) / 100, 2)
                );

                $situacaoCnab = self::situacaoCnab($validaOcorre);
                if($situacaoCnab && $situacaoCnab == '90'){
                    // CAMPO: Data de efetivação do crédito
                    // POSIÇÃO: 146 a 153
                    // PICTURE: X(8)
                    /** @var $vencimento DateTime */
                    $pagamento = substr($detalhe, 145, 8);
                    $this->retorno->setDtpagamento($pagamento);
                }
            }
        }
    }

    /**
     * @param $cdocorrencia
     * @return string
     */
    private function situacaoCnab($cdocorrencia){
        if($cdocorrencia == '09' || $cdocorrencia == '28')
            return '00'; //Não Agendado
        else if($cdocorrencia == '02' or $cdocorrencia == '19' or $cdocorrencia == '23')
            return '20'; //Retorno OK - Agendado
        else if($cdocorrencia == '03' or $cdocorrencia == '26' or $cdocorrencia == '30')
            return '01'; //Retorno com Erro
        else if($cdocorrencia == '06' or $cdocorrencia == '17' or $cdocorrencia == '25')
            return '90'; //Baixado
        return '10'; //Sem retorno
    }

}