<?php
namespace Teknisa\FIN\Modules\MovimentacaoBancaria\Controllers;

use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO;
use Teknisa\FIN\Controllers\TeknisaCrud;
use Financeiro\Entities\Movbancario;
use Financeiro\Entities\Movcredbanc;
use Financeiro\Entities\Movfluxo;
use Zeedhi\Framework\DataSource\ParameterBag;
use Teknisa\FIN\Modules\MovimentacaoBancaria\Services\Bco11100LancamentoContaCorrenteService;
use Geral\Util\TratamentoDeMensagens;
use Teknisa\Libs\Util\DateUtil;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Message;

class Bco11100LancamentoContaCorrenteController extends TeknisaCrud
{
    const STATUS_CONCILIACAO = 0;
    const IDMOVFLUXO = 'R';
    protected $environment;
    protected $entityManager;
    protected $dataSourceName = 'movcredbanc_lancamentoContaCorrente';

    public function __construct($crudService, ParameterBag $parameterBag, $environment, Bco11100LancamentoContaCorrenteService $service, EntityManager $entityManager) {
        $this->environment = $environment;
        $this->service = $service;
        $this->entityManager = $entityManager;
        parent::__construct($crudService, $parameterBag);
    }

    public function find(DTO\Request\Filter $request, DTO\Response $response) {
        $dataSource = substr($request->getRoutePath(), 1, strlen($request->getRoutePath()));
        $this->dataSourceName = $dataSource;
        parent::find($request, $response);
    }

    public function getAdiantamentoCliente(DTO\Request\Row $request, DTO\Response $response){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $adiantamento = '';
            $row = $request->getRow();
            $adiantamento = $this->service->getAdiantamentoCliente($this->environment->getNrOrgTrab(), $row['CDEMPRESA'], $row['CDCLIENTE']);
            $response->addDataSet(new DataSet("ADIANTAMENTO", $adiantamento));
        } catch (\Exception $e) {
            $connection->rollback();
            $response->setError(new DTO\Response\Error($e->getMessage(), 500));
        }
    }

    public function getAdiantamentoFornecedor(DTO\Request\Row $request, DTO\Response $response){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $adiantamento = '';
            $row = $request->getRow();
            $adiantamento = $this->service->getAdiantamentoFornecedor($this->environment->getNrOrgTrab(), $row['CDEMPRESA'], $row['CDFORNECED'], $row['NRINSJURFORN']);
            $response->addDataSet(new DataSet("ADIANTAMENTO", $adiantamento));
        } catch (\Exception $e) {
            $connection->rollback();
            $response->setError(new DTO\Response\Error($e->getMessage(), 500));
        }
    }

    public function getDevolucaoCliente(DTO\Request\Row $request, DTO\Response $response){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $devolucao = '';
            $row = $request->getRow();
            $devolucao = $this->service->getDevolucaoCliente($this->environment->getNrOrgTrab(), $row['CDEMPRESA'], $row['CDCLIENTE']);
            $response->addDataSet(new DataSet("DEVOLUCAO", $devolucao));
        } catch (\Exception $e) {
            $connection->rollback();
            $response->setError(new DTO\Response\Error($e->getMessage(), 500));
        }
    }

    public function getDevolucaoFornecedor(DTO\Request\Row $request, DTO\Response $response){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $devolucao = '';
            $row = $request->getRow();
            $lancamentocc = isset($data['LANCAMENTOCC'])?$data['LANCAMENTOCC']:'N';
            $devolucao = $this->service->getDevolucaoFornecedor($this->environment->getNrOrgTrab(), $row['CDEMPRESA'], $row['CDFORNECED'], $lancamentocc);
            $response->addDataSet(new DataSet("DEVOLUCAO", $devolucao));
        } catch (\Exception $e) {
            $connection->rollback();
            $response->setError(new DTO\Response\Error($e->getMessage(), 500));
        }
    }

    public function saveLancamentoContaCorrente(DTO\Request $request, DTO\Response $response) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $rows = $request->getDataSet()->getRows();
            $rows[0] = self::preparaRow($rows[0]);
            $adiantamento = self::preparaRowAdiantamento($rows);
            $devolucao = self::preparaRowDevolucao($rows);
            $contabiliza = isset($rows[1]['contabiliza']) ? $rows[1]['contabiliza'] : true;
            
            if ($rows[0]['__is_new']) {
                $this->service->insereLancamento($rows[0], $adiantamento, $devolucao, $response, $contabiliza);
            }else {
                $this->service->updateLancamento($rows[0]);
            }
            $retorno = array(
                "NRLANCCONT" => null,
                "NRORG" => $this->environment->getNrOrgTrab(),
                "NRMOVBANC" => $this->service->nrmovbanc,
                "IDMOVFLUXO" => 'R',
                "CODIGOLANCAMENTO" => $this->service->codigoLancamento
            );
            $response->addDataSet(new DataSet("lancamento", $retorno));
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollback();
            $response->setError(new DTO\Response\Error($e->getMessage(), 500));
        }
    }

    public function deleteLancamentoContaCorrente(DTO\Request $request, DTO\Response $response) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $row = $request->getRow();
            $this->service->validaMovimentacaoConciliados($row);
            $this->service->deleteLancamento($row['NRLANCCONT'], $row['CDEMPRESA'], $row['CDREDUCONT'], $row['NRMOVBANC'],
                $row['NRORG'], $row['DEBCRED'], round($row['VRMOVICRED'], 2), self::formataData($row['DTMOVBANC']),
                $row['CDFILIAL'], $row['CDCLASSFINA'], $row['IDMOVFLUXO'],  $row['CDCLIENTE'], $row['CDFORNECED'], $row['CODIGOLANCAMENTO'], $row['NRCHEQUE'], $row['NRMOVBANCORIG']);
            $this->entityManager->flush();
            $connection->commit();
            $response->addMessage(new Message("Deletado com sucesso."));
        } catch (\Exception $e) {
            $connection->rollback();
            $response->setError(new DTO\Response\Error($e->getMessage(), 500));
        }
    }

    private function preparaRow($row){
        $row['NRORG'] = $this->environment->getNrOrgTrab();
        $row['IDSTCONEXT'] = self::STATUS_CONCILIACAO;
        $row['IDMOVFLUXO'] = self::IDMOVFLUXO;
        $row['VRMOVICRED'] = round($row['VRMOVICRED'], 2);
        $row['DTMOVBANC'] = self::formataData($row['DTMOVBANC']);
        $row['IDSTATUSCTBBANC'] = 'P';
        $row['IDLANCOMCONTPAG'] = 'N';
        return $row;
    }

    public function preparaRowAdiantamento($rows){
        $adiantamento = array();
        for($i = 1; count($rows) > 1 && $i < $rows[0]['QTDEADIANTAMENTO'] + 1; $i++){
            $adiantamento[] = $rows[$i];
        }
        return $adiantamento;
    }

    public function preparaRowDevolucao($rows){
        $devolucao = array();
        for($i = $rows[0]['QTDEADIANTAMENTO'] + 1; $i < count($rows); $i++){
            $devolucao[] = $rows[$i];
        }
        return $devolucao;
    }

    public function emissaoCheque(DTO\Request\Row $request, DTO\Response $response){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $row = $request->getRow();
            $row['DTEMISCHEQ'] = self::formataData($row['DTEMISCHEQ']);
            $this->service->emissaoCheque($row, $response);
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollback();
            $response->setError(new DTO\Response\Error($e->getMessage(), 500));
        }
    }

    public static function formataData($data){
        if(strlen($data) == 19)
            $data = DateUtil::truncateString($data);
        $data = DateUtil::getDatadeString($data);
        $data = DateUtil::truncateData($data);

        return $data;
    }
}