<?php
namespace Teknisa\FIN\Modules\MovimentacaoBancaria\Services;
use Doctrine\ORM\EntityManager;
use Teknisa\GER\Modules\Fiscal\Factory;
use Geral\Entities\Prazopgtclie;
use Financeiro\Entities\Sdoccorr;
use Teknisa\FIN\Modules\MovimentacaoBancaria\Factories\Bco11100LancamentoContaCorrenteFactory;
use Teknisa\FIN\Modules\FluxoCaixa\Services\FluxoDeCaixaService;
use Teknisa\FIN\Services\ContabilizacaoService;

class Bco11100LancamentoContaCorrenteService
{
    const DEBITO = 'D';
    const CREDITO = 'C';
    const ENTRADA = 'E';
    const SAIDA = 'S';
    protected $environment;
    protected $newCode;
    protected $entityManager;
    protected $factory;
    protected $fluxodecaixaservice;
    protected $contabilizacaoService;
    public $nradiantareceb;
    public $nradiantapagar;
    public $nrmovbanc;
    public $codigoLancamento;

    public function __construct($environment, $newCode, EntityManager $entityManager,
                                Bco11100LancamentoContaCorrenteFactory $factory,
                                FluxoDeCaixaService $fluxodecaixaservice,
                                ContabilizacaoService $contabilizacaoService) {
        $this->environment = $environment;
        $this->newCode = $newCode;
        $this->entityManager = $entityManager;
        $this->factory = $factory;
        $this->fluxodecaixaservice = $fluxodecaixaservice;
        $this->contabilizacaoService = $contabilizacaoService;
    }

    public function getAdiantamentoCliente($nrorg, $cdempresa, $cdcliente){
        return $this->factory->getAdiantamentoCliente($nrorg, $cdempresa, $cdcliente);
    }

    public function getAdiantamentoFornecedor($nrorg, $cdempresa, $cdforneced, $nrinsjurforn){
        return $this->factory->getAdiantamentoFornecedor($nrorg, $cdempresa, $cdforneced, $nrinsjurforn);
    }

    public function getDevolucaoCliente($nrorg, $cdempresa, $cdcliente){
        return $this->factory->getDevolucaoCliente($nrorg, $cdempresa, $cdcliente);
    }

    public function getDevolucaoFornecedor($nrorg, $cdempresa, $cdforneced, $lancamentocc = 'N'){
        return $this->factory->getDevolucaoFornecedor($nrorg, $cdempresa, $cdforneced, $lancamentocc);
    }

    public function insereLancamento($row, $adiantamento, $devolucao, $response, $contabiliza){
        try {
            $this->nrmovbanc = $this->newCode->getCodeByTableName('MOVBANCARIO'.$row['CDREDUCONT'],$this->environment->getNrOrgTrab());
            $movbancario = $this->factory->insereMovbancario($row['CDREDUCONT'], $this->nrmovbanc, $row['DTMOVBANC'], $row['DEBCRED'], $row['NRDOCMBANC'], $row['DESCRICAO'],
                                      $row['VRMOVICRED'], $row['IDSTCONEXT'], $row['IDSTATUSCTBBANC'], $row['NRORG']);
            $this->entityManager->persist($movbancario);
            $this->entityManager->flush();
            
            if ($row['DEBCRED'] == 'C') {
                $row['IDOPERACAO'] = 'E';
                $row = self::preparaAtualizacaoFluxo($row);
                $movfluxo = $this->fluxodecaixaservice->atualizaFluxoCaixa($row['NRORG'], $row['CDEMPRESA'], $row['CDFILIAL'], $row['CDCLASSFINA'],
                    $row['DTMOVBANC'], $row['VRMOVICRED'], $row['IDMOVFLUXO'], self::ENTRADA);
                $this->entityManager->persist($movfluxo);
                $this->entityManager->flush();

                $this->geraCodigo('MOVCREDBANC' . $row['CDREDUCONT'], 'NRMOVICRED', $row);
                $this->codigoLancamento = $row['NRMOVICRED'];
                
                //inserido verificação para updatar o campo NRLCTBMCRE ou o NRLCTBMDEB
                if(isset($row["NRLANCCONT"])){
                    $nrlctbmcre = $row["NRLANCCONT"];
                } else {
                    $nrlctbmcre = null;
                }
                
                $movcredbanc = $this->factory->insereMovcredbanc($row['CDREDUCONT'], $row['NRMOVICRED'], $row['CDCLASSFINA'], $row['CDEMPRESA'], $row['CDFILIAL'],
                    $row['DTMOVBANC'], $row['IDMOVFLUXO'], $this->nrmovbanc, $row['CDTPCREDBANC'], $row['DTMOVBANC'],
                    $row['DESCRICAO'], $row['VRMOVICRED'], $row['CDEMPRESA'], $row['CDFILIAL'], $row['NRORG'],
                    $row['CDCLIENTE'], $row['CDFORNECED'], $nrlctbmcre);
                $this->entityManager->persist($movcredbanc);
                $this->entityManager->flush();

                $this->fluxodecaixaservice->atualizaSaldoContCorr($row['CDREDUCONT'], $row['DTMOVBANC'], $row['NRORG'], $row['VRMOVICRED']);

                self::insereAdiantamento($adiantamento, $row, 'C', $row['NRMOVICRED']);
                self::insereDevolucao($devolucao, $row, 'C', $row['NRMOVICRED']);
                
                if($contabiliza == true){
                    $this->contabilizacaoService->contabilizaLancamentoControleBancario($row['CDEMPRESA'], $row['CDREDUCONT'], $row['NRMOVICRED'], self::CREDITO, $row['IDCTBLCRBANC']);
                }
            } else {
                $row['IDOPERACAO'] = 'S';
                $row = self::preparaAtualizacaoFluxo($row);
                $movfluxo = $this->fluxodecaixaservice->atualizaFluxoCaixa($row['NRORG'], $row['CDEMPRESA'], $row['CDFILIAL'], $row['CDCLASSFINA'],
                    $row['DTMOVBANC'], $row['VRMOVICRED'], $row['IDMOVFLUXO'], self::SAIDA);
                $this->entityManager->persist($movfluxo);
                $this->entityManager->flush();
                $this->geraCodigo('MOVDEBBANC' . $row['CDREDUCONT'], 'NRMOVIDEBI', $row);
                $this->codigoLancamento = $row['NRMOVIDEBI'];
                
                //inserido verificação para updatar o campo NRLCTBMDEB
                if(isset($row["NRLANCCONT"])){
                    $nrlctbmdeb = $row["NRLANCCONT"];
                } else {
                    $nrlctbmdeb = null;
                }
                
                $movdebbanc = $this->factory->insereMovdebbanc($row['CDREDUCONT'], $row['NRMOVIDEBI'], $row['CDCLASSFINA'], $row['CDEMPRESA'], $row['CDFILIAL'],
                    $row['DTMOVBANC'], $row['IDMOVFLUXO'], $this->nrmovbanc, $row['CDTPDEBIBANC'], $row['DTMOVBANC'],
                    $row['DESCRICAO'], $row['VRMOVICRED'], $row['CDEMPRESA'], $row['CDFILIAL'], $row['NRORG'],
                    $row['CDCLIENTE'], $row['CDFORNECED'], $row['IDLANCOMCONTPAG'], $nrlctbmdeb);
                $this->entityManager->persist($movdebbanc);
                $this->entityManager->flush();

                $this->fluxodecaixaservice->atualizaSaldoContCorr($row['CDREDUCONT'], $row['DTMOVBANC'], $row['NRORG'], $row['VRMOVICRED'] * -1);
                self::insereAdiantamento($adiantamento, $row, 'D', $row['NRMOVIDEBI']);
                self::insereDevolucao($devolucao, $row, 'D', $row['NRMOVIDEBI']);
                
                if($contabiliza == true){
                    $this->contabilizacaoService->contabilizaLancamentoControleBancario($row['CDEMPRESA'], $row['CDREDUCONT'], $row['NRMOVIDEBI'], self::DEBITO, $row['IDCTBLDBBANC']);
                }
            }
        } catch (\Exception $e) {
            $this->contabilizacaoService->saveContabilizacaoData($response);
            throw $e;
        }
    }

    public function insereAdiantamento($adiantamento, $row, $tipoLancamento, $numeroLancamento){
        if ($row['CDCLIENTE']){
            for($i = 0; $i < count($adiantamento); $i++) {
                if(isset($adiantamento[$i]['CDFILIAL'])){
                    $this->nradiantareceb = $this->newCode->getCodeByTableName('ADIANTARECEB',$this->environment->getNrOrgTrab());
                    $descontoAdiantamento = $this->factory->insereAdiantamentoCliente($this->nradiantareceb, $row['CDREDUCONT'], $numeroLancamento, $adiantamento[$i]['CDFILIAL'], $adiantamento[$i]['NRLANCTONF'],
                        null, $adiantamento[$i]['NRSEQVENREC'], $adiantamento[$i]['NRPARCELAREC'], $adiantamento[$i]['VRDESCONTO'], $tipoLancamento, $row['NRORG']);
                    $this->entityManager->persist($descontoAdiantamento);
                }
            }
        }else if ($row['CDFORNECED']){
            for($i = 0; $i < count($adiantamento); $i++) {
                if(isset($adiantamento[$i]['CDFILIAL'])){
                    $this->nradiantapagar = $this->newCode->getCodeByTableName('ADIANTAPAGAR',$this->environment->getNrOrgTrab());
                    $descontoAdiantamento = $this->factory->insereAdiantamentoFornecedor($this->nradiantapagar, $row['CDREDUCONT'], $numeroLancamento, $adiantamento[$i]['CDFILIAL'], $adiantamento[$i]['NRLANCTONF'],
                        null, $adiantamento[$i]['NRSEQVENPAG'], $adiantamento[$i]['NRPARCELAPAG'], $adiantamento[$i]['VRDESCONTO'], $tipoLancamento, $row['NRORG']);
                    $this->entityManager->persist($descontoAdiantamento);
                }
            }
        }
    }

    public function insereDevolucao($devolucao, $row, $tipoLancamento, $numeroLancamento){
        $executarProcedure = 'N';
        if ($row['CDCLIENTE']){
            for($i = 0; $i < count($devolucao); $i++) {
                if(isset($devolucao[$i]['FILIDEVO'])){
                    $notafiscal = $this->factory->updateNotaFiscal($row['NRORG'],  $devolucao[$i]['FILIDEVO'], $devolucao[$i]['NRNFDEVO'], $devolucao[$i]['VRDESCONTO']);
                    $this->entityManager->persist($notafiscal);
                    $this->geraCodigo('DEVOLUCAOREC', 'NRDEVOLUCAOREC', $devolucao[$i]);
                    $descontoDevolucao = $this->factory->insereDevolucaoCliente($devolucao[$i]['NRDEVOLUCAOREC'], $row['CDREDUCONT'], $numeroLancamento, $devolucao[$i]['FILIDEVO'], $devolucao[$i]['NRNFDEVO'],
                        $devolucao[$i]['VRDESCONTO'], $tipoLancamento, $row['NRORG']);
                    $this->entityManager->persist($descontoDevolucao);
                    $executarProcedure = 'S';
                }
            }
        }else if ($row['CDFORNECED']){
            for($i = 0; $i < count($devolucao); $i++) {
                if(isset($devolucao[$i]['FILIDEVO'])){
                    $notafiscal = $this->factory->updateNotaFiscal($row['NRORG'],  $devolucao[$i]['FILIDEVO'], $devolucao[$i]['NRNFDEVO'], $devolucao[$i]['VRDESCONTO']);
                    $this->entityManager->persist($notafiscal);
                    $this->geraCodigo('DEVOLUCAOPAG', 'NRDEVOLUCAOPAG', $devolucao[$i]);
                    $descontoDevolucao = $this->factory->insereDevolucaoFornecedor($devolucao[$i]['NRDEVOLUCAOPAG'], $row['CDREDUCONT'], $numeroLancamento, $devolucao[$i]['FILIDEVO'], $devolucao[$i]['NRNFDEVO'],
                        $devolucao[$i]['VRDESCONTO'], $tipoLancamento, $row['NRORG']);
                    $this->entityManager->persist($descontoDevolucao);
                    $executarProcedure = 'S';
                }
            }
        }
        if($executarProcedure == 'S'){
            $this->factory->procedureGravaImpostoBaixa($row['CDREDUCONT'], $row['NRMOVICRED'], 'C');
        }
    }

    // Atualiza lançamento
    public function updateLancamento($row){
        try{
            $movbancario = $this->factory->getMovbancario($row['NRORG'], $row['NRMOVBANC'], $row['CDREDUCONT']);
            $movbancario = $this->factory->updateMovbancario($movbancario, $row['NRDOCMBANC'], $row['DESCRICAO']);
            $this->entityManager->persist($movbancario);
            if ($row['DEBCRED'] == 'C') {
                $movcredbanc = $this->factory->updateMovcredbanc($row['NRORG'], $row['CODIGOLANCAMENTO'], $row['DESCRICAO'], $row['CDREDUCONT']);
                $this->entityManager->persist($movcredbanc);
            }else {
                $movdebbanc = $this->factory->updateMovdebbanc($row['NRORG'], $row['CODIGOLANCAMENTO'], $row['DESCRICAO'], $row['CDREDUCONT']);
                $this->entityManager->persist($movdebbanc);
            }
        }catch(\Exception $e){
            throw $e;
        }
    }

    public function validaMovimentacaoConciliados($row){
        try{
            $movBancario = $this->factory->getMovbancario($row['NRORG'], $row['NRMOVBANC'], $row['CDREDUCONT']);
            if (($movBancario->getNrseqmovreba() && $movBancario->getCdsequexba()) || ($movBancario->getNrconcimovb())){
                throw new \Exception('Exclusão não permitida! Movimento bancário do lancamento foi conciliado junto com outros movimentos bancários.');
            }
        }catch(\Exception $e){
            Throw $e;
        }
    }

    public function atualizaMovimentacaoBancariaOrigem($nrorg, $cdreducont, $nrmovbancorig, $vrmovicred, $tipoLancamento){
        try{
            // Se existir Movimentação Bancária de Origem, ele irá atualiza/excluir a mesma
            if($nrmovbancorig){
                $movBancario = $this->factory->getMovbancario($nrorg, $nrmovbancorig, $cdreducont);
            
                $vrmovbanc = $movBancario->getVrmovbanc() - $vrmovicred;
                
                if ($vrmovbanc > 0){
                    $movBancario->setVrmovbanc($vrmovbanc);
                    $this->entityManager->persist($movBancario);
                }else{
                    $this->entityManager->remove($movBancario);
                }
                
                $vrmovicred = $tipoLancamento == 'C'?$vrmovicred : $vrmovicred * -1;
                
                // Atualiza Saldo Conta Corrente
                $this->fluxodecaixaservice->atualizaSaldoContCorr(
                    $movBancario->getCdreducont(), $movBancario->getDtmovbanc(), $movBancario->getNrorg(), $vrmovicred
                );
                
                $this->entityManager->flush();
            }
        }catch(\Exception $e){
            Throw $e;
        }
    }
    
    public function deleteLancamento($nrlanccont, $cdempresa, $cdreducont, $nrmovbanc, $nrorg, $tipoLancamento, $vrmovicred, $dtmovbanc,
        $cdfilial, $cdclassfina, $idmovfluxo, $cliente, $fornecedor, $numeroLancamento, $nrcheque, $nrmovbancorig = null){
        try {
            $this->factory->deleteLancccusto($nrlanccont, $cdempresa, $nrorg);
            $this->factory->deleteLanccont($nrlanccont, $cdempresa, $nrorg);
            if ($tipoLancamento == 'C') {
                if ($cliente) {
                    self::deleteDevolucaoAdiantamentoCliente($nrorg, $cdreducont, $numeroLancamento, 'nrmovicred');
                }elseif ($fornecedor) {
                    self::deleteDevolucaoAdiantamentoFornecedor($nrorg, $cdreducont, $numeroLancamento, 'nrmovicred');
                }
                $this->factory->deleteMovcredbanc($cdreducont, $nrmovbanc, $nrorg);
                $this->fluxodecaixaservice->atualizaSaldoContCorr($cdreducont, $dtmovbanc, $nrorg, $vrmovicred * -1);
            }else{
                if ($cliente) {
                    self::deleteDevolucaoAdiantamentoCliente($nrorg, $cdreducont, $numeroLancamento, 'nrmovidebi');
                }elseif ($fornecedor) {
                    self::deleteDevolucaoAdiantamentoFornecedor($nrorg, $cdreducont, $numeroLancamento, 'nrmovidebi');
                }
                $this->entityManager->flush();
                if ($nrcheque){
                    $this->desassociaCheque($cdempresa, $cdreducont, $nrorg, $numeroLancamento, $nrcheque);
                }
                $this->factory->deleteMovdebbanc($cdreducont, $nrmovbanc, $nrorg);
                $this->fluxodecaixaservice->atualizaSaldoContCorr($cdreducont, $dtmovbanc, $nrorg, $vrmovicred);
            }
            $this->factory->deleteMovbancario($nrmovbanc, $cdreducont, $nrorg);
            self::atualizaMovimentacaoBancariaOrigem($nrorg, $cdreducont, $nrmovbancorig, $vrmovicred, $tipoLancamento);
            $idoperacao = ($tipoLancamento == 'C')? self::ENTRADA: self::SAIDA;
            $movfluxo = $this->fluxodecaixaservice->atualizaFluxoCaixa($nrorg, $cdempresa, $cdfilial, $cdclassfina,
                $dtmovbanc, $vrmovicred * -1, $idmovfluxo, $idoperacao);
            $this->entityManager->persist($movfluxo);
        }catch(\Exception $e){
            throw $e;
        }
    }

    public function deleteDevolucaoAdiantamentoCliente($nrorg, $cdreducont, $nrmovbanc, $fieldMovimento){
        $adiantareceb = $this->factory->getAdiantareceb($nrorg, $cdreducont, $nrmovbanc, $fieldMovimento);
        $devolucaorec = $this->factory->getDevolucaorec($nrorg, $cdreducont, $nrmovbanc, $fieldMovimento);
        forEach($adiantareceb as $adiantamento) {
            $this->entityManager->remove($adiantamento);
        }
        forEach ($devolucaorec as $devolucao) {
            $this->entityManager->remove($devolucao);
        }
    }

    public function deleteDevolucaoAdiantamentoFornecedor($nrorg, $cdreducont, $nrmovbanc, $fieldMovimento){
        $adiantapagar = $this->factory->getAdiantapagar($nrorg, $cdreducont, $nrmovbanc, $fieldMovimento);
        $devolucaopag = $this->factory->getDevolucaopag($nrorg, $cdreducont, $nrmovbanc, $fieldMovimento);
        forEach($adiantapagar as $adiantamento) {
            $this->entityManager->remove($adiantamento);
        }
        forEach ($devolucaopag as $devolucao) {
            $this->entityManager->remove($devolucao);
        }
    }

    public function preparaAtualizacaoFluxo($row){
        if ($row['IDOPERACAO'] == 'E')
        {
            $row['VRENTMFLUXO'] = floatval($row['VRMOVICRED']);
            $row['VRSAIMFLUXO'] = 0.0;
        }else{
            $row['VRSAIMFLUXO'] = floatval($row['VRMOVICRED'] * -1);
            $row['VRENTMFLUXO'] = 0.0;
        }
        return $row;
    }

    private function geraCodigo($cdcontador, $chave, $row){
        $row['__is_new_code'] = true;
        $row['__new_code_pk'] = $cdcontador;
        $row['__table_pk'] = $chave;
        $row['__new_code_length'] = 10;
        $row['__is_new'] = true;
        if(!empty($row['__new_code_pk'])) {
            $castNumber = empty($row['__cast_number']) ? false : $row['__cast_number'];
            $newCodeKey = empty($row['__using_unit']) ? $row['__new_code_pk'] : ($row['__new_code_pk'].$this->environment->getCdFilial());
            $newCodeKey = strtoupper($newCodeKey);
            $row[$chave] = $this->newCode->getCodeByTableName($newCodeKey, $this->environment->getNrOrgTrab(), 1, $row['__new_code_length'], $castNumber);
        }
    }

    public function desassociaCheque($cdempresa, $cdreducont, $nrorg, $numeroLancamento, $nrcheque){
        $cheque = $this->factory->getCheque($nrcheque, $cdreducont, $nrorg);
        if($cheque) {
            $cheque = $this->factory->updateSituacaoCheque($cheque, '0', null, null, null, null, null, $nrorg);
            $this->entityManager->persist($cheque);
        }
        $movdebbanc = $this->factory->getLancamentoDebito($nrorg, $numeroLancamento, $cdreducont, $cdempresa);
        if ($movdebbanc){
            $movdebbanc = $this->factory->setChequeLancamentoDebito($movdebbanc, null, $nrorg);
            $this->entityManager->persist($movdebbanc);
        }
        $this->entityManager->flush();

    }

    public function emissaoCheque($row, $response){
        try {
                $nrorg = $this->environment->getNrOrgTrab();
                $cheque = $this->factory->getCheque($row['NRCHEQUE'], $row['CDREDUCONT'], $nrorg);
                if ($cheque){
                    $cheque = $this->factory->updateSituacaoCheque($cheque, '1', null, $row['DTEMISCHEQ'], $row['VRCHEQUE'] , $row['DSNOMCHEQ'], null, $nrorg);
                    $this->entityManager->persist($cheque);
                }
                $movdebbanc = $this->factory->getLancamentoDebito($nrorg, $row['NRMOVIDEBI'], $row['CDREDUCONT'], $row['CDEMPRESA']);
                if ($movdebbanc){
                    $movdebbanc = $this->factory->setChequeLancamentoDebito($movdebbanc, $row['NRCHEQUE'], $nrorg);
                    $this->entityManager->persist($movdebbanc);
                }
                $this->entityManager->flush();
                $this->contabilizacaoService->contabilizaLancamentoControleBancario($row['CDEMPRESA'], $row['CDREDUCONT'], $row['NRMOVIDEBI'], self::DEBITO, $row['IDCTBLBANC']);
        } catch (\Exception $e) {
            $this->contabilizacaoService->saveContabilizacaoData($response);
            throw $e;
        }
    }
}