<?php
namespace Teknisa\FIN\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\FIN\Factories\Ger38200ImportacaoContasPagarFactory;
use Zeedhi\Framework\DTO\Request\DataSet;
use Zeedhi\Framework\DataSource;
use Financeiro\Util\Util;
use Teknisa\FIN\Modules\ContaPagar\Services\TitulosAPagarService as TitulosAPagarService; //Instanciado somente para pegar função de novo código (Nota Fiscal)

class Ger38200ImportacaoContasPagarService {

    protected $entityManager;
    protected $titulosAPagarService;
    
    public function __construct(EntityManager $entityManager, Ger38200ImportacaoContasPagarFactory $ger38200ImportacaoContasPagarFactory,  TitulosAPagarService $titulosAPagarService, Environment $environment){
        $this->entityManager = $entityManager;
        $this->ger38200ImportacaoContasPagarFactory = $ger38200ImportacaoContasPagarFactory;
        $this->titulosAPagarService = $titulosAPagarService;
        $this->environment  = $environment;
        $this->nrorg = $environment->getNrOrgTrab();
    }
    
    //Função responsável por tratar os parametros de entrada da tela e realizar validações de acesso a API HCM
    public function importacaoContasPagar($dados) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            
            $idrequisicao  = $dados['IDREQUISICAO'];
            $nrnotafisc = isset($dados['NRNOTAFISC']) ? $dados['NRNOTAFISC'] : null; // Número da Nota informado pelo cliente na tela
            $nrorg = $dados['NRORG'];
            
            // Dados de Parametros para a requisição ao HCM.
            $params = array(
                'requestType' => 'Row',
                'row' => array(
                    "CNPJ"             => Util::removeMascaraInscricao(isset($dados["CNPJ"])?$dados["CNPJ"]:null),
                    "DATA"             => isset($dados['DTPAGAMENTO'])?$dados['DTPAGAMENTO']:null,
                    "MOVIMENTO"        => $dados['IDPAGAMENTO'],
                    "CDTPCTPAGAR"      => isset($dados['CDTPCONTAREQUISICAO'])?$dados['CDTPCONTAREQUISICAO']:null,
                    "CNPJFORNECEDOR"   => isset($dados["NRINSJURFORN"])?$dados["NRINSJURFORN"]:null,
                    "DATACOMPETENCIA"  => '01/'.$dados['DTCOMPETENCIA']
                )
            );
            
            // Verifica tipo de requisição | 1 - HcmServices / 2 - URL
            if ($idrequisicao == '1') {
                
                $tokenControle = ''; // Variavel de controle para passar o token válido corretamente
                
                // Busca Parametrização Geral para Web Service HCM
                $paramgeral = $this->ger38200ImportacaoContasPagarFactory->buscaParamgeral();
                
                //Seta variaveis de controle de acordo com os dados da parametrização da PARAMGERAL
                $stUrlHcm       = $paramgeral['DSURLHCM'];
                $stClientID     = $paramgeral['DSIDHCM'];
                $stClientSecret = $paramgeral['DSSECRETHCM'];
                $stToken        = $paramgeral['DSTOKENHCM'];
                
                //Verifica se os parmetros não sao nulos para conexão correta
                if ($stUrlHcm and $stClientID and $stClientSecret) {
                    
                    // Verifica se token é valido antes de chamar a api  
                    $accessApi = $this->checkAccess($stUrlHcm, $stToken, $stClientSecret);
                    
                    if($accessApi == 'Denied'){
                        
                        //Caso o acesso seja negado na requisição, solicita um novo Token para nova requisição a API
                        $token = $this->requestToken($stUrlHcm, $stClientID,$stClientSecret);
                        
                        //Atualiza a tabela PARAMGERAL para não sobrescrever o token válido de 24 horas a toda requisição realizada na API
                        $this->updateToken($token, '0001', $nrorg);
                        
                        $connection->commit();
                        
                        $tokenControle = $token;
                        
                    } else {
                        $tokenControle = $stToken;
                    }
                    
                    //Retorna os dados da API ou os erros de requisição
                    $connectionApi = $this->requestDataApiHCM($stUrlHcm, $stClientID,$stClientSecret,$tokenControle, $params);

                    if(array_key_exists('messages', $connectionApi)){
                        throw new \Exception('Operação bloqueada. '.$connectionApi['messages'][0]["message"]);
                    } else {
                        $organiseData = $this->consolidateData($connectionApi["dataset"]["pagamentos"] , $nrnotafisc);
                        return $organiseData;
                    }
                    
                }else{
                    throw new \Exception('Operação bloqueada. Dados de conexão do HCM não definidos.');
                }
            }else{
                throw new \Exception('Operação bloqueada. Importação via URL não configurada.');
            }
            
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
    
    // Função responsável por requisitar os dados na API do HCM
    public function requestDataApiHCM($stUrlHcm, $stClientID,$stClientSecret,$stToken, $params){
        try {
            
            //Recurso a ser acessado pela API junto com o URL BASE
            $resourcePath = '/retornaPagamentosPorCnpjDataMovimento';
            
            //Cabeçalho da Requisição 
            $headers = [
                'Cache-Control: no-cache',
                'Content-type: application/json',
                'OAuth-SecretId:'.$stClientSecret,
                'OAuth-Token:'.$stToken
            ];

            //CONFIGURAÇÃO DO CURL
            $curl = curl_init();
            curl_setopt_array($curl,[
              CURLOPT_URL            => $stUrlHcm.$resourcePath, //URL base + recurso
              CURLOPT_POST           => true, //Metodo da requisição, segundo HCM é somente POST
              CURLOPT_POSTFIELDS     => json_encode($params), //Parametros de requisição
              CURLOPT_RETURNTRANSFER => true, // Parametro usado para esperar a resposta da URL
              CURLOPT_HTTPHEADER     => $headers //cabeçalho da requisição
            ]);
            
            //EXECUTA O CURL
            $response = curl_exec($curl);
            
            //Verificação de erros na execução da solicitação.
            if (curl_errno($curl)) {
                throw new \Exception('Erro cURL: ' . curl_error($curl));
            }
            
            curl_close($curl);
            
            //RETORNA O ARRAY DA RESPOSTA
            return json_decode($response,true);
            
            }catch(\Exception $e){
                throw $e;
            }
        }
        
    // Função responsável por retornar novo TOKEN quando o anterior passar de 24 horas e não for mais válido
    public function requestToken($stUrlHcm, $stClientID,$stClientSecret){
        try {
            
            $uri_produ = $stUrlHcm.'/requestToken';
            
            $clientData = array(
                "CLIENT_ID" => $stClientID,
                "CLIENT_SECRET"=> $stClientSecret,
            );
            
            $header[] = 'Content-Type: application/json';
            $header[] = 'Content-Length: ' . strlen(json_encode($clientData));
            $result = $this->connectionWebService($header, $uri_produ, $clientData);
            
            return $result['dataset']['token'][0];

        }catch(\Exception $e){
            throw $e;
        }
    }
    
    // Função responsável por Verificar se o token é valido antes de requisitar os dados.
    public function checkAccess($stUrlHcm, $stToken, $stClientSecret)
    {
        try {
            $uri_produ = $stUrlHcm.'/checkAccess';
            
            $clientData = array(
                "TOKEN" => $stToken,
                "CLIENT_SECRET" => $stClientSecret,
            );
            
            $header[] = 'Content-Type: application/json';
            $header[] = 'Content-Length: ' . strlen(json_encode($clientData));
            $result = $this->connectionWebService($header, $uri_produ, $clientData);
            
            if ($result['messages'][0]['message'] == "Authorized") {
                return "Authorized";
            } else {
                return "Denied";
            }
            
        }catch(\Exception $e){
            echo($e->getMessage());
        }
    }
    
    //Função auxiliar utilizado pela verificação de acesso e geração de novo TOKEN API HCM
    public function connectionWebService($header, $uri_produ, $data) {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
        curl_setopt($ch, CURLOPT_URL, $uri_produ);
        curl_setopt($ch, CURLOPT_POST, count($data));
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($data));
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "POST");
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
        $result = curl_exec($ch);
        
        //Verificação de erros na execução da solicitação.
        if (curl_errno($ch)) {
            throw new \Exception('Erro cURL: ' . curl_error($ch));
        }
        curl_close($ch);
        return json_decode($result, true);
    }
    
    //Função responsável por updatar o token novo gerado
    public function updateToken($stToken, $cdpargeral, $nrorg)
    {
        try {
            
            // Parametro para Update (TOKEN) 
            $dataUpdate = array(
                "DSTOKENHCM" => $stToken,
                "CDPARGERAL" => $cdpargeral,
                "NRORG"      => $nrorg
            );
            
            $this->ger38200ImportacaoContasPagarFactory->updateToken($dataUpdate);
            
        }catch(\Exception $e){
            throw $e; 
        }
    }
    
    /**
     * Função para ajustar os dados, renomeando as colunas e passando alguns parametros fixos necessários".
     * @param array $array
     * @param $nrnotafisc
     * @return array
     */
    public function consolidateData($pagamentos, $nrnotafisc){
        $dadosConsolidados = array();
        foreach ($pagamentos as $index => $pagamento) {
            
            // Incrementa $nrnotafisc apenas a partir do segundo pagamento e quando $nrnotafisc não for nulo
            if ($index > 0 && $nrnotafisc != null) {
                $nrnotafisc = str_pad((int)$nrnotafisc + 1, 9, '0', STR_PAD_LEFT);
            }
            //Renomeia alguns campos para facilitar e passa null quando o dado não retornar da API
            $dadosRenomeados = array(
                "NRINSCRICAO" => $pagamento["cpf"],
                "NMINSCRICAO" => $pagamento["nome"],
                "DTPAGTO" => $pagamento["data"],
                "CDBANCO" =>  $pagamento["contas"] ? $pagamento["contas"][0]["banco"] : null,
                "NMBANCO" => $pagamento["contas"] ? $pagamento["contas"][0]["banco_nome"] : null,
                "CDAGENCIA" => $pagamento["contas"] ? $pagamento["contas"][0]["agencia"] : null,
                "CDCONTCORR" => $pagamento["contas"] ? $pagamento["contas"][0]["conta"] : null,
                "TIPOCONT" => $pagamento["contas"] ? $pagamento["contas"][0]["tipo"] : null,
                "VRPGTO" => $pagamento["valor"],
                "NRNOTAFISC"=> $nrnotafisc ? $nrnotafisc :null,
                "CENTROCUSTOS" => isset($pagamento["centrocustos"])?$pagamento["centrocustos"]: null,
                "FILIAIS" => isset($pagamento["filiais"], $pagamento["valor"]) ? $this->adjustStructure($pagamento["filiais"], $pagamento["valor"]): null,
                "IDSISTEMA" => "HCM"
            );
            
            $dadosConsolidados[] = $dadosRenomeados;
        }
        
        return $dadosConsolidados;
    }
    
    /**
     * Função para ajustar a estrutura do array "filiais". Que são os rateios dos títulos
     * @param array $array
     * @params $vrtotalPag
     * @return array 
     */
    public function adjustStructure($array,$vrtotalPag) {
        $result = array();
        
        foreach ($array as $key => $value) {
            if (is_array($value)) {
                // Verifica se o array é associativo 
                if ($this->isAssociativeArray($value)) {
                    foreach ($value as $filial => $valor) {
                        $result[] = array(
                            "CDCENTCUST" => (string) $filial, 
                            "CDFIRATCTPAG" => (string) $key, 
                            "VRRATCCCTPAG" => $valor,
                            "VRPERATEIO" =>  (($valor / $vrtotalPag) * 100) 
                        );
                    }
                } else {
                    // Se o array não for associativo, manten a estrutura existente
                    $result[$key] = $this->adjustStructure($value);
                }
            } else {
                // Se for um valor simples, manten a estrutura existente
                $result[$key] = $value;
            }
        }
    
        return $result;
    }
    
    // Função auxiliar para verificar se um array é associativo
    private function isAssociativeArray($array) {
        return array_keys($array) !== range(0, count($array) - 1);
    }

    /**
     * Função auxiliar para transformar um array multidimensional em um array associativo simples,
     * adicionando prefixos aos índices.
     * @param array  $array 
     * @param string $prefix 
     * @return array 
     */
    public function flattenArray($array, $prefix = '') {
        $result = array();
        foreach ($array as $key => $value) {
            if (is_array($value)) {
                $result += $this->flattenArray($value, $prefix . $key . '_');
            } else {
                $result[$prefix . $key] = $value;
            }
        }
        return $result;
    }
    
    private function formatDate($date){
        return $date?date('d/m/Y H:i:s', strtotime($date)):date('d/m/Y H:i:s');
     }
     
    /**
     * Função para receber os dados que vem da tela, adicionar valores que estão 
     * dentro do array de pagamentos no array de dados fixos e enviar ao build para preparar outros parametros para inserção.
     * @param array  $array 
     * @return $array 
     */
    public function buildTitulosAPagarDataSet($dataSet){
        
        $dataBuild = array();
        
        $dadosForm = $dataSet[0]; // Dados fixos do form 
        $dadosGrid = $dataSet[1]; // Dados de Pagamento
        
        foreach($dadosGrid as $dados){
            
            $dadosForm["VRNOTAFISC"]   = $dados["VRPGTO"];
            $dadosForm["VRCONTA"]      = $dados["VRPGTO"];
            $dadosForm["DTEMISSAO"]    = $dados["DTPAGTO"];
            $dadosForm["DTENTRSAID"]   = $dados["DTPAGTO"];
            $dadosForm["DTATUAVENPAG"] = $dados["DTPAGTO"];
            $dadosForm["CNPJCPFPAGAMENTO"] = $dados["NRINSCRICAO"];
            
            array_push($dataBuild, $this->buildTitulosAPagar($dadosForm));
        }
        return $dataBuild;
    }
     
     /**
     * Função responsável por preparar os dados para inserir o contas a pagar
     * @param array  $array
     * @return array $array
     */
    public function buildTitulosAPagar($dataSet) {
        
        $row = array();
        
        $row['NRORG']         = $dataSet["NRORG"];
        
        //insere empresa
        $row['CDEMPRESARECE'] = $dataSet["CDEMPRESA"];
        
        //insere dados da filial
        $filial = $this->ger38200ImportacaoContasPagarFactory->getFilialDeOrigem($dataSet["CDEMPRESA"], $dataSet["CDFILIAL"], $dataSet["NRORG"]);
        $row['CDFILIAL']       = $filial['CDFILIAL'];
        $row['CDFILIRECE']     = $filial['CDFILIAL'];
        $row['CDFILIALRECE']   = $filial['CDFILIAL'];
        $row['CDINSCESTARECE'] = $filial['CDINSCESTA'];
        $row['CDINSCMUNIRECE'] = $filial['CDINSCMUNI'];
        
        //insere dados do fornecedor
        $fornecedor = $this->ger38200ImportacaoContasPagarFactory->getCNPJOrCPFPagar($dataSet["CNPJCPFPAGAMENTO"], $dataSet["NRORG"]);
        $row['CDFORNECED']     = $fornecedor['CDFORNECED'];
        $row['NRINSJURFORN']   = $fornecedor['NRINSJURFORN'];
        $row['IDTPIJURFORN']   = $fornecedor['IDTPIJURFORN'];
        
        //Parametros para geração de número de nota fiscal quando não é passado pela tela.
        $nrorg      = $dataSet["NRORG"];
        $cdcontador = "NRNOTAFISCPAG";
        $novocodigo = $this->titulosAPagarService->geraNovoCodigo($nrorg, $cdcontador);
        
        //Busca tamanho padrão da nota fiscal para inserir conforme parametrização da filial
        $tamanho = $this->ger38200ImportacaoContasPagarFactory->getTamanhoNotaFiscal($dataSet["CDFILIAL"], $dataSet["CDEMPRESA"]);
        if($tamanho && $tamanho["NRTAMNUMNFUTDFE"] > 0){
            $tamanho = $tamanho["NRTAMNUMNFUTDFE"];
        } else {
            $tamanho = 9;
        }
        
        //insere dados nota fiscal
        $row['NRNOTAFISC'] = isset($dataSet["NRNOTAFISC"])?substr($dataSet["NRNOTAFISC"], -$tamanho):substr($novocodigo, -$tamanho) ;
        $row['CDSERIE']    = $dataSet['CDSERIE'];
        $row['IDSTATUS']   = 'A';
        $row['CDTIPOOPER'] = $dataSet["CDTIPOOPER"]; 
        $row['VRNOTAFISC'] = $dataSet["VRNOTAFISC"];
        
        
        //insere dados da nota
        $tipoConta = $this->ger38200ImportacaoContasPagarFactory->getTipoDeContaPagar($dataSet["CDTPCTPAGAR"],$dataSet["NRORG"]);
        $tpPagamento = [
                1 => "Mensal",
                2 => "Férias",
                3 => "13º Salário",
                4 => "Rescisão",
                5 => "Adiantamento",
                6 => "Participação nos Lucros",
                18 => "Vale Transporte",
                21 => "Benefício",
            ][$dataSet["IDPAGAMENTO"]] ?? "";
        
        $row['CDTPCTPAGAR'] = $tipoConta['CDTPCTPAGAR'];
        $row['DSCONTAPAG']  = "IMPORTAÇÃO WEBSERVICE - Referente a pagamento da folha:". $tpPagamento. ": ". $dataSet["DTCOMPETENCIA"]." Pgto: ".substr($dataSet["DTEMISSAO"],0,10);
        $row['CDCLASSFINA'] = $tipoConta['CDCLASSFINA'];
        $row['VRCONTA']     = $dataSet['VRCONTA'];
        
        $row['NRINSJURNF']      = $fornecedor['NRINSJURFORN'];
        $row['IDTPIJURNF']      = $fornecedor['IDTPIJURFORN'];
        $row['CDEMPRESA']       = $dataSet["CDEMPRESA"];
        $row['CDINSCESTA']      = $filial['CDINSCESTA'];
        $row['CDINSCMUNI']      = $filial['CDINSCMUNI'];
        $row['VRPRODUTOS']      = $dataSet["VRNOTAFISC"];
        $row['DTLIBERACAOFIN']  = DateUtil::getDataAtual();
        $row['DTLANCAMENNF']    = DateUtil::getDataAtual();
        $row['IDSISTEMA']       = 'PAG';
        $row['IDBLOQUEADO']     = 'S';
        $row['IDDOCFISCAL']     = 'N';
        $row['IDDOCFINANC']     = 'S';
        $row['IDENTRSAIDOP']    = 'E';
        $row['IDDOCCONTAB']     = 'S';
        
        //insere dados operador
        $row['CDOPERACTPG'] = $dataSet['CDOPERADOR'];
        
        //Insere dados de chave PIX
        $row['CHAVEPIX']  = null;
        $row['CODEQRPIX'] = null;
        
        //insere datas
        $row['DATAATUAL']  = DateUtil::getDataAtual();
        $dataEmissao = $this->titulosAPagarService->formataData($dataSet['DTEMISSAO']);
        $dataEntrada = $this->titulosAPagarService->formataData($dataSet['DTENTRSAID']);
        $dataVencimento = $this->titulosAPagarService->formataData($dataSet['DTATUAVENPAG']);
        $row['DTATUAVENPAG'] = $dataVencimento;
        $row['DTEMISSAO']    = $dataEmissao;
        $row['DTENTRSAID']   = $dataEntrada;
        
        // Parâmetros Boleto
        $row['NRLANBOLPAG']     = $this->titulosAPagarService->geraNovoCodigo($dataSet["NRORG"], 'BOLETPAGAR'.$dataSet["CDFILIAL"]);
        $row['VRBOLPAG']        = $dataSet['VRCONTA'];
        $row['VRFINALBPAG']     = $dataSet['VRCONTA'];
        $row['DTVENBOLPAG']     = $dataVencimento;
        
        // Parâmetros Movimentação Fluxo
        $row['DTMOVFLUXO']      = $dataVencimento;
        $row['IDMOVFLUXO']      = ($row['IDSTATUS'] == 'V')?'P':'E';
        
        // Parâmetros Conta
        $row['NRSEQVENPAG']     = '01';
        $row['NRPARCELAPAG']    = '00';
        $row['DTORIGVENPAG']    = $dataVencimento;
        $row['VRATUAPAG']       = $dataSet['VRCONTA'];
        $row['VRORIGPAG']       = $dataSet['VRCONTA'];
        $row['CDFILIBPAG']      = $filial['CDFILIAL'];
        $row['IDSTATPAG']       = 'A';
        $row['QTDIASVENPAG']    = 0;
        $row['CDFIMOVFLPAG']    = $filial['CDFILIAL'];
        $row['CDFILILANC']      = $filial['CDFILIAL'];
        
        //parametro contabilização, caso dê erro de contabilização no lancamento e for corrigido não gerar uma contabilização nova
        $row['CONTABILIZA']  = false;
        $row['NRLANCCTBDOC'] = null;
        
        return $row;
    }
    
     /**
     * Função responsável por preparar o rateio do pagamento
     * @param array  $array
     * @return array $array
     */
     public function buildRateioTitulosAPagar($dataSet){
         
        $row = array();
        $dadosGrid = $dataSet[1]; // Dados de Pagamento
        
        foreach($dadosGrid as $dados){
            // Itera sobre o indice de filiais para gerar o rateamento e inserir no mesmo array de
            if($dados["FILIAIS"]){
                foreach($dados["FILIAIS"] as $dadosGridPagamento){
                    array_push($row, [$dadosGridPagamento]);
                }
            }
            
        }
        return $row;
     }
     
     /**
     * Função responsável por enviar os dados para o service do contas a pagar para inserir o título
     */
     public function inserirContasAPagar($dataBuild,$response, $rateioBuild){
         try {
             
            $permiteParcelamento = false;
            
            // Gera os Titulos a Pagar
            $contas = $this->titulosAPagarService->geraTitulosAPagar($dataBuild, $response, $permiteParcelamento, $rateioBuild);

         } catch (\Exception $e) {
            throw $e;
        }
     }
     
     /**
     * Função responsável por realizar as validações dos dados de pagamento que retornaram da API do HCM
     */
     public function validations($dataBuild){
         
         try{
             $errors = NULL;
             $errors = array();
             
             $dadosGrid = $dataBuild[1]; // Dados de Pagamento
             
             $cdempresa = $dataBuild[0]["CDEMPRESA"];
             $serieNotaFiscal = $dataBuild[0]["CDSERIE"];
             $cdtpctpagar =  $dataBuild[0]["CDTPCTPAGAR"];
             $cdfilibpag = $dataBuild[0]["CDFILIAL"];
        
             //Itera sobre os dados de pagamento para verificar se os Cód. Fornecedor, Data, Centro de Custo, Filial são validos
             foreach($dadosGrid as $indice => $dados){
                 
                 $linha = $indice + 1;
                
                 //Valida se existe um fornecedor cadastrado com os dados que vieram do pagamento HCM
                 $fornecedor = $this->ger38200ImportacaoContasPagarFactory->getCNPJOrCPFPagar($dados["NRINSCRICAO"], $this->nrorg);
                 
                 if(!$fornecedor){
                    array_push($errors, [ "LINHA" =>'Linha: '. $linha, "DSERROTEMP" => "Fornecedor " . $dados["NRINSCRICAO"]." não cadastrado " ]);
                 } else {
                     //Valida se o número da nota fiscal se é valida
                     if($dados["NRNOTAFISC"]){
                         $notafiscal = $this->ger38200ImportacaoContasPagarFactory->getNotaFiscalUtilizada($fornecedor["CDFORNECED"],$serieNotaFiscal,$dados["NRNOTAFISC"]);
                         if($notafiscal){
                            array_push($errors, [ "LINHA" =>'Linha: '. $linha, "DSERROTEMP" => "A nota fiscal ".$dados["NRNOTAFISC"] ." Já foi utilizada"]);
                         }  
                     }
                 }
                 
                 //Valida se a data do pagamento é valida
                 $dataPgto = $this->validaDataPagamento($dados["DTPAGTO"]);
                 if($dataPgto ==  null){
                     array_push($errors, [ "LINHA" =>'Linha: '. $linha, "DSERROTEMP" => "Data ".substr($dados["DTPAGTO"],0,10)." inválida "]);
                 }
                 
                 //valida se o centro de custo e filial existem
                 if($dados["FILIAIS"] && is_array($dados["FILIAIS"])){
                     foreach($dados["FILIAIS"] as $centroCustoeFilial){
                         
                         //valida se o centro de custo esta cadastrado
                         $cdcentcust = $this->ger38200ImportacaoContasPagarFactory->getCentroDeCusto($centroCustoeFilial["CDCENTCUST"],$this->nrorg);
                         if(!$cdcentcust){
                             array_push($errors, [ "LINHA" =>'Linha: '. $linha, "DSERROTEMP" => "Centro de custo ". $centroCustoeFilial["CDCENTCUST"]. " não cadastrado "]);
                         }
                         
                         // Valida se a filial esta cadastrada
                         $filal = $this->ger38200ImportacaoContasPagarFactory->getFilial($centroCustoeFilial["CDFIRATCTPAG"],$cdempresa,$this->nrorg);
                         if(!$filal){
                             array_push($errors, [ "LINHA" =>'Linha: '. $linha, "DSERROTEMP" => "Unidade ". $centroCustoeFilial["CDFIRATCTPAG"]. " não cadastrada "]);
                         }
                     }
                 }
                 
                 //valida se o pagamento já foi criado, impedindo que seja inserido duplicado
                 $conta = $this->ger38200ImportacaoContasPagarFactory->getContaInserida($cdempresa, substr($dados["DTPAGTO"],0,10),  substr($dados["DTPAGTO"],0,10),
                   $cdtpctpagar, $cdfilibpag, $dados["VRPGTO"], $dados["VRPGTO"], $dados["NRINSCRICAO"]);
                 
                 if($conta){
                     array_push($errors, [ "LINHA" =>'Linha: '. $linha, "DSERROTEMP" => "Conta já lançada para a unidade ". $conta["CDFILIAL"]. ", Nº de Lançamento ".$conta["NRLANCTONF"]]);
                 }
             }
             
             return $errors;
             
         }catch(\Exception $e){
            throw $e; 
        }
     }
     
     /**
     * Função responsável por validar se a data do pagamento esta no formato correto
     */
    public function validaDataPagamento($dataSemCorte){
        
        $dia = substr($dataSemCorte, 0, 2);
        $mes = substr($dataSemCorte, 3, 2);
        $ano = substr($dataSemCorte, 6, 4);
        
        $valida = checkdate((int)$mes,(int)$dia, (int)$ano);
        
        if($valida === true){
           return $dataSemCorte;
        } else {
            return null;
        }
    }
}
