var rec00200_RetornoCobranca_Controller = function(ZHPromise, General, templateManager, ScreenService, Rec00200_RetornoCobranca_Service, ZhFilterPreferences)
{
    /** Variáveis de Inicialização */
    var self = this;

    self.incrementaDataFinal = function(){
        var retorno = templateManager.container.getWidget('RetornoGrid');
        retorno.dataSourceFilter.forEach(function(filtro){
            if (filtro.name == 'DTCNABREC'){
                filtro.value[1] = moment(filtro.value[1].replace('/', ''), 'DDMMYYYY').add(1, 'days').format('DD/MM/YYYY');
            }
        });
        

    };
    
    self.openFilterRetornoCobranca = function(retorno){
        self.buscaPadraoEmpresa(retorno).then(function(empresa){
            var data = moment().format('DD/MM/YYYY');
            var rowFilter = {
                'CDEMPRESA': empresa.CDEMPRESA,
                'NMEMPRESA': empresa.NMEMPRESA,
                'DTCNABREC': {start : data, end : data},
            };
            ZhFilterPreferences.setOriginalFilter(retorno.id, rowFilter);
            ZhFilterPreferences.openFilterWidget(retorno, rowFilter).then( function(sucess){
                retorno.getField('DSTPCTRECEB').dataSourceFilter[0].value = empresa.CDEMPRESA;
                retorno.getField('DSTPCTRECEB').reload();
            });
        });
    };

    self.voltarBoletos = function(){
        ScreenService.goBack().then(function(){
            var tabbed = templateManager.container.getWidget('ImportacaoTabbed');
            tabbed.fields.forEach(function(field){
                field.readOnly = false;
            });
        });
    };

    self.ativaGridOcorrencias = function(widget){
        if (widget.name == 'BoletosGrid'){
            templateManager.container.getWidget('OcorrenciasGrid').activate();
        }else{
            templateManager.container.getWidget('ImportacaoOcorrenciasTabbed').activate();
        }
        //self.setGridNaoClicavel();
    };

    self.ativaWidgetPrincipal = function(widget){
        widget.activate();
    };
    /**
     * Busca Parâmetros de Estoque
     * @return dataProcessamento
     */
    self.buscaEmpresa = function (cdempresa){
        return Rec00200_RetornoCobranca_Service.buscaEmpresa(cdempresa).then(function(empresa) {
            return empresa;
        });
    };

    /**
     * Busca Filial
     * @return filialLogada
     */
    self.buscaFilial = function (){
        return Rec00200_RetornoCobranca_Service.buscaFilial().then(function(filial) {
            return {dataset : {filial : filial}};
        });
    };

    /**
     * Busca Padrão Empresa
     * @param widget
     */
    self.buscaPadraoEmpresa = function(widget) {
        var deferred = ZHPromise.defer();
        self.buscaFilial().then(function(response_filial){
            var filial   = response_filial.dataset.filial[0];
            var cdempresa = filial.CDEMPRESA;

            self.buscaEmpresa(cdempresa).then(function (response_empresa){
                var empresa = response_empresa.dataset.empresa[0];
                deferred.resolve(empresa);
            });
        });
        return deferred.promise;
    };

    var setGridNaoClicavel = function(){
        var swipe = document.getElementsByClassName('zh-swipe-inner zh-content ng-scope open');
        var div = document.createElement('div');
        div.setAttribute('class', 'swipe-overlay ng-scope');
        div.setAttribute('data-ng-if', 'swipe.editing || swipe.currentWidget.editing');
        swipe[0].appendChild(div);
    };

    self.adicionaRetorno = function(){
        var tabbed = templateManager.container.getWidget('ImportacaoTabbed');
        tabbed.isVisible = true;
        ScreenService.openSwipe(tabbed).then(function(){
            window.setTimeout(setGridNaoClicavel, 100);
        });
    };

    /**
     * Cancela Retorno
     */
    self.cancelaRetorno = function(){
        var tabbed = templateManager.container.getWidget('ImportacaoTabbed');
        tabbed.isVisible = false;
        ScreenService.closeSwipe().then(function(){
            tabbed.getParent().activate();
            tabbed.widgets[0].dataSource.data = [];
            tabbed.widgets[0].ocorrencias     = [];
        });
    };

    /**
     * Valída Visualização Importacao
     * @param widget
     */
    self.validaVisualizacaoImportacao = function(widget){
        widget.getField('DSARQCNABREC').readOnly = !widget.getField('NMBANCO').value();

        self.deleteDataRetorno();
    };

    /**
     * Deleta Data Source Retorno
     */
    self.deleteDataRetorno = function(){
        var widgetBoletos = templateManager.container.getWidget('ImportacaoBoletosTabbed');
        var widgetOcorrencias = templateManager.container.getWidget('ImportacaoOcorrenciasTabbed');
        widgetBoletos.dataSource.data = [];
        widgetOcorrencias.dataSource.data = [];
        widgetOcorrencias.ocorrencias = [];
    };

    /**
     * Prepara Arquivo CNAB
     * @param widget
     */
    self.preparaArquivoRetorno = function(widget){
        var route = 'fin_contaReceber#/carregaRetornoCobranca';

        var cdempresa   = templateManager.container.getWidget('ImportacaoTabbed').currentRow.CDEMPRESA;
        var cdbanccnab  = widget.currentRow.CDBANCCNAB;
        var cnabretorno = widget.currentRow.DSARQCNABREC[0].b64File.split(',')[1];

        var row = {
            'CDEMPRESA'  : cdempresa,
            'CDBANCCNAB' : cdbanccnab,
            'CNABRETORNO': cnabretorno
        };

        General.callRestEngine(route, 'Row', row)
            .then(function(response){
                this.setDataSourceRetorno(response.dataset.BOLETOS);
                this.setDataSourceOcorrencias(response.dataset.OCORRENCIAS);
                widget .widgets[0].SITUACAOCNAB = response.dataset.SITUACAOCNAB[0];
            }.bind(this))
            .catch(function(erro){
                widget.currentRow.DSARQCNABREC = [];
                ScreenService.showMessage(erro.data.error);
            });
    };

    /**
     * Data Source Boletos
     * @param boletos
     */
    self.setDataSourceRetorno = function(boletos){
        var widgetBoletos = templateManager.container.getWidget('ImportacaoBoletosTabbed');
        widgetBoletos.dataSource.data = [];
        widgetBoletos.dataSource.data = boletos;
    };

    /**
     * Data Source Ocorrências
     * @param ocorrencias
     */
    self.setDataSourceOcorrencias = function(ocorrencias){
        var widgetOcorrencias = templateManager.container.getWidget('ImportacaoOcorrenciasTabbed');
        widgetOcorrencias.dataSource.data = [];
        widgetOcorrencias.ocorrencias = ocorrencias;
    };

    /**
     * Abre Ocorrências
     * @param widget
     */
    self.openOcorrencias = function(widget){
        var tabbed = templateManager.container.getWidget('ImportacaoTabbed');
        var widgetOcorrencias = templateManager.container.getWidget('ImportacaoOcorrenciasTabbed');
        widgetOcorrencias.dataSource.data = widgetOcorrencias.ocorrencias.filter(function(ocorrencia){
            return (ocorrencia.CDLINHA == widget.currentRow.CDLINHA || ocorrencia.GERAL);
        });
        tabbed.fields.forEach(function(field){
            field.readOnly = true;
        });
    };

    /**
     * Prepara Ocorrências
     * @param widgetOcorrencias
     * @param nrlanbolrec
     * @param cdlinha
     * @returns {*}
     */
    self.preparaOcorrenciasBoleto = function(widgetOcorrencias, nrlanbolrec, cdlinha){
        var deferred = ZHPromise.defer();
        widgetOcorrencias.dataSource.data = widgetOcorrencias.ocorrencias.filter(function(ocorrencia){
            return (ocorrencia.CDLINHA == cdlinha || ocorrencia.GERAL);
        });
        deferred.resolve();
        return deferred.promise;
    };

    /**
     * Insere Retorno CNAB
     */
    self.insereRetorno = function(){
        var widgetBoletos     = templateManager.container.getWidget('ImportacaoBoletosTabbed');
        var widgetOcorrencias = templateManager.container.getWidget('ImportacaoOcorrenciasTabbed');
        var retorno = templateManager.container.getWidget('RetornoGrid');
        if (widgetBoletos.getParent().isValid()){
            if (widgetBoletos.dataSource.data.length > 0){
                if (self.validaSituacaoCnab(widgetBoletos)){
                    var dataSet = [];
                    var route = 'fin_contaReceber#/insereRetornoCobranca';

                    var form = {
                        'NOMEARQUIVO' : widgetBoletos.getParent().currentRow.DSARQCNABREC[0].name,
                        'CDBANCCNAB'  : widgetBoletos.getParent().currentRow.CDBANCCNAB,
                        'CDBANCO'     : widgetBoletos.getParent().currentRow.CDBANCCNAB,
                        'CDEMPRESA'   : widgetBoletos.getParent().currentRow.CDEMPRESA
                    };

                    dataSet.push(form);
                    dataSet.push(widgetBoletos.dataSource.data);
                    dataSet.push(widgetOcorrencias.ocorrencias);

                    General.callRestEngine(route, 'DataSet', dataSet)
                        .then(function(response){
                            ScreenService.goBack().then(function(){
                                this.deleteDataRetorno();
                                widgetBoletos.getParent().isVisible = false;
                                retorno.activate();
                                retorno.reload();
                            }.bind(this));
                        }.bind(this))
                        .catch(function(erro){
                            ScreenService.showMessage(erro.data.error);
                        });
                }else{
                    ScreenService.showMessage('Registro(s) do arquivo de retorno possui(em) ocorrência(s) que não possibilita(m) o salvamento.');
                }
            }else{
                ScreenService.showMessage('Não existem boletos importados.');
            }
        }
    };

    /**
     * Valida Situação CNAB
     * @param boletos
     * @returns {boolean}
     */
    self.validaSituacaoCnab = function(boletos){
        function arrayValidation(element, index, array) {
            return element.SITUACAOBOLETO == false;
        }
        var boletoValido = !boletos.dataSource.data.every(arrayValidation);
        return (boletos.SITUACAOCNAB && boletoValido) ? true : false;
    };

    /**
     * Cancela Título Retorno
     */
    self.cancelaTituloRetorno = function(){
        var route = 'fin_contaReceber#/cancelaTituloRetornoCob';
        var widgetRetornoGrid = templateManager.container.getWidget('RetornoGrid');
        var widgetBoletosGrid = templateManager.container.getWidget('BoletosGrid');

        if(widgetBoletosGrid.currentRow.IDSITUCNABR == '90') {
            ScreenService.showMessage('Operação Bloqueada. Título baixado via CNAB não pode ser cancelado do retorno.');
            return;
        }

        ScreenService.confirmMessage('Deseja realmente cancelar o título do retorno?', 'question', function () {
            var dataset = {
                retorno : widgetRetornoGrid.currentRow,
                boleto  : widgetBoletosGrid.currentRow
            };

            General.callRestEngine(route, 'DataSet', dataset)
                .then(function(){
                    ScreenService.showMessage('Título cancelado do retorno com sucesso.', 'checked');
                    ScreenService.goBack();
                    widgetBoletosGrid.reload();
                })
                .catch(function(erro){
                    ScreenService.showMessage(erro.data.error);
                });
        });
    };
    
    self.changeTipoConta = function(widget){
        widget.getField('CDTPCTRECEB').dataSourceFilter[0].value = widget.getField('CDEMPRESA').value();
        widget.getField('CDTPCTRECEB').clearValue();
        widget.getField('CDTPCTRECEB').reload();
    };
    
};

Configuration(function (ContextRegister)
{
    ContextRegister.register('Rec00200_RetornoCobranca_Controller', rec00200_RetornoCobranca_Controller);
});