var Bco11100_extratoContaCorrente_Controller = function(ScreenService, General, templateManager, ZHPromise, restEngine, contabilizacaoController, Bco11100_movimentacaoContaCorrente_Service, ZhFilterPreferences){
    var self = this;

    self.openFilter = function(extrato){
        self.iniciaExtratoFiltro(extrato).then(function(filterRow){
            self.abrirFiltro(extrato, filterRow);
        });
    };

    self.abrirFiltro = function(extrato, filterRow){
        ZhFilterPreferences.setOriginalFilter(extrato.id, filterRow);
        ZhFilterPreferences.openFilterWidget(extrato, filterRow);
    };

    self.iniciaExtratoFiltro = function(extrato){
        var deferred = ZHPromise.defer();
        filterRow = {};
        var dtmovbanc = extrato.dataSourceFilter.filter(function(filtro){
            return filtro.name == 'DTMOVBANC';
        });
        if (dtmovbanc.length == 0){
            var start = moment().subtract(1, 'Month').format('DD/MM/YYYY');
            var end   = moment().format('DD/MM/YYYY');
            var periodoInicial = {
                "name": "DTMOVBANC",
                "value": [start, end],
                "operator": "BETWEEN",
                "isCustomFilter": true
            };
            extrato.dataSourceFilter.push(periodoInicial);
            filterRow.DTMOVBANC = {start: start, end: end};
        }
        deferred.resolve(filterRow);
        return deferred.promise;
    };

    self.preparaFiltroExtrato = function(preference){
        var extrato = templateManager.container.getWidget('extrato');
        var dataInicial = null;
        var dataFinal = null;
        if (preference.FILTER_ROW.hasOwnProperty('DTMOVBANC')){
            if (preference.FILTER_ROW.DTMOVBANC.hasOwnProperty('start')){
                dataInicial = preference.FILTER_ROW.DTMOVBANC.start;
                dataFinal = preference.FILTER_ROW.DTMOVBANC.end;
                
            }
        }
        extrato.dataSourceFilter.forEach(function(filtro){
            switch(filtro.name){
                case 'CDREDUCONT': filtro.value = (preference.FILTER_ROW.CDREDUCONT) ? preference.FILTER_ROW.CDREDUCONT : null;
                    break;
                case 'DATAINI': filtro.value = (dataInicial) ? dataInicial : null;
                    break;
                case 'DATAFIN': filtro.value = (dataFinal) ? dataFinal : null;
                    break;
            }
        });
        var dtmovbanc = extrato.dataSourceFilter.filter(function(filtro){
            return filtro.name == 'DTMOVBANC';
        });
        var empresa = extrato.dataSourceFilter.filter(function(filtro){
            return filtro.name == 'CDEMPRESA';
        });
        
        preference.CUSTOM_FILTER_ROW = preference.FILTER_ROW;
        preference.CUSTOM_FILTER_ROW = {};
        delete preference.CUSTOM_FILTER_ROW.CDEMPRESA;
        //delete preference.CUSTOM_FILTER_ROW.DTMOVBANC;
    };

    self.habilitaContaCorrente = function(filtro){
        var extrato = templateManager.container.getWidget('extrato');
        if (filtro.currentRow.NMEMPRESA) {
            filtro.getField('CONTACORRENTE').dataSourceFilter[0].value = filtro.currentRow.CDEMPRESA;
            filtro.getField('CONTACORRENTE').readOnly = false;
        }else {
            filtro.getField('CONTACORRENTE').dataSourceFilter[0].value = null;
            filtro.getField('CONTACORRENTE').readOnly = true;
        }
        filtro.currentRow.CONTACORRENTE = null;
    };

    self.chooseTipoDeMovimentacaoBancaria = function(movimentacaoTabbed){
        var id = movimentacaoTabbed.getParent().currentRow.IDORIGMBANC;
        var movimentacaoWidget = templateManager.container.getWidget("tipoDeMovimentacaoBancaria_"+id);
        this.switchWidgetChild(movimentacaoTabbed, movimentacaoWidget);
        movimentacaoTabbed.widgets.forEach(
            function(widget){
                widget.isVisible = (widget.name === movimentacaoWidget.name);
            }
        );
        this.updateTipoDeMovimentacaoBancariaFilter(movimentacaoWidget);
    };

    self.openDetalhesFromExtrato = function(extratoWidget){
        if(extratoWidget.currentRow.IDMOVBANC != 'X'){
            var movimentacaoTabbed = extratoWidget.widgets[0];
            ScreenService.openSwipe(movimentacaoTabbed).then(
                function(promise){
                    self.chooseTipoDeMovimentacaoBancaria(movimentacaoTabbed);
                }
            );
        }else ScreenService.closeSwipe();
    };

    self.updateTipoDeMovimentacaoBancariaFilter = function(movimentacaoWidget){
        var extratoWidget = movimentacaoWidget.getParent().getParent();
        var codigoReduzido  = extratoWidget.currentRow.CDREDUCONT;
        var numeroMovimentacao = extratoWidget.currentRow.NRMOVBANC;
        codigoReduzido = (codigoReduzido) ? codigoReduzido : null;
        numeroMovimentacao = (numeroMovimentacao) ? numeroMovimentacao : null;
        var paramenterNames = ["CDREDUCONT","NRMOVBANC"];
        var values = [codigoReduzido, numeroMovimentacao];
        movimentacaoWidget.dataSourceFilter = [];
        this.setFilterParameters(movimentacaoWidget, paramenterNames , values);
        movimentacaoWidget.reload();
    };

    self.updateTipoDeMovimentacaoBancariaGridFilter = function(movimentacaoGridWidget){
        var movimentacaoWidget = movimentacaoGridWidget.getParent();
        var id = movimentacaoWidget.getParent().getParent().currentRow.IDORIGMBANC;
        var paramenterNames, values;
        var codigoUnidade;
        switch (id) {
            case 'T':
                var numeroTransferencia  = movimentacaoWidget.currentRow.NRTRANBANC;
                var codigoEmpresa = movimentacaoWidget.currentRow.CDEMPRESA;
                numeroTransferencia = (numeroTransferencia) ? numeroTransferencia : null;
                codigoEmpresa = (codigoEmpresa) ? codigoEmpresa : null;
                paramenterNames = ["NRTRANBANC","CDEMPRESA"];
                values = [numeroTransferencia, codigoEmpresa];
                break;
            case 'P':
                codigoUnidade = movimentacaoWidget.currentRow.CDFILIAL;
                var numeroBaixaPagar  = movimentacaoWidget.currentRow.NRBAIXAPAG;
                codigoUnidade = (codigoUnidade) ? codigoUnidade : null;
                numeroBaixaPagar = (numeroBaixaPagar) ? numeroBaixaPagar : null;
                paramenterNames = ["CDFILIAL","NRBAIXAPAG"];
                values = [codigoUnidade, numeroBaixaPagar];
                break;
            case 'R':
                codigoUnidade = movimentacaoWidget.currentRow.CDFILIAL;
                var numeroBaixaReceber  = movimentacaoWidget.currentRow.NRBAIXAREC;
                codigoUnidade = (codigoUnidade) ? codigoUnidade : null;
                numeroBaixaReceber = (numeroBaixaReceber) ? numeroBaixaReceber : null;
                paramenterNames = ["CDFILIAL","NRBAIXAREC"];
                values = [codigoUnidade, numeroBaixaReceber];
        }
        movimentacaoGridWidget.dataSourceFilter = [movimentacaoGridWidget.dataSourceFilter[0]];
        this.setFilterParameters(movimentacaoGridWidget, paramenterNames , values);
        movimentacaoGridWidget.reload();
        movimentacaoGridWidget.activate();
    };

    self.setFilterParameters = function(object, parameterNames, values){
        var dataSourceFilter = object.dataSourceFilter;
        for (var i = 0; i< parameterNames.length; i++){
            dataSourceFilter = dataSourceFilter.concat(
                {
                    "name": parameterNames[i],
                    "operator": "=",
                    "value": values[i]
                }
            );
        }
        object.dataSourceFilter = dataSourceFilter;
    };

    self.switchWidgetChild = function(widget , widgetChild){
        widget.currentWidget = widgetChild;
        widgetChild.activate();
    };
    
    self.calcSaldo = function(widget){
        var saldo = widget.dataSource.data[0].SALDOINIC;
        
        widget.dataSource.data.forEach(function(data){
            if(data.IDMOVBANC != 'X')
                saldo += data.VRMOVBANCSIMB;
            else saldo = data.VRMOVBANCSIMB;
            data.SALDO = saldo;
        });
    };

};

Configuration(function (ContextRegister){
    ContextRegister.register('Bco11100_extratoContaCorrente_Controller', Bco11100_extratoContaCorrente_Controller);
});