function Pag01600_RelatorioPrazoMedio_Controller(General, ScreenService, templateManager, ZHPromise, ZhFilterPreferences, restEngine) {
    var self = this;
     
    self.openFilterTitulosAPagar = function(widgetTitulosAPagar){
        self.buscaDadosIniciaisFiltro(widgetTitulosAPagar).then(function(){
            var rowSetFilter = {
                'DTATUAVENPAG' : widgetTitulosAPagar.DTATUAVENPAG,
                'CDEMPRESA'   : [widgetTitulosAPagar.empresa.CDEMPRESA]
            };
            ZhFilterPreferences.setOriginalFilter(widgetTitulosAPagar.id, rowSetFilter);
            ZhFilterPreferences.openFilterWidget(widgetTitulosAPagar, rowSetFilter);
        });

    };
    
    self.buscaDadosIniciaisFiltro = function(widgetTitulosAPagar) {
        var deferred = ZHPromise.defer();
        self.buscaPadraoEmpresa(widgetTitulosAPagar).then(function(){
            self.buscaPadraoIntervaloVencimento(widgetTitulosAPagar).then(function(){
                deferred.resolve();        
            });    
        });
        
        return deferred.promise;
    };
    
    self.buscaPadraoEmpresa = function(widget) {
        var deferred = ZHPromise.defer();
        var filterFilial = [{"name": "CDFILIAL", "operator": "=", "value": "FILIAL_LOGADA"}];
        var routeFilial = 'fin#/crud/filial';
            General.callRestEngine(routeFilial, 'FilterData', filterFilial).then(function(response_filial){
            var filial   = response_filial.dataset.filial[0];
            var routeEmpresa = 'fin#/crud/empresa';
            var filterEmpresa = [{"name": "CDEMPRESA", "operator": "=", "value": filial.CDEMPRESA}];
            General.callRestEngine(routeEmpresa, 'FilterData', filterEmpresa).then(function(response_empresa){
                var empresa = response_empresa.dataset.empresa[0];
                widget.empresa = empresa;
                deferred.resolve();
            });
        });
        
        return deferred.promise;
    };
    
    self.buscaGeralPag = function(){
        return General.callRestEngine(
            'fin#/crud/geral_pag',
            'FilterData',
            []
        );
    };
    
    self.buscaParamEstoque = function(cdFilialOpcional) {
        var filter = [{"name": "CDFILIAL", "operator": "=", "value": 'FILIAL_LOGADA'}];
        return General.callRestEngine(
            'fin#/crud/paramestoque_processamento',
            'FilterData',
            filter
        ); 
    };
    
    self.buscaPadraoIntervaloVencimento = function (widget){
        var deferred = ZHPromise.defer();
        var geralpag     = self.buscaGeralPag();
        var paramestoque = self.buscaParamEstoque();

        ZHPromise.all([geralpag, paramestoque]).then(function(response){
            geralpag     = response[0].dataset.geral_pag[0];
            paramestoque = response[1].dataset.paramestoque_processamento[0];

            var qtdiaantlanr = geralpag?geralpag.QTDIAANTBOLP?geralpag.QTDIAANTBOLP:0:0;
            var qtdiaapolanr = geralpag?geralpag.QTDIAAPOBOLP?geralpag.QTDIAAPOBOLP:0:0;
            var dtprocessa   = paramestoque?paramestoque.DTPROCESSA?paramestoque.DTPROCESSA:moment():moment();

            var start = moment(dtprocessa, 'DD/MM/YYYY').subtract(qtdiaantlanr, 'days').format('DD/MM/YYYY');
            var end   = moment(dtprocessa, 'DD/MM/YYYY').add(qtdiaapolanr, 'days').format('DD/MM/YYYY');

            widget.DTATUAVENPAG = {start : start, end : end};
            deferred.resolve();
        });
        return deferred.promise;
    };
    
    
    self.buildFilter = function (widget, preference) {
        var filtro = widget.dataSourceFilter;
        var filterRow = preference.FILTER_ROW;

        filtro.forEach(function(fil, index) {
            switch (fil.name) {
                case 'CDEMPRESA':
                    fil.value = filterRow.CDEMPRESA ? filterRow.CDEMPRESA.join("_") : 'T';
                    break;
                case 'CDFORNECEDOR':
                    if(filterRow.CDFORNECED === undefined || filterRow.CDFORNECED.length === 0){
                            fil.value = 'T';
                        }else{
                            fil.value = filterRow.CDFORNECED ? filterRow.CDFORNECED.join("_"):'';
                        }
                    break;
                case 'IDTPIJURFORN':
                    fil.value = filterRow.IDTPIJURFORN ? null : 'T';
                    break;
                case 'CDGRUPFORN':
                    fil.value = filterRow.CDGRUPFORN ? null : 'T';
                    break;
                    
                /* Unidade de Origem */    
                case 'CDFILIAL':
                    fil.value = filterRow.CDFILIRECE ? filterRow.CDFILIRECE.join("_") : 'T';
                    break;
                    
                /* data de vencimento */    
                case 'DTATUAVENPAGINI':
                    fil.value = filterRow.DTATUAVENPAG ? filterRow.DTATUAVENPAG.start : null;
                    break;
                case 'DTATUAVENPAGFIN':
                    fil.value = filterRow.DTATUAVENPAG ? filterRow.DTATUAVENPAG.end : null;
                    break;
                case 'DTATUAVENPAG':
                    if(filterRow.DTATUAVENPAG){
                        if(filterRow.DTATUAVENPAG.start){
                            fil.value = null;
                        }else {
                            fil.value =  'T';
                        }
                    }
                    else {
                        fil.value = 'T';
                    }
                    break;    
                
                /* data de emissao */     
                case 'DTEMISSAOINI':
                    fil.value = filterRow.DTEMISSAO ? filterRow.DTEMISSAO.start : null;
                    break;
                case 'DTEMISSAOFIN':
                    fil.value = filterRow.DTEMISSAO ? filterRow.DTEMISSAO.end : null;
                    break;
                case 'DTEMISSAO':
                    if(filterRow.DTEMISSAO){
                        if(filterRow.DTEMISSAO.start){
                            fil.value = null;
                        }else {
                            fil.value =  'T';
                        }
                    }
                    else {
                        fil.value = 'T';
                    }
                    break;
                    
                /* data de entrada */ 
                case 'DTENTRSAIDINI':
                    fil.value = filterRow.DTENTRSAID ? filterRow.DTENTRSAID.start : null;
                    break;
                case 'DTENTRSAIDFIN':
                    fil.value = filterRow.DTENTRSAID ? filterRow.DTENTRSAID.end : null;
                    break;
                case 'DTENTRSAID':
                    if(filterRow.DTENTRSAID){
                        if(filterRow.DTENTRSAID.start){
                            fil.value = null;
                        }else {
                            fil.value =  'T';
                        }
                    }
                    else {
                        fil.value = 'T';
                    }
                    break;
                    
                /* data de baixa */     
                case 'DTBAIXAINI':
                    fil.value = filterRow.DTBAIXA ? filterRow.DTBAIXA.start : null;
                    break;
                case 'DTBAIXAFIN':
                    fil.value = filterRow.DTBAIXA ? filterRow.DTBAIXA.end : null;
                    break;
                case 'DTBAIXA':
                    fil.value = filterRow.DTBAIXA ? null : 'T';
                    break;
                    
                case 'IDSTATUS':
                    fil.value = filterRow.IDSTATUS ? null : 'T';
                    break;
                    
                /* status do filtro*/    
                case 'STATUS':
                    var status = '';
                        if (filterRow.IDSTATPAG == 'A'){
                            status = ['A'];
                        } else if (filterRow.IDSTATPAG == 'B'){
                            status = ['B'];
                        } else if (filterRow.IDSTATPAG == 'C'){
                            status = ['C'];
                        }else if (filterRow.IDSTATPAG == 'P'){
                            status = ['P'];
                        }else if (filterRow.IDSTATPAG == 'D'){
                            status = ['D'];
                        }else {
                            status = 'T';
                        }
                        fil.value = status;
    	           break;
                
                /* valor liquido inicial e final*/
                case 'VRATUAPAGINI':
                    fil.value = filterRow.VRATUAPAGINI ? filterRow.VRATUAPAGINI : 0;
                    break;
                case 'VRATUAPAGFIN':
                    fil.value = filterRow.VRATUAPAGFIN ? filterRow.VRATUAPAGFIN : 0;
                    break;
                case 'VRATUAPAG':
                    fil.value = filterRow.VRATUAPAGINI || filterRow.VRATUAPAGFIN ? null : 'T';
                    break;
            }
        });
        
        preference.CUSTOM_FILTER_ROW = {
            'CDFORNECED'   : preference.FILTER_ROW.CDFORNECED,
            'CDFILIRECE'   : preference.FILTER_ROW.CDFILIRECE,
            'CDTPCTPAGAR'  : preference.FILTER_ROW.CDTPCTPAGAR,
            'IDSTATUS'     : preference.FILTER_ROW.IDSTATUS
        };
    };
    
    self.changeInputValues = function(widget, preference) {
        var filterRow = preference.FILTER_ROW;
        validateStartEndValues(filterRow);
    };
    
    /*
     * Verificação para igualar valores iniciais e finais do filtro, no caso
     * do usuário preencher um e deixar o outro em branco. Isso irá fazer
     * com que a query retorne somente resultados que sejam exatamente os  
     * mesmos valores inseridos em um dos campos.
     */
    var validateStartEndValues = function(filterRow) {
        
        // Verifica valor líquido inicial e final.
        filterRow.VRATUAPAGINI = filterRow.VRATUAPAGINI || filterRow.VRATUAPAGFIN || null;
        filterRow.VRATUAPAGFIN = filterRow.VRATUAPAGFIN || filterRow.VRATUAPAGINI;
    };
    
    self.onChangeEmpresaFilter = function(widget){
        var cdempresa = widget.currentRow.CDEMPRESA;
        var cdtpctpagar = widget.getField('CDTPCTPAGAR');
        
        cdtpctpagar.setValue("");
        if(Array.isArray(cdempresa)){
            cdtpctpagar.dataSourceFilter[0].value = "";
            cdempresa.forEach(function(empresa){
                cdtpctpagar.dataSourceFilter[0].value = cdtpctpagar.dataSourceFilter[0].value+empresa+'_';
            });
            cdtpctpagar.dataSourceFilter[0].value = '_'+cdtpctpagar.dataSourceFilter[0].value;
        } else {
            cdtpctpagar.dataSourceFilter[0].value = cdempresa;
        }
        cdtpctpagar.reload();
    };
}

Configuration(function (ContextRegister) {
	ContextRegister.register('Pag01600_RelatorioPrazoMedio_Controller', Pag01600_RelatorioPrazoMedio_Controller);
});